C&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& 
                            SUBROUTINE ADV2 
C     ****************************************************************** 
C$$$  SUBPROGRAM DOCUMENTATION BLOCK 
C                .      .    . 
C SUBPROGRAM:    ADV2        HORIZ/VERT ADVECTION OF VAPOR AND CLOUD 
C   PRGRMMR: JANJIC          ORG: W/NP22     DATE: 96-07-19 
C 
C ABSTRACT: 
C     ADV2 CALCULATES THE CONTRIBUTION OF THE HORIZONTAL AND VERTICAL
C     ADVECTION TO THE TENDENCIES OF WATER SUBSTANCE AND TKE AND THEN 
C     UPDATES THOSE VARIABLES.  AN ANTI-FILTERING TECHNIQUE IS USED. 
C 
C PROGRAM HISTORY LOG: 
C   96-07-19  JANJIC  - ORIGINATOR 
C   96-08-    BLACK   - CRAY DIRECTIVES, CRAY OPTIMIZATION 
C   99-07-    JANJIC  - ADDED HORIZONTAL ADVECTION OF TKE 
C   99-07-    JANJIC  - ADDED VERTICAL ADVECTION 
C   00-01-06  BLACK   - DISTRIBUTED MEMORY AND THREADS
!   01-09-15  JANJIC  - DXDY, HORIZONTAL ADVECTION LOOPING
C 
C USAGE: CALL ADV2 FROM MAIN PROGRAM 
C   INPUT ARGUMENT LIST: 
C       NONE 
C 
C   OUTPUT ARGUMENT LIST 
C       NONE 
C 
C   OUTPUT FILES: 
C       NONE 
C   SUBPROGRAMS CALLED: 
C 
C     UNIQUE: NONE 
C 
C     LIBRARY: NONE 
C 
C   COMMON BLOCKS: CTLBLK 
C                  LOOPS 
C                  MASKS 
C                  DYNAM 
C                  CONTIN 
C                  VRBLS 
C                  PVRBLS 
C                  CLDWTR 
C                  INDX 
C 
C ATTRIBUTES: 
C   LANGUAGE: FORTRAN 90 
C   MACHINE : IBM SP
C$$$ 
C*********************************************************************** 
      INCLUDE "parmeta" 
      INCLUDE "mpp.h"
      INCLUDE "mpif.h"
C----------------------------------------------------------------------- 
                             P A R A M E T E R 
     &(IM1=IM-1,JAM=6+2*(JM-10),JAMD=(JAM*2-10)*3,LP1=LM+1) 
C----------------------------------------------------------------------- 
                             P A R A M E T E R
!test     &(EPSQ=1.E-12,EPSQ2=0.2,CLIMIT=1.E-20
     &(epsq=0.,epsq2=0.,climit=0.
     &,FF1=0.52500,FF2=-0.64813,FF3=0.24520,FF4=-0.12189) 
C----------------------------------------------------------------------- 
                             L O G I C A L
     & RUN,FIRST,RESTRT,SIGMA,TOP,BOT 
C----------------------------------------------------------------------- 
      INCLUDE "CTLBLK.comm" 
C----------------------------------------------------------------------- 
      INCLUDE "LOOPS.comm" 
C----------------------------------------------------------------------- 
      INCLUDE "MASKS.comm" 
C----------------------------------------------------------------------- 
      INCLUDE "DYNAM.comm" 
C----------------------------------------------------------------------- 
      INCLUDE "CONTIN.comm" 
C----------------------------------------------------------------------- 
      INCLUDE "VRBLS.comm" 
C----------------------------------------------------------------------- 
      INCLUDE "PVRBLS.comm" 
C----------------------------------------------------------------------- 
      INCLUDE "CLDWTR.comm" 
C----------------------------------------------------------------------- 
      INCLUDE "INDX.comm" 
C
C-------------LOCAL VARIABLES REQUIRING DECLARATION--------------------- 
C
                             D I M E N S I O N
     & Q3(LM),W3(LM),E3(LM),Q4(LM),W4(LM),E4(LM) 
     &,petdtl(LM),DQL(LM),DWL(LM),DEL(LM),LA(LM),AFR(LM)  
C 
                             D I M E N S I O N
     & IFPA(IDIM1:IDIM2,JDIM1:JDIM2,LM),IFQA(IDIM1:IDIM2,JDIM1:JDIM2,LM)
     &,IFPF(IDIM1:IDIM2,JDIM1:JDIM2,LM),IFQF(IDIM1:IDIM2,JDIM1:JDIM2,LM)
     &,JFPA(IDIM1:IDIM2,JDIM1:JDIM2,LM),JFQA(IDIM1:IDIM2,JDIM1:JDIM2,LM)
     &,JFPF(IDIM1:IDIM2,JDIM1:JDIM2,LM),JFQF(IDIM1:IDIM2,JDIM1:JDIM2,LM)
     &,AFP (IDIM1:IDIM2,JDIM1:JDIM2,LM),AFQ (IDIM1:IDIM2,JDIM1:JDIM2,LM)
     &,Q1  (IDIM1:IDIM2,JDIM1:JDIM2,LM),DQST(IDIM1:IDIM2,JDIM1:JDIM2,LM)
     &,W1  (IDIM1:IDIM2,JDIM1:JDIM2,LM),DWST(IDIM1:IDIM2,JDIM1:JDIM2,LM)
     &,E1  (IDIM1:IDIM2,JDIM1:JDIM2,LM),DEST(IDIM1:IDIM2,JDIM1:JDIM2,LM)
     &,E2  (IDIM1:IDIM2,JDIM1:JDIM2,LM)
     &,DARE(IDIM1:IDIM2,JDIM1:JDIM2),   DVOL(IDIM1:IDIM2,JDIM1:JDIM2,LM)
     &,EMH (IDIM1:IDIM2,JDIM1:JDIM2)
C----------------------------------------------------------------------- 
                             R E A L
     & GSUMS(6,LM),XSUMS(6,LM)
C-----------------------------------------------------------------------
C*********************************************************************** 
      ADDT=FLOAT(IDTAD)*DT 
C----------------------------------------------------------------------- 
C-----------------VERTICAL ADVECTION------------------------------------ 
C----------------------------------------------------------------------- 
!$omp parallel do
!$omp& private (afr,afrp,arr,bot,d2pqe,d2pqq,d2pqw,del,dep,detap,
!$omp&          dpdn,dpup,dql,dqp,dwl,dwp,e00,e3,e4,e4p,ep,ep0,
!$omp&          hbm2ij,la,lap,llap,lmp,petdtl,q00,q3,q4,q4p,qp,qp0,
!$omp&          rface,rfacq,rfacw,rr,sumne,sumnq,sumnw,
!$omp&          sumpe,sumpq,sumpw,top,w00,w3,w4,w4p,wp,wp0)
C----------------------------------------------------------------------- 
      DO 200 J=MYJS2_P3,MYJE2_P3
      DO 200 I=MYIS1_P2,MYIE1_P2
C----------------------------------------------------------------------- 
      LMP=LMH(I,J) 
C----------------------------------------------------------------------- 
      E3(1)=Q2(I,J,1)*0.5 
      DO L=2,LMP 
        E3(L)=MAX((Q2(I,J,L-1)+Q2(I,J,L))*0.5,EPSQ2) 
      ENDDO 
C
      DO L=1,LMP 
        Q3(L)=MAX(Q(I,J,L),EPSQ) 
        W3(L)=MAX(CWM(I,J,L),CLIMIT) 
        E4(L)=E3(L) 
        Q4(L)=Q3(L) 
        W4(L)=W3(L) 
      ENDDO 
C 
     
C
      DO L=2,LMP-1 
        petdtl(L)=(petdt(I,J,L-1)+petdt(I,J,L))*0.5 
      ENDDO 
C
      petdtl(LMP)=petdt(I,J,LMP-1)*0.5 
C----------------------------------------------------------------------- 
      DO 25 L=1,LMP
      RR=petdtl(L)*(-ADDT)
!zj      ARR=ABS(RR)
C
!zj      IF(ARR.GT.0.)THEN
!zj        LAP=RR/ARR
!zj      ELSE
!zj        LAP=0
!zj      ENDIF
!
          if(rr.gt.0.) then
      lap=1
          else
              if(rr.lt.0.) then
          lap=-1
              else
          lap=0
              endif
          endif
C
      LA(L)=LAP
      LLAP=L+LAP
C
      TOP=.FALSE.
      BOT=.FALSE.
C
      IF(LLAP.GT.0.AND.LLAP.LT.LMH(I,J)+1.AND.LAP.NE.0)THEN
!zj        rr=arr/abs((aeta1(llap)-aeta1(l))*pdtop
!zj     &            +(aeta2(llap)-aeta2(l))*pdsl(i,j))
!
        rr=rr/((aeta1(llap)-aeta1(l))*pdtop
     &        +(aeta2(llap)-aeta2(l))*pdsl(i,j))
        if(rr.lt.0.) rr=-rr
!
        AFR(L)=(((FF4*RR+FF3)*RR+FF2)*RR+FF1)*RR 
        DQP=(Q3(LLAP)-Q3(L))*RR 
        DWP=(W3(LLAP)-W3(L))*RR 
        DEP=(E3(LLAP)-E3(L))*RR 
        DQL(L)=DQP 
        DWL(L)=DWP            
        DEL(L)=DEP 
      ELSE 
        TOP=LLAP.EQ.0 
        BOT=LLAP.EQ.LMH(I,J)+1 
C 
        RR=0. 
        AFR(L)=0. 
        DQL(L)=0. 
        DWL(L)=0.            
        DEL(L)=0. 
      ENDIF 
   25 CONTINUE
C 
      IF(TOP)THEN 
        IF(LA(2).LT.0)THEN 
          rfc=(deta1(2)*pdtop+deta2(2)*pdsl(i,j))
     &       /(deta1(1)*pdtop+deta2(1)*pdsl(i,j))
          dql(1)=-dql(2)*rfc
          dwl(1)=-dwl(2)*rfc
          del(1)=-del(2)*rfc 
        ENDIF 
      ENDIF 
C
      IF(BOT)THEN 
        IF(LA(LMP-1).GT.0)THEN 
          rfc=(deta1(lmp-1)*pdtop+deta2(lmp-1)*pdsl(i,j))
     &       /(deta1(lmp)*pdtop+deta2(lmp)*pdsl(i,j))
          dql(lmp)=-dql(lmp-1)*rfc
          dwl(lmp)=-dwl(lmp-1)*rfc
          del(lmp)=-del(lmp-1)*rfc
        ENDIF 
      ENDIF 
C
        DO L=1,LMP
          q4(l)=q3(l)+dql(l)
          w4(l)=w3(l)+dwl(l)
          e4(l)=e3(l)+del(l)
        ENDDO
C--------------ANTI-FILTERING STEP-------------------------------------- 
      SUMPQ=0. 
      SUMNQ=0. 
      SUMPW=0. 
      SUMNW=0. 
      SUMPE=0. 
      SUMNE=0. 
C--------------ANTI-FILTERING LIMITERS---------------------------------- 
      DO 50 L=2,LMP-1 
      detap=deta1(l)*pdtop+deta2(l)*pdsl(i,j) 
C 
      Q4P=Q4(L) 
      W4P=W4(L) 
      E4P=E4(L) 
C 
      LAP=LA(L) 
C 
      IF(LAP.NE.0)THEN 
        dpdn=(aeta1(l+lap)-aeta1(l))*pdtop
     &      +(aeta2(l+lap)-aeta2(l))*pdsl(i,j)
        dpup=(aeta1(l)-aeta1(l-lap))*pdtop
     &      +(aeta2(l)-aeta2(l-lap))*pdsl(i,j)
!
        afrp=2.*afr(l)*dpdn*dpdn/(dpdn+dpup)
        d2pqq=((q4(l+lap)-q4p)/dpdn 
     1        -(q4p-q4(l-lap))/dpup)*afrp 
        d2pqw=((w4(l+lap)-w4p)/dpdn 
     1        -(w4p-w4(l-lap))/dpup)*afrp 
        d2pqe=((e4(l+lap)-e4p)/dpdn 
     1        -(e4p-e4(l-lap))/dpup)*afrp 
      ELSE 
        D2PQQ=0. 
        D2PQW=0. 
        D2PQE=0. 
      ENDIF 
C 
      QP=Q4P-D2PQQ 
      WP=W4P-D2PQW 
      EP=E4P-D2PQE 
C 
      Q00=Q3(L) 
      QP0=Q3(L+LAP) 
C 
      W00=W3(L) 
      WP0=W3(L+LAP) 
C 
      E00=E3(L) 
      EP0=E3(L+LAP) 
C 
      IF(LAP.NE.0)THEN 
        QP=MAX(QP,MIN(Q00,QP0)) 
        QP=MIN(QP,MAX(Q00,QP0)) 
        WP=MAX(WP,MIN(W00,WP0)) 
        WP=MIN(WP,MAX(W00,WP0)) 
        EP=MAX(EP,MIN(E00,EP0)) 
        EP=MIN(EP,MAX(E00,EP0)) 
      ENDIF 
C 
      dqp=qp-q00 
      dwp=wp-w00 
      dep=ep-e00 
C 
      DQL(L)=DQP 
      DWL(L)=DWP 
      DEL(L)=DEP 
C 
      dqp=dqp*detap 
      dwp=dwp*detap 
      dep=dep*detap 
C 
      IF(DQP.GT.0.)THEN 
        SUMPQ=SUMPQ+DQP 
      ELSE 
        SUMNQ=SUMNQ+DQP 
      ENDIF 
C 
      IF(DWP.GT.0.)THEN 
        SUMPW=SUMPW+DWP 
      ELSE 
        SUMNW=SUMNW+DWP 
      ENDIF 
C 
      IF(DEP.GT.0.)THEN 
        SUMPE=SUMPE+DEP 
      ELSE 
        SUMNE=SUMNE+DEP 
      ENDIF 
C 
   50 CONTINUE
C----------------------------------------------------------------------- 
      DQL(1)=0. 
      DWL(1)=0. 
      DEL(1)=0. 
C 
      DQL(LMP)=0. 
      DWL(LMP)=0. 
      DEL(LMP)=0. 
C--------------FIRST MOMENT CONSERVING FACTOR--------------------------- 
      IF(SUMPQ.GT.1.E-9)THEN 
        RFACQ=-SUMNQ/SUMPQ 
      ELSE 
        RFACQ=1. 
      ENDIF 
C
      IF(SUMPW.GT.1.E-9)THEN 
        RFACW=-SUMNW/SUMPW 
      ELSE 
        RFACW=1. 
      ENDIF 
C
      IF(SUMPE.GT.1.E-9)THEN 
        RFACE=-SUMNE/SUMPE 
      ELSE 
        RFACE=1. 
      ENDIF 
C
      IF(RFACQ.LT.0.9.OR.RFACQ.GT.1.1) RFACQ=1. 
      IF(RFACW.LT.0.9.OR.RFACW.GT.1.1) RFACW=1. 
      IF(RFACE.LT.0.9.OR.RFACE.GT.1.1) RFACE=1. 
C
C--------------IMPOSE CONSERVATION ON ANTI-FILTERING-------------------- 
C
        DO L=1,LMP 
          DQP=DQL(L) 
          IF(DQP.GE.0.) DQP=DQP*RFACQ 
          q(i,j,l)=q3(l)+dqp 
        ENDDO 
C----------------------------------------------------------------------- 
        DO L=1,LMP 
          DWP=DWL(L) 
          IF(DWP.GE.0.) DWP=DWP*RFACW 
          cwm(i,j,l)=w3(l)+dwp 
        ENDDO 
C----------------------------------------------------------------------- 
        DO L=1,LMP 
          DEP=DEL(L) 
          IF(DEP.GE.0.) DEP=DEP*RFACE 
          e3(l)=e3(l)+dep 
        ENDDO 
C
      HBM2IJ=HBM2(I,J)
      Q2(I,J,1)=MAX(E3(1)+E3(1)-EPSQ2,EPSQ2)*HBM2IJ 
     1         +Q2(I,J,1)*(1.-HBM2IJ)
      DO L=2,LMP-1 
      Q2(I,J,L)=MAX(E3(L)+E3(L)-Q2(I,J,L-1),EPSQ2)*HBM2IJ
     1         +Q2(I,J,L)*(1.-HBM2IJ)
      ENDDO 
C----------------------------------------------------------------------- 
  200 CONTINUE
C--------------END OF VERTICAL ADVECTION-------------------------------- 
C*********************************************************************** 
      ENH=ADDT/(08.*DY) 
C----------------------------------------------------------------------- 
!$omp parallel do
      DO J=MYJS_P2,MYJE_P2
      DO I=MYIS_P1,MYIE_P1
        EMH (I,J)=ADDT/(08.*DX(I,J)) 
        DARE(I,J)=HBM2(I,J)*DX(I,J)*DY 
        E1(I,J,1)=MAX(Q2(I,J,1)*0.5,EPSQ2)
        E2(I,J,1)=E1(I,J,1)
      ENDDO
      ENDDO
C
C----------------------------------------------------------------------- 
C--------------HORIZONTAL ADVECTION------------------------------------- 
C----------------------------------------------------------------------- 
C
C*********************************************************************** 
!$omp parallel do
!$omp& private (destij,dqstij,dvolp,dwstij,hbm2ij,hm,
!$omp&          jfp,jfq,pp,qp,sumne,sumnq,sumnw,sumpe,sumpq,sumpw,
!$omp&          tta,ttb)
      DO 300 L=1,LM
C*********************************************************************** 
      DO J=MYJS_P2,MYJE_P2
      DO I=MYIS_P1,MYIE_P1
        dvol(i,j,l)=dare(i,j)*(deta1(l)*pdtop+deta2(l)*pdsl(i,j)) 
        HM=HTM(I,J,L) 
        Q  (I,J,L)=MAX(Q  (I,J,L),EPSQ)*HM 
        CWM(I,J,L)=MAX(CWM(I,J,L),CLIMIT)*HM 
        Q1 (I,J,L)=Q  (I,J,L) 
        W1 (I,J,L)=CWM(I,J,L) 
      ENDDO 
      ENDDO 
C
      IF(L.GT.1)THEN 
        DO J=MYJS_P2,MYJE_P2
        DO I=MYIS_P1,MYIE_P1
          E1(I,J,L)=MAX((Q2(I,J,L-1)+Q2(I,J,L))*0.5,EPSQ2) 
          E2(I,J,L)=E1(I,J,L) 
        ENDDO 
        ENDDO 
      ENDIF 
C----------------------------------------------------------------------- 
      DO 225 J=MYJS2_P1,MYJE2_P1
      DO 225 I=MYIS1_P1,MYIE1_P1
C
      HBM2IJ=HBM2(I,J)
      TTA=(U(I,J-1,L)+U(I+IHW(J),J,L)+U(I+IHE(J),J,L)+U(I,J+1,L)) 
     2   *HBM2IJ*EMH(I,J) 
      TTB=(V(I,J-1,L)+V(I+IHW(J),J,L)+V(I+IHE(J),J,L)+V(I,J+1,L)) 
     2   *HBM2IJ*ENH 
C 
      spp=-tta-ttb 
      sqp= tta-ttb 
C 
      pp=abs(spp)
      qp=abs(sqp)
!
          if(pp.gt.0.) then
      jfp=spp/pp
          else
      jfp=0
          endif
          if(qp.gt.0.) then 
      jfq=sqp/qp
          else
      jfq=0
          endif 
! 
      IFPA(I,J,L)=IHE(J)+I+( JFP-1  )/2 
      IFQA(I,J,L)=IHE(J)+I+(-JFQ-1  )/2 
C 
      JFPA(I,J,L)=J+JFP 
      JFQA(I,J,L)=J+JFQ 
C 
      IFPF(I,J,L)=IHE(J)+I+(-JFP-1)/2 
      IFQF(I,J,L)=IHE(J)+I+( JFQ-1)/2 
C 
      JFPF(I,J,L)=J-JFP 
      JFQF(I,J,L)=J-JFQ 
C 
      spp=spp*htm(i,j,l)*htm(ifpa(i,j,l),jfpa(i,j,l),l) 
      sqp=sqp*htm(i,j,l)*htm(ifqa(i,j,l),jfqa(i,j,l),l) 
!
      fpq=spp*sqp*hbm2ij*htm(i,j-2,l)*htm(i-1,j,l)
     &                  *htm(i+1,j,l)*htm(i,j+2,l)*0.25
     &                  *hbm3(i,j)
!
      pp=abs(spp)
      qp=abs(sqp)
C 
      AFP (I,J,L)=(((FF4*PP+FF3)*PP+FF2)*PP+FF1)*PP 
      AFQ (I,J,L)=(((FF4*QP+FF3)*QP+FF2)*QP+FF1)*QP 
!
      q1(i,j,l)=(q  (ifpa(i,j,l),jfpa(i,j,l),l)-q  (i,j,l))*pp 
     &         +(q  (ifqa(i,j,l),jfqa(i,j,l),l)-q  (i,j,l))*qp 
     &         +(q  (i,j-2,l)+q  (i,j+2,l)-q  (i-1,j,l)-q  (i+1,j,l))
     &         *fpq
     &         +q(i,j,l)
!
      w1(i,j,l)=(cwm(ifpa(i,j,l),jfpa(i,j,l),l)-cwm(i,j,l))*pp 
     &         +(cwm(ifqa(i,j,l),jfqa(i,j,l),l)-cwm(i,j,l))*qp 
     &         +(cwm(i,j-2,l)+cwm(i,j+2,l)-cwm(i-1,j,l)-cwm(i+1,j,l))
     &         *fpq
     &         +cwm(i,j,l)
!
      e2(i,j,l)=(e1 (ifpa(i,j,l),jfpa(i,j,l),l)-e1 (i,j,l))*pp 
     &         +(e1 (ifqa(i,j,l),jfqa(i,j,l),l)-e1 (i,j,l))*qp 
     &         +(e1 (i,j-2,l)+e1 (i,j+2,l)-e1 (i-1,j,l)-e1 (i+1,j,l))
     &         *fpq
     &         +e1(i,j,l)
!
  225 continue
!-----------------------------------------------------------------------
  300 CONTINUE          ! End of Vertical Loop
C--------------ANTI-FILTERING STEP-------------------------------------- 
      do 400 l=1,lm
!-----------------------------------------------------------------------
      SUMPQ=0. 
      SUMNQ=0. 
      SUMPW=0. 
      SUMNW=0. 
      SUMPE=0. 
      SUMNE=0. 
C--------------ANTI-FILTERING LIMITERS---------------------------------- 
      DO 350 J=MYJS2,MYJE2
      DO 350 I=MYIS1,MYIE1
C
      DVOLP=DVOL(I,J,L)
      Q1IJ =Q1(I,J,L)
      W1IJ =W1(I,J,L)
      E2IJ =E2(I,J,L)
C
      hafp=htm(ifpf(i,j,l),jfpf(i,j,l),l)*afp(i,j,l)
      hafq=htm(ifqf(i,j,l),jfqf(i,j,l),l)*afq(i,j,l)
!
      d2pqq=(q1(ifpa(i,j,l),jfpa(i,j,l),l)-q1ij
     2      -q1ij+q1(ifpf(i,j,l),jfpf(i,j,l),l))
     3     *hafp
     4     +(q1(ifqa(i,j,l),jfqa(i,j,l),l)-q1ij
     5      -q1ij+q1(ifqf(i,j,l),jfqf(i,j,l),l))
     6     *hafq
C 
      d2pqw=(w1(ifpa(i,j,l),jfpa(i,j,l),l)-w1ij
     2      -w1ij+w1(ifpf(i,j,l),jfpf(i,j,l),l))
     3     *hafp
     4     +(w1(ifqa(i,j,l),jfqa(i,j,l),l)-w1ij
     5      -w1ij+w1(ifqf(i,j,l),jfqf(i,j,l),l))
     6     *hafq
C 
      d2pqe=(e2(ifpa(i,j,l),jfpa(i,j,l),l)-e2ij
     2      -e2ij+e2(ifpf(i,j,l),jfpf(i,j,l),l))
     3      *hafp
     4     +(e2(ifqa(i,j,l),jfqa(i,j,l),l)-e2ij
     5      -e2ij+e2(ifqf(i,j,l),jfqf(i,j,l),l))
     6      *hafq
C 
      QSTIJ=Q1IJ-D2PQQ 
      WSTIJ=W1IJ-D2PQW 
      ESTIJ=E2IJ-D2PQE 
C 
      Q00=Q  (I          ,J          ,L) 
      QP0=Q  (IFPA(I,J,L),JFPA(I,J,L),L) 
      Q0Q=Q  (IFQA(I,J,L),JFQA(I,J,L),L) 
C 
      W00=CWM(I          ,J          ,L) 
      WP0=CWM(IFPA(I,J,L),JFPA(I,J,L),L) 
      W0Q=CWM(IFQA(I,J,L),JFQA(I,J,L),L) 
C 
      E00=E1 (I          ,J          ,L) 
      EP0=E1 (IFPA(I,J,L),JFPA(I,J,L),L) 
      E0Q=E1 (IFQA(I,J,L),JFQA(I,J,L),L) 
C 
      QSTIJ=AMAX1(QSTIJ,AMIN1(Q00,QP0,Q0Q)) 
      QSTIJ=AMIN1(QSTIJ,AMAX1(Q00,QP0,Q0Q)) 
      WSTIJ=AMAX1(WSTIJ,AMIN1(W00,WP0,W0Q)) 
      WSTIJ=AMIN1(WSTIJ,AMAX1(W00,WP0,W0Q)) 
      ESTIJ=AMAX1(ESTIJ,AMIN1(E00,EP0,E0Q)) 
      ESTIJ=AMIN1(ESTIJ,AMAX1(E00,EP0,E0Q)) 
C 
!      dqstij=qstij-q(i,j,l) 
!      dwstij=wstij-cwm(i,j,l) 
!      destij=estij-e1(i,j,l)
      dqstij=0.
      dwstij=0.
      destij=0.
C 
      DQST(I,J,L)=DQSTIJ 
      DWST(I,J,L)=DWSTIJ 
      DEST(I,J,L)=DESTIJ 
C 
      DQSTIJ=DQSTIJ*DVOLP 
      DWSTIJ=DWSTIJ*DVOLP 
      DESTIJ=DESTIJ*DVOLP 
C 
      IF(DQSTIJ.GT.0.)THEN
        SUMPQ=SUMPQ+DQSTIJ
      ELSE
        SUMNQ=SUMNQ+DQSTIJ
      ENDIF
C
      IF(DWSTIJ.GT.0.)THEN
        SUMPW=SUMPW+DWSTIJ
      ELSE
        SUMNW=SUMNW+DWSTIJ
      ENDIF
C
      IF(DESTIJ.GT.0.)THEN
        SUMPE=SUMPE+DESTIJ
      ELSE
        SUMNE=SUMNE+DESTIJ
      ENDIF
C
  350 CONTINUE
C-----------------------------------------------------------------------
      XSUMS(1,L)=SUMPQ
      XSUMS(2,L)=SUMNQ
      XSUMS(3,L)=SUMPW
      XSUMS(4,L)=SUMNW
      XSUMS(5,L)=SUMPE
      XSUMS(6,L)=SUMNE
C
  400 CONTINUE            ! End of Vertical Loop
C-----------------------------------------------------------------------
C
C***  GLOBAL REDUCTION
C
!zj      CALL MPI_ALLREDUCE(XSUMS,GSUMS,6*LM,MPI_REAL,MPI_SUM,
!zj     1                     MPI_COMM_WORLD,IRECV)
!zj      IF(IRECV.NE.0)THEN
!zj         PRINT*, ' RETURN CODE FROM 2nd ALLREDUCE IN ADV2 = ',IRECV
!zj         STOP
!zj      ENDIF
C
C***  END OF GLOBAL REDUCTION
C
c     if(mype.eq.0)then
c       do l=1,lm
c         read(93)gsums(1,l)
c         read(93)gsums(2,l)
c         read(93)gsums(3,l)
c         read(93)gsums(4,l)
c         read(93)gsums(5,l)
c         read(93)gsums(6,l)
c       enddo
c     endif
c     call mpi_bcast(gsums,6*lm,mpi_real,0,MPI_COMM_WORLD,irtn)
C-----------------------------------------------------------------------
      gsums(1,l)=sumpq
      gsums(2,l)=sumnq
      gsums(3,l)=sumpw
      gsums(4,l)=sumnw
      gsums(5,l)=sumpe
      gsums(6,l)=sumne
C-----------------------------------------------------------------------
      DO 425 L=1,LM
C
      SUMPQ=GSUMS(1,L)
      SUMNQ=GSUMS(2,L)
      SUMPW=GSUMS(3,L)
      SUMNW=GSUMS(4,L)
      SUMPE=GSUMS(5,L)
      SUMNE=GSUMS(6,L)
C
C--------------FIRST MOMENT CONSERVING FACTOR--------------------------- 
C
      IF(SUMPQ.GT.1.)THEN
        RFACQ=-SUMNQ/SUMPQ
      ELSE
        RFACQ=1.
      ENDIF
C
      IF(SUMPW.GT.1.)THEN
        RFACW=-SUMNW/SUMPW
      ELSE
        RFACW=1.
      ENDIF
C
      IF(SUMPE.GT.1.)THEN
        RFACE=-SUMNE/SUMPE
      ELSE
        RFACE=1.
      ENDIF
C
      IF(RFACQ.LT.0.9.OR.RFACQ.GT.1.1)RFACQ=1. 
      IF(RFACW.LT.0.9.OR.RFACW.GT.1.1)RFACW=1. 
      IF(RFACE.LT.0.9.OR.RFACE.GT.1.1)RFACE=1. 



!zj test
      rfacq=1.
      rfacw=1.
      rface=1.




C
C--------------IMPOSE CONSERVATION ON ANTI-FILTERING-------------------- 
C
      IF(RFACQ.LT.1.)THEN
!$omp parallel do private (dqstij,rfqij)
        DO J=MYJS2,MYJE2
        DO I=MYIS1,MYIE1
          DQSTIJ=DQST(I,J,L)
          RFQIJ=HBM2(I,J)*(RFACQ-1.)+1.
          IF(DQSTIJ.GE.0.)DQSTIJ=DQSTIJ*RFQIJ
          q(i,j,l)=q(i,j,l)+dqstij 
        ENDDO
        ENDDO
      ELSE
!$omp parallel do private (dqstij,rfqij)
        DO J=MYJS2,MYJE2
        DO I=MYIS1,MYIE1
          DQSTIJ=DQST(I,J,L)
          RFQIJ=HBM2(I,J)*(RFACQ-1.)+1.
          IF(DQSTIJ.LT.0.)DQSTIJ=DQSTIJ/RFQIJ
          q(i,j,l)=q(i,j,l)+dqstij 
        ENDDO
        ENDDO
      ENDIF
C----------------------------------------------------------------------- 
      IF(RFACW.LT.1.)THEN
!$omp parallel do private (dwstij,rfwij)
        DO J=MYJS2,MYJE2
        DO I=MYIS1,MYIE1
          DWSTIJ=DWST(I,J,L)
          RFWIJ=HBM2(I,J)*(RFACW-1.)+1.
          IF(DWSTIJ.GE.0.)DWSTIJ=DWSTIJ*RFWIJ
          cwm(i,j,l)=cwm(i,j,l)+dwstij
        ENDDO
        ENDDO
      ELSE
!$omp parallel do private (dwstij,rfwij)
        DO J=MYJS2,MYJE2
        DO I=MYIS1,MYIE1
          DWSTIJ=DWST(I,J,L)
          RFWIJ=HBM2(I,J)*(RFACW-1.)+1.
          IF(DWSTIJ.LT.0.)DWSTIJ=DWSTIJ/RFWIJ
          cwm(i,j,l)=cwm(i,j,l)+dwstij
        ENDDO
        ENDDO
      ENDIF
C----------------------------------------------------------------------- 
      IF(RFACE.LT.1.)THEN
!$omp parallel do private (destij,rfeij)
        DO J=MYJS2,MYJE2
        DO I=MYIS1,MYIE1
          DESTIJ=DEST(I,J,L)
          RFEIJ=HBM2(I,J)*(RFACE-1.)+1.
          IF(DESTIJ.GE.0.)DESTIJ=DESTIJ*RFEIJ
          e1(i,j,l)=e1(i,j,l)+destij
        ENDDO
        ENDDO
      ELSE
!$omp parallel do private (destij,rfeij)
        DO J=MYJS2,MYJE2
        DO I=MYIS1,MYIE1
          DESTIJ=DEST(I,J,L)
          RFEIJ=HBM2(I,J)*(RFACE-1.)+1.
          IF(DESTIJ.LT.0.)DESTIJ=DESTIJ/RFEIJ
          e1(i,j,l)=e1(i,j,l)+destij
        ENDDO
        ENDDO
      ENDIF
C----------------------------------------------------------------------- 
!$omp parallel do private (hm)
      DO J=MYJS,MYJE
      DO I=MYIS,MYIE
        HM=HTM(I,J,L) 
        Q  (I,J,L)=AMAX1(Q  (I,J,L),EPSQ)*HM 
        CWM(I,J,L)=AMAX1(CWM(I,J,L),CLIMIT)*HM 
      ENDDO 
      ENDDO 
C***
      IF(L.EQ.LM)GO TO 425
C***
      IF(L.EQ.1)THEN 
!$omp parallel do 
        DO J=MYJS,MYJE
        DO I=MYIS,MYIE
          Q2(I,J,L)=MAX(E1(I,J,L)+E1(I,J,L)-EPSQ2,EPSQ2)*HTM(I,J,1) 
        ENDDO 
        ENDDO 
      ELSE 
!$omp parallel do 
        DO J=MYJS,MYJE
        DO I=MYIS,MYIE
          IF(L.LT.LMH(I,J))THEN 
            Q2(I,J,L)=MAX(E1(I,J,L)+E1(I,J,L)-Q2(I,J,L-1),EPSQ2)
     1                   *HTM(I,J,L) 
          ELSE
            Q2(I,J,L)=Q2(I,J,L-1)
          ENDIF 
        ENDDO 
        ENDDO 
      ENDIF 
C*********************************************************************** 
  425 CONTINUE
C----------------------------------------------------------------------- 
                             RETURN 
                             END 
