!&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
                             SUBROUTINE ADVE
C     ******************************************************************
C$$$  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .
C SUBPROGRAM:    ADVE        HORIZONTAL AND VERTICAL ADVECTION
C   PRGRMMR: JANJIC          ORG: W/NP22     DATE: 93-10-28
C
C ABSTRACT:
C     ADVE CALCULATES THE CONTRIBUTION OF THE HORIZONTAL AND VERTICAL
C     ADVECTION TO THE TENDENCIES OF TEMPERATURE AND WIND AND THEN
C     UPDATES THOSE VARIABLES.
C     THE JANJIC ADVECTION SCHEME FOR THE ARAKAWA E GRID IS USED
C     FOR ALL VARIABLES INSIDE THE FIFTH ROW.  AN UPSTREAM SCHEME
C     IS USED ON ALL VARIABLES IN THE THIRD, FOURTH, AND FIFTH
C     OUTERMOST ROWS.  THE ADAMS-BASHFORTH TIME SCHEME IS USED.
C
C PROGRAM HISTORY LOG:
C   85-07-??  JANJIC     - ORIGINATOR
C   95-03-25  BLACK      - CONVERSION FROM 1-D TO 2-D IN HORIZONTAL
C   96-03-28  BLACK      - ADDED EXTERNAL EDGE
C   99-07-    JANJIC     - CONVERTED TO ADAMS-BASHFORTH SCHEME
C                          FROM SUBROUTINE HZADV
C   99-07-    JANJIC     - ADDED VERTICAL ADVECTION
C   00-01-04  BLACK      - DISTRIBUTED MEMORY AND THREADS
!   02-01-31  black      - cfl check
!   02-02-09  jajic      - weighting of tendencies in cfl check
C
C USAGE: CALL ADVE FROM MAIN PROGRAM
C   INPUT ARGUMENT LIST:
C       NONE
C
C   OUTPUT ARGUMENT LIST:
C     NONE
C
C   OUTPUT FILES:
C     none
C
C   SUBPROGRAMS CALLED:
C
C     UNIQUE: NONE
C
C     LIBRARY: NONE
C
C   COMMON BLOCKS: CTLBLK
C                  LOOPS
C                  MASKS
C                  DYNAM
C                  VRBLS
C                  CONTIN
C                  PVRBLS
C                  INDX
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN 90
C   MACHINE : IBM SP
C$$$
C***********************************************************************
                             P A R A M E T E R
     &(FFC=1.533,FBC=1.-FFC)
                             P A R A M E T E R
     &(CFL_MAX=.97)!0.66)!97) !zjtest

C-----------------------------------------------------------------------
      INCLUDE "parmeta"
      INCLUDE "mpp.h"
C-----------------------------------------------------------------------
                             P A R A M E T E R
     &(JAM=6+2*(JM-10),JAMD=(JAM*2-10)*3
     &,LM1=LM-1,LP1=LM+1)
C-----------------------------------------------------------------------
                             L O G I C A L
     & RUN,FIRST,RESTRT,SIGMA
C-----------------------------------------------------------------------
      INCLUDE "CTLBLK.comm"
C-----------------------------------------------------------------------
      INCLUDE "LOOPS.comm"
C-----------------------------------------------------------------------
      INCLUDE "MASKS.comm"
C-----------------------------------------------------------------------
      INCLUDE "DYNAM.comm"
C-----------------------------------------------------------------------
      INCLUDE "VRBLS.comm"
C-----------------------------------------------------------------------
      INCLUDE "CONTIN.comm"
C-----------------------------------------------------------------------
      INCLUDE "PVRBLS.comm"
C-----------------------------------------------------------------------
      INCLUDE "INDX.comm"
C-----------------------------------------------------------------------
                             R E A L
     & HM    (IDIM1:IDIM2,JDIM1:JDIM2),VM    (IDIM1:IDIM2,JDIM1:JDIM2)
     &,RDPD  (IDIM1:IDIM2,JDIM1:JDIM2)
     &,RDPDX (IDIM1:IDIM2,JDIM1:JDIM2),RDPDY (IDIM1:IDIM2,JDIM1:JDIM2)
     &,ADPDX (IDIM1:IDIM2,JDIM1:JDIM2),ADPDY (IDIM1:IDIM2,JDIM1:JDIM2)
     &,uned  (idim1:idim2,jdim1:jdim2),used  (idim1:idim2,jdim1:jdim2)
     &,ADT   (IDIM1:IDIM2,JDIM1:JDIM2)
     &,ADU   (IDIM1:IDIM2,JDIM1:JDIM2),ADV   (IDIM1:IDIM2,JDIM1:JDIM2)
     &,TDN   (IDIM1:IDIM2,JDIM1:JDIM2)
     &,UDN   (IDIM1:IDIM2,JDIM1:JDIM2),VDN   (IDIM1:IDIM2,JDIM1:JDIM2)
     &,petdtL(IDIM1:IDIM2,JDIM1:JDIM2)
     &,TTB   (IDIM1:IDIM2,JDIM1:JDIM2)
     &,TUB   (IDIM1:IDIM2,JDIM1:JDIM2),TVB   (IDIM1:IDIM2,JDIM1:JDIM2)

                             R E A L
     & DPDE  (IDIM1:IDIM2,JDIM1:JDIM2)
     &,TST   (IDIM1:IDIM2,JDIM1:JDIM2)
     &,UST   (IDIM1:IDIM2,JDIM1:JDIM2),VST   (IDIM1:IDIM2,JDIM1:JDIM2)
     &,TEW   (IDIM1:IDIM2,JDIM1:JDIM2),TNS   (IDIM1:IDIM2,JDIM1:JDIM2)
C
                             R E A L
     & TNE   (IDIM1:IDIM2,JDIM1:JDIM2),TSE   (IDIM1:IDIM2,JDIM1:JDIM2)
     &,UEW   (IDIM1:IDIM2,JDIM1:JDIM2),UNS   (IDIM1:IDIM2,JDIM1:JDIM2)
     &,VEW   (IDIM1:IDIM2,JDIM1:JDIM2),VNS   (IDIM1:IDIM2,JDIM1:JDIM2)
     &,UNE   (IDIM1:IDIM2,JDIM1:JDIM2),USE   (IDIM1:IDIM2,JDIM1:JDIM2)
     &,VNE   (IDIM1:IDIM2,JDIM1:JDIM2),VSE   (IDIM1:IDIM2,JDIM1:JDIM2)
C
      REAL ARRAY0(JAMD)
      REAL ARRAY1(JAMD)
      REAL ARRAY2(JAMD)
      REAL ARRAY3(JAMD)
      INTEGER KHHAS(JAMD)
      INTEGER IHLAS(JAMD)
      INTEGER JHLAS(JAMD)
      INTEGER KVHAS(JAMD)
      INTEGER IVLAS(JAMD)
      INTEGER JVLAS(JAMD)
      INTEGER ISPA(JAMD)
      INTEGER ISQA(JAMD)
C
      LOGICAL UPSTRM,LJRA(JAM)
!
!***  VERTICAL CFL WORKING ARRAYS
!
                             R E A L
     & VAD_TEND_T(IDIM1:IDIM2,JDIM1:JDIM2,LM)
     &,VAD_TEND_U(IDIM1:IDIM2,JDIM1:JDIM2,LM)
     &,VAD_TEND_V(IDIM1:IDIM2,JDIM1:JDIM2,LM)
     &,VAD_TNDX_T(LM),VAD_TNDX_U(LM),VAD_TNDX_V(LM)
     &,DP_HI(IDIM1:IDIM2,JDIM1:JDIM2)
     &,DP_LO(IDIM1:IDIM2,JDIM1:JDIM2)
!
                             I N T E G E R
     & LBOT_CFL_T(IDIM1:IDIM2,JDIM1:JDIM2)
     &,LTOP_CFL_T(IDIM1:IDIM2,JDIM1:JDIM2)
     &,LBOT_CFL_U(IDIM1:IDIM2,JDIM1:JDIM2)
     &,LTOP_CFL_U(IDIM1:IDIM2,JDIM1:JDIM2)
     &,LBOT_CFL_V(IDIM1:IDIM2,JDIM1:JDIM2)
     &,LTOP_CFL_V(IDIM1:IDIM2,JDIM1:JDIM2)
C-----------------------------------------------------------------------
C***********************************************************************
C-----------------------------------------------------------------------
C
C***  FIGURE OUT IF WE ARE IN THE UPSTREAM REGION
C
      UPSTRM=.FALSE.
      IF(MYPE.LE.INPES-1)UPSTRM=.TRUE.
      IF(MYPE.GE.NPES-INPES)UPSTRM=.TRUE.
      IF(MOD(MYPE,INPES).EQ.0)UPSTRM=.TRUE.
      IF(MOD(MYPE+1,INPES).EQ.0)UPSTRM=.TRUE.
C
      JAKONE=0
C
      DO 25 JA=1,JAM
      IHL=IHLA(JA)
      IHH=IHHA(JA)
      J=JRA(JA)
      LJRA(JA)=.FALSE.
C
      IF(J.GE.MY_JS_GLB-JBPAD2.AND.J.LE.MY_JE_GLB+JTPAD2)THEN
        LJRA(JA)=.TRUE.
        DO I=IHL,IHH
          IF(I.GE.MY_IS_GLB-ILPAD2.AND.I.LE.MY_IE_GLB+IRPAD2)THEN
            JAKONE=JAKONE+1
            KHHAS(JAKONE)=JA
            IHLAS(JAKONE)=I
            JHLAS(JAKONE)=J
          ENDIF
        ENDDO
      ENDIF
C
   25 CONTINUE
C
      JAKTWO=0
      DO 50 JA=1,JAM
        IVL=IVLA(JA)
        IVH=IVHA(JA)
        J=JRA(JA)
C
      DO 50 I=IVL,IVH
      IF(I.GE.MY_IS_GLB-ILPAD2.AND.I.LE.MY_IE_GLB+IRPAD2.AND.
     1   J.GE.MY_JS_GLB-JBPAD2.AND.J.LE.MY_JE_GLB+JTPAD2)THEN
        JAKTWO=JAKTWO+1
        KVHAS(JAKTWO)=JA
        IVLAS(JAKTWO)=I
        JVLAS(JAKTWO)=J
      ENDIF
   50 CONTINUE
C
!-----------------------------------------------------------------------
!
!***  IF THE CFL CRITERION IS VIOLATED THEN LOCATE VERTICAL LIMITS
!***  BETWEEN WHICH TO SMOOTH THE TENDENCIES.
!
!-----------------------------------------------------------------------
!$omp parallel do
      DO J=MYJS,MYJE
      DO I=MYIS,MYIE
        LTOP_CFL_T(I,J)=0
        LBOT_CFL_T(I,J)=0
        LTOP_CFL_U(I,J)=0
        LBOT_CFL_U(I,J)=0
        LTOP_CFL_V(I,J)=0
        LBOT_CFL_V(I,J)=0
        DP_HI(I,J)=0.
        DP_LO(I,J)=0.
        TTB(I,J)=0.
        TUB(I,J)=0.
        TVB(I,J)=0.
      ENDDO
      ENDDO
!
      DO J=MYJS_P1,MYJE_P1
      DO I=MYIS_P1,MYIE_P1
        DP_HI(I,J)=DETA1(1)*PDTOP+DETA2(1)*PDSLO(I,J)
      ENDDO
      ENDDO
!
!-----------------------------------------------------------------------
      LTOP_CFL_TSUM=0
      LTOP_CFL_USUM=0
      LTOP_CFL_VSUM=0
!
      DO L=1,LM1
!
!$omp parallel do private (cfl)
!
        DO J=MYJS_P1,MYJE_P1
        DO I=MYIS_P1,MYIE_P1
          DP_LO(I,J)=DETA1(L+1)*PDTOP+DETA2(L+1)*PDSLO(I,J)
          PETDTL(I,J)=PETDT(I,J,L)
        ENDDO
        ENDDO
!
        DO J=MYJS2,MYJE2
        DO I=MYIS1,MYIE1
!
!***  MASS POINTS
!
          CFL=PETDT(I,J,L)*DT*HBM2(I,J)/(0.5*(DP_HI(I,J)+DP_LO(I,J)))
!
          IF(ABS(CFL).GT.CFL_MAX)THEN
            IF(LTOP_CFL_T(I,J).EQ.0)LTOP_CFL_T(I,J)=MAX(L,2)
            IF(LBOT_CFL_T(I,J).LT.L)LBOT_CFL_T(I,J)=MAX(L,2)
!zjtest            print*,'cflt ',i,j,l,fis(i,j),cfl,pdslo(i,j),pdsl(i,j) !zjtest
          ENDIF
!
!***  U COMPONENT
!
          CFL=(PETDTL(I+IVW(J),J)+PETDTL(I+IVE(J),J))*DT*VBM2(I,J)
     1       /((DP_HI(I+IVW(J),J)+DP_HI(I+IVE(J),J)
     2         +DP_LO(I+IVW(J),J)+DP_LO(I+IVE(J),J))*0.5)
!
          IF(ABS(CFL).GT.CFL_MAX)THEN
            IF(LTOP_CFL_U(I,J).EQ.0)LTOP_CFL_U(I,J)=MAX(L,2)
            IF(LBOT_CFL_U(I,J).LT.L)LBOT_CFL_U(I,J)=MAX(L,2)
          ENDIF
!
!***  V COMPONENT
!
          CFL=(PETDTL(I,J-1)+PETDTL(I,J+1))*DT*VBM2(I,J)
     1       /((DP_HI(I,J-1)+DP_HI(I,J+1)
     2         +DP_LO(I,J-1)+DP_LO(I,J+1))*0.5)
!
          IF(ABS(CFL).GT.CFL_MAX)THEN
            IF(LTOP_CFL_V(I,J).EQ.0)LTOP_CFL_V(I,J)=MAX(L,2)
            IF(LBOT_CFL_V(I,J).LT.L)LBOT_CFL_V(I,J)=MAX(L,2)
          ENDIF
!
        ENDDO
        ENDDO
!
        DO J=MYJS,MYJE
        DO I=MYIS,MYIE
          DP_HI(I,J)=DP_LO(I,J)
        ENDDO
        ENDDO
!
      ENDDO
!
!***  COUNT THE NUMBER OF POINTS WHERE VERTICAL CFL IS VIOLATED
!
      DO J=MYJS2,MYJE2
      DO I=MYIS1,MYIE1
        IF(LTOP_CFL_T(I,J).GT.0)THEN
          LTOP_CFL_TSUM=LTOP_CFL_TSUM+1
        ENDIF
!
        IF(LTOP_CFL_U(I,J).GT.0)THEN
          LTOP_CFL_USUM=LTOP_CFL_USUM+1
        ENDIF
!
        IF(LTOP_CFL_V(I,J).GT.0)THEN
          LTOP_CFL_VSUM=LTOP_CFL_VSUM+1
        ENDIF
      ENDDO
      ENDDO
!----------------------------------------------------------------------
      if(LTOP_CFL_TSUM.gt.0.or.
     &   LTOP_CFL_USUM.gt.0.or.LTOP_CFL_VSUM.gt.0) then
      print*,'*** CFL violation summary: T ',LTOP_CFL_TSUM
     &                                ,' U ',LTOP_CFL_USUM
     &                                ,' V ',LTOP_CFL_VSUM
      endif
!----------------------------------------------------------------------
!
!***  COMPUTE VERTICAL ADVECTION TENDENCIES
!
!----------------------------------------------------------------------
!$omp parallel do
!$omp& private (dpde,petdtl,t_hi,tdn,tta,ttb,tua,tub,tva,tvb,
!$omp&          u_hi,udn,v_hi,vdn)
      do l=1,lm
!
        do j=myjs_p1,myje_p1
        do i=myis_p1,myie_p1
          dpde(i,j)=deta1(l)*pdtop+deta2(l)*pdslo(i,j)
        enddo
        enddo
!
        if(l.lt.lm)then
          do j=myjs1_p1,myje1_p1
          do i=myis1_p1,myie1_p1
            tdn(i,j)=t(i,j,l+1)
            udn(i,j)=u(i,j,l+1)
            vdn(i,j)=v(i,j,l+1)
            petdtl(i,j)=petdt(i,j,l)
          enddo
          enddo
        else
          do j=myjs,myje
          do i=myis,myie
            petdtl(i,j)=0.
          enddo
          enddo
        endif
!
        do j=myjs2,myje2
        do i=myis1,myie1
!
          t_hi=t(i,j,l)
          u_hi=u(i,j,l)
          v_hi=v(i,j,l)
!
!***  temperature
!
          tta=(tdn(i,j)-t_hi)*petdtl(i,j)*f4d
          vad_tend_t(i,j,l)=(tta+ttb(i,j))/dpde(i,j)+t(i,j,l)
          ttb(i,j)=tta
!
!***  u component
!
          tua=(udn(i,j)-u_hi)
     1       *(petdtl(i+ivw(j),j)+petdtl(i+ive(j),j))*f4d
          vad_tend_u(i,j,l)=(tua+tub(i,j))
     1                     /(dpde(i+ivw(j),j)+dpde(i+ive(j),j))
     &                     +u(i,j,l)
          tub(i,j)=tua
!
!***  v component
!
          tva=(vdn(i,j)-v_hi)*(petdtl(i,j-1)+petdtl(i,j+1))*f4d
          vad_tend_v(i,j,l)=(tva+tvb(i,j))/(dpde(i,j-1)+dpde(i,j+1))
     &                     +v(i,j,l)
          tvb(i,j)=tva
        enddo
        enddo
!
      enddo
!--------------second matsuno step-------------------------------------
      do l=1,lm
!
        do j=myjs_p1,myje_p1
        do i=myis_p1,myie_p1
          dpde(i,j)=deta1(l)*pdtop+deta2(l)*pdslo(i,j)
        enddo
        enddo
!
        if(l.lt.lm)then
          do j=myjs1_p1,myje1_p1
          do i=myis1_p1,myie1_p1
            tdn(i,j)=vad_tend_t(i,j,l+1)
            udn(i,j)=vad_tend_u(i,j,l+1)
            vdn(i,j)=vad_tend_v(i,j,l+1)
            petdtl(i,j)=petdt(i,j,l)
          enddo
          enddo
        else
          do j=myjs,myje
          do i=myis,myie
            petdtl(i,j)=0.
          enddo
          enddo
        endif
!
        do j=myjs2,myje2
        do i=myis1,myie1
!
          t_hi=vad_tend_t(i,j,l)
          u_hi=vad_tend_u(i,j,l)
          v_hi=vad_tend_v(i,j,l)
!
!***  temperature
!
          tta=(tdn(i,j)-t_hi)*petdtl(i,j)*f4d
          vad_tend_t(i,j,l)=(tta+ttb(i,j))/dpde(i,j)
          ttb(i,j)=tta
!
!***  u component
!
          tua=(udn(i,j)-u_hi)
     1       *(petdtl(i+ivw(j),j)+petdtl(i+ive(j),j))*f4d
          vad_tend_u(i,j,l)=(tua+tub(i,j))
     1                     /(dpde(i+ivw(j),j)+dpde(i+ive(j),j))
          tub(i,j)=tua
!
!***  v component
!
          tva=(vdn(i,j)-v_hi)*(petdtl(i,j-1)+petdtl(i,j+1))*f4d
          vad_tend_v(i,j,l)=(tva+tvb(i,j))/(dpde(i,j-1)+dpde(i,j+1))
          tvb(i,j)=tva
        enddo
        enddo
!
      enddo

!----------------------------------------------------------------------
!
!***  IF THE CFL CRITERION IS VIOLATED THEN VERTICALLY SMOOTH
!***  THE TENDENCIES
!
!-----------------------------------------------------------------------
!
!***  TEMPERATURE
!
      IF(LTOP_CFL_TSUM.GT.0)THEN
!$omp parallel do private(lstart,lstop,vad_tndx_t)
        DO J=MYJS2,MYJE2
        DO I=MYIS1,MYIE1
!
          LSTART=LTOP_CFL_T(I,J)
          LSTOP =MIN(LBOT_CFL_T(I,J),LM-1)
!
              if(lstop.gt.0) then
          pdp=pdslo(i,j)
          dphi=deta1(lstart-1)*pdtop+deta2(lstart-1)*pdp
          dpmd=deta1(lstart)*pdtop+deta2(lstart)*pdp
!
          do l=lstart,lstop
            dplo=deta1(l+1)*pdtop+deta2(l+1)*pdp
!
            rdpm=1./(dphi+dpmd+dpmd+dplo)
!
            vad_tndx_t(l)=(vad_tend_t(i,j,l-1)*dphi
     &                    +vad_tend_t(i,j,l)*(dpmd+dpmd)
     &                    +vad_tend_t(i,j,l+1)*dplo)*rdpm
!
            dphi=dpmd
            dpmd=dplo
          enddo

          do l=lstart,lstop
            vad_tend_t(i,j,l)=vad_tndx_t(l)
          enddo
              endif
!
        ENDDO
        ENDDO
      ENDIF
!
!***  U COMPONENT
!
      IF(LTOP_CFL_USUM.GT.0)THEN
!$omp parallel do private(lstart,lstop,vad_tndx_u)
        DO J=MYJS2,MYJE2
        DO I=MYIS1,MYIE1
!
          LSTART=LTOP_CFL_U(I,J)
          LSTOP =MIN(LBOT_CFL_U(I,J),LM-1)
!
              if(lstop.gt.0) then
          pdp=(pdslo(i+ivw(j),j)+pdslo(i+ive(j),j))*0.5
          dphi=deta1(lstart-1)*pdtop+deta2(lstart-1)*pdp
          dpmd=deta1(lstart)*pdtop+deta2(lstart)*pdp
!
          do l=lstart,lstop
            dplo=deta1(l+1)*pdtop+deta2(l+1)*pdp
!
            rdpm=1./(dphi+dpmd+dpmd+dplo)
!
            vad_tndx_u(l)=(vad_tend_u(i,j,l-1)*dphi
     &                    +vad_tend_u(i,j,l)*(dpmd+dpmd)
     &                    +vad_tend_u(i,j,l+1)*dplo)*rdpm
!
            dphi=dpmd
            dpmd=dplo
          enddo

          do l=lstart,lstop
            vad_tend_u(i,j,l)=vad_tndx_u(l)
          enddo
              endif
        ENDDO
        ENDDO
      ENDIF
!
!***  V COMPONENT
!
      IF(LTOP_CFL_VSUM.GT.0)THEN
!$omp parallel do private(lstart,lstop,vad_tndx_v)
        DO J=MYJS2,MYJE2
        DO I=MYIS1,MYIE1
!
          LSTART=LTOP_CFL_V(I,J)
          LSTOP =MIN(LBOT_CFL_V(I,J),LM-1)
!
              if(lstop.gt.0) then
          pdp=(pdslo(i,j-1)+pdslo(i,j+1))*0.5
          dphi=deta1(lstart-1)*pdtop+deta2(lstart-1)*pdp
          dpmd=deta1(lstart)*pdtop+deta2(lstart)*pdp
!
          do l=lstart,lstop
            dplo=deta1(l+1)*pdtop+deta2(l+1)*pdp
!
            rdpm=1./(dphi+dpmd+dpmd+dplo)
!
            vad_tndx_v(l)=(vad_tend_v(i,j,l-1)*dphi
     &                    +vad_tend_v(i,j,l)*(dpmd+dpmd)
     &                    +vad_tend_v(i,j,l+1)*dplo)*rdpm
!
            dphi=dpmd
            dpmd=dplo
          enddo

          do l=lstart,lstop
            vad_tend_v(i,j,l)=vad_tndx_v(l)
          enddo
              endif
        ENDDO
        ENDDO
      ENDIF
C-----------------------------------------------------------------------
      CALL ZERO2(ADT)
      CALL ZERO2(ADU)
      CALL ZERO2(ADV)
!      CALL ZERO2(DPDE)
!      CALL ZERO2(TST)
!      CALL ZERO2(UST)
!      CALL ZERO2(VST)
!      CALL ZERO2(TEW)
!      CALL ZERO2(TNS)
!      CALL ZERO2(TNE)
!      CALL ZERO2(TSE)
!      CALL ZERO2(UEW)
!      CALL ZERO2(UNS)
!      CALL ZERO2(VEW)
!      CALL ZERO2(VNS)
!      CALL ZERO2(UNE)
!      CALL ZERO2(USE)
!      CALL ZERO2(VNE)
!      CALL ZERO2(VSE)
C***********************************************************************
C-----------------------------------------------------------------------
      DO 500 L=1,LM         ! Main Vertical Loop
C-----------------------------------------------------------------------
C***********************************************************************
!$omp parallel do
      DO J=MYJS_P4,MYJE_P4
      DO I=MYIS_P4,MYIE_P4
        HM(I,J)=HTM(I,J,L)*HBM2(I,J)
        VM(I,J)=VTM(I,J,L)*VBM2(I,J)
        DPDE(I,J)=DETA1(L)*PDTOP+DETA2(L)*PDSLO(I,J)
        RDPD(I,J)=1./DPDE(I,J)
        UST(I,J)=U(I,J,L)*FFC+UOLD(I,J,L)*FBC
        VST(I,J)=V(I,J,L)*FFC+VOLD(I,J,L)*FBC
        TST(I,J)=T(I,J,L)*FFC+TOLD(I,J,L)*FBC
      ENDDO
      ENDDO
C-----------------------------------------------------------------------
      IF(L.LT.LM)THEN
!$omp parallel do
        DO J=MYJS_P1,MYJE_P1
        DO I=MYIS_P1,MYIE_P1
!         UDN(I,J)=U(I,J,L+1)*FFC+UOLD(I,J,L+1)*FBC
!         VDN(I,J)=V(I,J,L+1)*FFC+VOLD(I,J,L+1)*FBC
!         TDN(I,J)=T(I,J,L+1)*FFC+TOLD(I,J,L+1)*FBC
          PETDTL(I,J)=PETDT(I,J,L)
        ENDDO
        ENDDO
      ELSE
!$omp parallel do
        DO J=MYJS_P1,MYJE_P1
        DO I=MYIS_P1,MYIE_P1
          PETDTL(I,J)=0.
        ENDDO
        ENDDO
      ENDIF
!--------------MASS FLUXES AND MASS POINTS ADVECTION COMPONENTS---------
              do j=myjs1_p3,myje1_p3
          do i=myis_p3,myie_p3
      adpdx(i,j)=dpde(i+ivw(j),j)+dpde(i+ive(j),j)
      adpdy(i,j)=dpde(i,j-1)+dpde(i,j+1)
      rdpdx(i,j)=1./adpdx(i,j)
      rdpdy(i,j)=1./adpdy(i,j)
!
      udy=u(i,j,l)*dy
      vdx=v(i,j,l)*dx(i,j)
!
      fewp=udy*adpdx(i,j)
      fnsp=vdx*adpdy(i,j)
!
      few(i,j,l)=fewp
      fns(i,j,l)=fnsp
!
      tew(i,j)=fewp*(tst(i+ive(j),j)-tst(i+ivw(j),j))
      tns(i,j)=fnsp*(tst(i,j+1)-tst(i,j-1))
!
      uned(i,j)=udy+vdx
      used(i,j)=udy-vdx
          enddo
              enddo
!--------------DIAGONAL FLUXES AND DIAGONALLY AVERAGED WIND-------------
              do j=myjs1_p2,myje2_p2
          do i=myis_p2,myie_p2
      fnep=(uned(i+ihe(j),j)+uned(i       ,j+1))
     1    *(dpde(i       ,j)+dpde(i+ihe(j),j+1))
      fne(i,j,l)=fnep
      tne(i,j)=fnep*(tst(i+ihe(j),j+1)-tst(i,j))
          enddo
              enddo
!
              do j=myjs2_p2,myje1_p2
          do i=myis_p2,myie_p2
      fsep=(used(i+ihe(j),j)+used(i       ,j-1))
     1    *(dpde(i       ,j)+dpde(i+ihe(j),j-1))
      fse(i,j,l)=fsep
      tse(i,j)=fsep*(tst(i+ihe(j),j-1)-tst(i,j))
          enddo
              enddo
C--------------THERMODYNAMIC EQUATION-----------------------------------
C***
C***  THE AD ARRAYS IN THE LOOP ARE ON H POINTS
C***
!$omp parallel do
!      DO J=MYJS5_P1,MYJE5_P1
!      DO I=MYIS1_P1,MYIE1_P1
      DO J=MYJS5,MYJE5
      DO I=MYIS2,MYIE2
        ADT(I,J)=(TEW(I+IHW(J),J)+TEW(I+IHE(J),J)+TNS(I,J-1)+TNS(I,J+1)
     &           +TNE(I+IHW(J),J-1)+TNE(I,J)+TSE(I,J)+TSE(I+IHW(J),J+1))
     &           *RDPD(I,J)*FAD(I,J)
      ENDDO
      ENDDO
C-----------------------------------------------------------------------
C------------------UPSTREAM ADVECTION OF T------------------------------
C-----------------------------------------------------------------------
      IF(UPSTRM)THEN
!$omp parallel do
!$omp& private (i,ix,j,ja,jx,pp,qp,ta,tb)
        DO 100 JAK=1,JAKONE
        JA=KHHAS(JAK)
        I=IHLAS(JAK)
        J=JHLAS(JAK)
        IX=I-MY_IS_GLB+1
        JX=J-MY_JS_GLB+1
        TA=EMT(JA)*(UST(IX,JX-1)+UST(IX+IHW(JX),JX)
     &             +UST(IX+IHE(JX),JX)+UST(IX,JX+1))
        TB=ENT*(VST(IX,JX-1)+VST(IX+IHW(JX),JX)
     &         +VST(IX+IHE(JX),JX)+VST(IX,JX+1))
        PP=-TA-TB
        QP= TA-TB
C
        IF(PP.LT.0.)THEN
          ISPA(JAK)=-1
        ELSE
          ISPA(JAK)= 1
        ENDIF
C
        IF(QP.LT.0.)THEN
          ISQA(JAK)=-1
        ELSE
          ISQA(JAK)= 1
        ENDIF
C
        PP=ABS(PP)
        QP=ABS(QP)
        ARRAY3(JAK)=PP*QP
        ARRAY0(JAK)=ARRAY3(JAK)-PP-QP
        ARRAY1(JAK)=PP-ARRAY3(JAK)
        ARRAY2(JAK)=QP-ARRAY3(JAK)
  100   CONTINUE
C
C***
C
        JAK=0
        DO 125 JA=1,JAM
        IHL=IHLA(JA)
        IHH=IHHA(JA)
        J=JRA(JA)
        IF(.NOT.LJRA(JA))GO TO 125
C
        DO I=IHL,IHH
        IF(I.GE.MY_IS_GLB-ILPAD2.AND.I.LE.MY_IE_GLB+IRPAD2)THEN
C
          JAK=JAK+1
          ISP=ISPA(JAK)
          ISQ=ISQA(JAK)
          IFP=(ISP-1)/2
          IFQ=(-ISQ-1)/2
          IPQ=(ISP-ISQ)/2
C
          IX=I-MY_IS_GLB+1
          JX=J-MY_JS_GLB+1
C
          IF(HTM(IX+IHE(JX)+IFP,JX+ISP,L)*HTM(IX+IHE(JX)+IFQ,JX+ISQ,L)
     1      *HTM(IX+IPQ,JX+ISP+ISQ,L).GT.0.1)GO TO 120
C
          IF(HTM(IX+IHE(JX)+IFP,JX+ISP,L)+HTM(IX+IHE(JX)+IFQ,JX+ISQ,L)
     1      +HTM(IX+IPQ,JX+ISP+ISQ,L).LT.0.1)THEN
C
            T(IX+IHE(JX)+IFP,JX+ISP,L)=T(IX,JX,L)
            T(IX+IHE(JX)+IFQ,JX+ISQ,L)=T(IX,JX,L)
            T(IX+IPQ,JX+ISP+ISQ,L)=T(IX,JX,L)
C
          ELSEIF
     1    (HTM(IX+IHE(JX)+IFP,JX+ISP,L)+HTM(IX+IPQ,JX+ISP+ISQ,L)
     2     .LT.0.99)THEN
C
            T(IX+IHE(JX)+IFP,JX+ISP,L)=T(IX,JX,L)
            T(IX+IPQ,JX+ISP+ISQ,L)    =T(IX+IHE(JX)+IFQ,JX+ISQ,L)
C
          ELSEIF
     1     (HTM(IX+IHE(JX)+IFQ,JX+ISQ,L)+HTM(IX+IPQ,JX+ISP+ISQ,L)
     2      .LT.0.99)THEN
C
            T(IX+IHE(JX)+IFQ,JX+ISQ,L)=T(IX,JX,L)
            T(IX+IPQ,JX+ISP+ISQ,L)    =T(IX+IHE(JX)+IFP,JX+ISP,L)
C
          ELSEIF
     1     (HTM(IX+IHE(JX)+IFP,JX+ISP,L)+HTM(IX+IHE(JX)+IFQ,JX+ISQ,L)
     2      .LT.0.99)THEN
            T(IX+IHE(JX)+IFP,JX+ISP,L)=
     1                    0.5*(T(IX,JX,L)+T(IX+IPQ,JX+ISP+ISQ,L))
            T(IX+IHE(JX)+IFQ,JX+ISQ,L)=T(IX+IHE(JX)+IFP,JX+ISP,L)
C
          ELSEIF(HTM(IX+IHE(JX)+IFP,JX+ISP,L).LT.0.99)THEN
            T(IX+IHE(JX)+IFP,JX+ISP,L)=
     1        T(IX,JX,L)+T(IX+IPQ,JX+ISP+ISQ,L)
     2                  -T(IX+IHE(JX)+IFQ,JX+ISQ,L)
C
          ELSEIF(HTM(IX+IHE(JX)+IFQ,JX+ISQ,L).LT.0.99)THEN
            T(IX+IHE(JX)+IFQ,JX+ISQ,L)=
     1        T(IX,JX,L)+T(IX+IPQ,JX+ISP+ISQ,L)
     2                  -T(IX+IHE(JX)+IFP,JX+ISP,L)
C
          ELSE
            T(IX+IPQ,JX+ISP+ISQ,L)=
     1        T(IX+IHE(JX)+IFP,JX+ISP,L)
     2       +T(IX+IHE(JX)+IFQ,JX+ISQ,L)-T(IX,JX,L)
C
          ENDIF
C
  120     CONTINUE
C
          F0=ARRAY0(JAK)
          F1=ARRAY1(JAK)
          F2=ARRAY2(JAK)
          F3=ARRAY3(JAK)
          ADT(IX,JX)=F0*T(IX,JX,L)
     1              +F1*T(IX+IHE(JX)+IFP,JX+ISP,L)
     2              +F2*T(IX+IHE(JX)+IFQ,JX+ISQ,L)
     3              +F3*T(IX+IPQ,JX+ISP+ISQ,L)

!      print*,ntsd,i,j,l,ix,jx
!     &,ADT(IX,JX),T(IX,JX,L)
!     1,T(IX+IHE(JX)+IFP,JX+ISP,L)
!     2,T(IX+IHE(JX)+IFQ,JX+ISQ,L)
!     3,T(IX+IPQ,JX+ISP+ISQ,L)


        ENDIF
        ENDDO
  125   CONTINUE
      ENDIF
C
C--------------CALCULATION OF MOMENTUM ADVECTION COMPONENTS-------------
C
!$omp parallel do
      DO J=MYJS4_P1,MYJE4_P1
      DO I=MYIS_P1,MYIE_P1
C***
C***  THE EW AND NS ARRAYS ARE ON H POINTS
C***
        uew(i,j)=(few(i+ihw(j),j,l)+few(i+ihe(j),j,l))
     1          *(ust(i+ihe(j),j)-ust(i+ihw(j),j))
        uns(i,j)=(fns(i+ihw(j),j,l)+fns(i+ihe(j),j,l))
     1          *(ust(i,j+1)-ust(i,j-1))
        vew(i,j)=(few(i,j-1,l)+few(i,j+1,l))
     1          *(vst(i+ihe(j),j)-vst(i+ihw(j),j))
        vns(i,j)=(fns(i,j-1,l)+fns(i,j+1,l))
     &          *(vst(i,j+1)-vst(i,j-1))
C***
C***  THE NE AND SE ARRAYS ARE TIED TO V POINTS
C***
        une(i,j)=(fne(i+ivw(j),j,l)+fne(i+ive(j),j,l))
     1          *(ust(i+ive(j),j+1)-ust(i,j))
        use(i,j)=(fse(i+ivw(j),j,l)+fse(i+ive(j),j,l))
     1          *(ust(i+ive(j),j-1)-ust(i,j))
        vne(i,j)=(fne(i,j-1,l)+fne(i,j+1,l))
     &          *(vst(i+ive(j),j+1)-vst(i,j))
        vse(i,j)=(fse(i,j-1,l)+fse(i,j+1,l))
     &          *(vst(i+ive(j),j-1)-vst(i,j))
      ENDDO
      ENDDO
C--------------EQUATION OF MOTION---------------------------------------
C***
C***  ADU AND ADV ARE ON V POINTS
C***
!$omp parallel do
      DO J=MYJS5,MYJE5
      DO I=MYIS2,MYIE2
        ADU(I,J)=(UEW(I+IVW(J),J)+UEW(I+IVE(J),J)+UNS(I,J-1)+UNS(I,J+1)
     1           +UNE(I+IVW(J),J-1)+UNE(I,J)+USE(I,J)+USE(I+IVW(J),J+1))
     2           *RDPDX(I,J)*FAD(I+IVW(J),J)
        ADV(I,J)=(VEW(I+IVW(J),J)+VEW(I+IVE(J),J)+VNS(I,J-1)+VNS(I,J+1)
     1           +VNE(I+IVW(J),J-1)+VNE(I,J)+VSE(I,J)+VSE(I+IVW(J),J+1))
     2           *RDPDY(I,J)*FAD(I+IVW(J),J)
      ENDDO
      ENDDO
C--------------UPSTREAM ADVECTION OF VELOCITY COMPONENTS----------------
C
      IF(UPSTRM)THEN
!$omp parallel do
!$omp& private (f0,f1,f2,f3,i,ifp,ifq,ipq,isp,isq,ix,j,ja,jx,
!$omp&          pp,qp,ta,tb)
        DO 200 JAK=1,JAKTWO
        JA=KVHAS(JAK)
        I=IVLAS(JAK)
        J=JVLAS(JAK)
C
        IX=I-MY_IS_GLB+1
        JX=J-MY_JS_GLB+1
C
        TA=EM(JA)*UST(IX,JX)
        TB=EN    *VST(IX,JX)
        PP=-TA-TB
        QP=TA-TB
C
        IF(PP.LT.0.)THEN
          ISP=-1
        ELSE
          ISP= 1
        ENDIF
C
        IF(QP.LT.0.)THEN
          ISQ=-1
        ELSE
          ISQ= 1
        ENDIF
C
        IFP=(ISP-1)/2
        IFQ=(-ISQ-1)/2
        IPQ=(ISP-ISQ)/2
        PP=ABS(PP)
        QP=ABS(QP)
        F3=PP*QP
        F0=F3-PP-QP
        F1=PP-F3
        F2=QP-F3
        ADU(IX,JX)=F0*U(IX,JX,L)+F1*U(IX+IVE(JX)+IFP,JX+ISP,L)
     1                          +F2*U(IX+IVE(JX)+IFQ,JX+ISQ,L)
     2                          +F3*U(IX+IPQ,JX+ISP+ISQ,L)
        ADV(IX,JX)=F0*V(IX,JX,L)+F1*V(IX+IVE(JX)+IFP,JX+ISP,L)
     1                          +F2*V(IX+IVE(JX)+IFQ,JX+ISQ,L)
     2                          +F3*V(IX+IPQ,JX+ISP+ISQ,L)
  200   CONTINUE
      ENDIF
C***
C***  END OF THIS UPSTREAM REGION
C***
C-----------------------------------------------------------------------
!$omp parallel do
              do j=myjs_p4,myje_p4
          do i=myis_p4,myie_p4
      told(i,j,l)=t(i,j,l)
          enddo
              enddo
!
!***  VERTICAL PLUS HORIZONTAL UPDATES
!
      DO J=MYJS2,MYJE2
      DO I=MYIS1,MYIE1
        T(I,J,L)=(VAD_TEND_T(I,J,L)+2.*ADT(I,J))*HM(I,J)+T(I,J,L)
      ENDDO
      ENDDO
!
              do j=myjs_p4,myje_p4
          do i=myis_p4,myie_p4
      uold(i,j,l)=u(i,j,l)
      vold(i,j,l)=v(i,j,l)
          enddo
              enddo
!
      DO J=MYJS2,MYJE2
      DO I=MYIS1,MYIE1
        FPP=CURV(I,J)*2.*UST(I,J)+F(I,J)*2.
        U(I,J,L)=(VAD_TEND_U(I,J,L)+2.*ADU(I,J)+VST(I,J)*FPP)*VM(I,J)
     1           +U(I,J,L)
        V(I,J,L)=(VAD_TEND_V(I,J,L)+2.*ADV(I,J)-UST(I,J)*FPP)*VM(I,J)
     1           +V(I,J,L)
      ENDDO
      ENDDO
C-----------------------------------------------------------------------
C***********************************************************************
  500 CONTINUE             ! End of the Main Vertical Loop
C***********************************************************************
C-----------------------------------------------------------------------
                             RETURN
                             END


