C&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&& 
                            SUBROUTINE HAD2 
C     ****************************************************************** 
C$$$  SUBPROGRAM DOCUMENTATION BLOCK 
C                .      .    . 
C SUBPROGRAM:    ADV2        HORIZ/VERT ADVECTION OF VAPOR AND CLOUD 
C   PRGRMMR: JANJIC          ORG: W/NP22     DATE: 96-07-19 
C 
C ABSTRACT: 
C     ADV2 CALCULATES THE CONTRIBUTION OF THE HORIZONTAL AND VERTICAL
C     ADVECTION TO THE TENDENCIES OF WATER SUBSTANCE AND TKE AND THEN 
C     UPDATES THOSE VARIABLES.  AN ANTI-FILTERING TECHNIQUE IS USED. 
C 
C PROGRAM HISTORY LOG: 
C   96-07-19  JANJIC  - ORIGINATOR 
C   96-08-    BLACK   - CRAY DIRECTIVES, CRAY OPTIMIZATION 
C   99-07-    JANJIC  - ADDED HORIZONTAL ADVECTION OF TKE 
C   99-07-    JANJIC  - ADDED VERTICAL ADVECTION 
C   00-01-06  BLACK   - DISTRIBUTED MEMORY AND THREADS
!   01-09-15  JANJIC  - DXDY, HORIZONTAL ADVECTION LOOPING
C 
C USAGE: CALL ADV2 FROM MAIN PROGRAM 
C   INPUT ARGUMENT LIST: 
C       NONE 
C 
C   OUTPUT ARGUMENT LIST 
C       NONE 
C 
C   OUTPUT FILES: 
C       NONE 
C   SUBPROGRAMS CALLED: 
C 
C     UNIQUE: NONE 
C 
C     LIBRARY: NONE 
C 
C   COMMON BLOCKS: CTLBLK 
C                  LOOPS 
C                  MASKS 
C                  DYNAM 
C                  CONTIN 
C                  VRBLS 
C                  PVRBLS 
C                  CLDWTR 
C                  INDX 
C 
C ATTRIBUTES: 
C   LANGUAGE: FORTRAN 90 
C   MACHINE : IBM SP
C$$$ 
C*********************************************************************** 
      INCLUDE "parmeta" 
      INCLUDE "parm.tbl"
      INCLUDE "mpp.h"
      INCLUDE "mpif.h"
C----------------------------------------------------------------------- 
                             P A R A M E T E R 
     &(IM1=IM-1,JAM=6+2*(JM-10),JAMD=(JAM*2-10)*3,LP1=LM+1) 
C----------------------------------------------------------------------- 
                             P A R A M E T E R
     &(g=9.806
     &,EPSQ=1.E-12,EPSQ2=0.2,CLIMIT=1.E-20
!zj     &,FF1=0.52500,FF2=-0.64813,FF3=0.24520,FF4=-0.12189) 
     &,FF1=0.53000,FF2=-0.64813,FF3=0.24520,FF4=-0.12189)
C----------------------------------------------------------------------- 
                             L O G I C A L
     & RUN,FIRST,RESTRT,SIGMA
C----------------------------------------------------------------------- 
      INCLUDE "CTLBLK.comm" 
C----------------------------------------------------------------------- 
      INCLUDE "LOOPS.comm" 
C----------------------------------------------------------------------- 
      INCLUDE "MASKS.comm" 
C----------------------------------------------------------------------- 
      INCLUDE "DYNAM.comm" 
!-----------------------------------------------------------------------
      include "PHYS2.comm"
C----------------------------------------------------------------------- 
      INCLUDE "CONTIN.comm" 
C----------------------------------------------------------------------- 
      INCLUDE "VRBLS.comm" 
C----------------------------------------------------------------------- 
      INCLUDE "PVRBLS.comm" 
!-----------------------------------------------------------------------
      include 'NHYDRO.comm'
!----------------------------------------------------------------------- 
      INCLUDE "CLDWTR.comm" 
C----------------------------------------------------------------------- 
      INCLUDE "INDX.comm" 
C
C-------------LOCAL VARIABLES REQUIRING DECLARATION--------------------- 
                             D I M E N S I O N
     & IFPA(IDIM1:IDIM2,JDIM1:JDIM2,LM),IFQA(IDIM1:IDIM2,JDIM1:JDIM2,LM)
     &,IFPF(IDIM1:IDIM2,JDIM1:JDIM2,LM),IFQF(IDIM1:IDIM2,JDIM1:JDIM2,LM)
     &,JFPA(IDIM1:IDIM2,JDIM1:JDIM2,LM),JFQA(IDIM1:IDIM2,JDIM1:JDIM2,LM)
     &,JFPF(IDIM1:IDIM2,JDIM1:JDIM2,LM),JFQF(IDIM1:IDIM2,JDIM1:JDIM2,LM)
     &,AFP (IDIM1:IDIM2,JDIM1:JDIM2,LM),AFQ (IDIM1:IDIM2,JDIM1:JDIM2,LM)
     &,Q1  (IDIM1:IDIM2,JDIM1:JDIM2,LM),DQST(IDIM1:IDIM2,JDIM1:JDIM2,LM)
     &,W1  (IDIM1:IDIM2,JDIM1:JDIM2,LM),DWST(IDIM1:IDIM2,JDIM1:JDIM2,LM)
     &,E1  (IDIM1:IDIM2,JDIM1:JDIM2,LM),DEST(IDIM1:IDIM2,JDIM1:JDIM2,LM)
     &,E2  (IDIM1:IDIM2,JDIM1:JDIM2,LM)
     &,DARE(IDIM1:IDIM2,JDIM1:JDIM2),   DVOL(IDIM1:IDIM2,JDIM1:JDIM2,LM)
     &,EMH (IDIM1:IDIM2,JDIM1:JDIM2)
!zj     &,hmask(idim1:idim2,jdim1:jdim2),hmask1(idim1:idim2,jdim1:jdim2)
!----------------------------------------------------------------------- 
                             R E A L
     & GSUMS(6,LM),XSUMS(6,LM)
C-----------------------------------------------------------------------
C--------------HORIZONTAL ADVECTION------------------------------------- 
!-----------------------------------------------------------------------
!zj dy in common dynam
!
      rdy=1./dy
      slopac=slopht*sqrt(2.)*0.5*50.
      crit=slopac*float(idtad)*dt*rdy*1000.
!----------------------------------------------------------------------- 
      ADDT=FLOAT(IDTAD)*DT
C***********************************************************************
      ENH=ADDT/(08.*DY)
!-----------------------------------------------------------------------
!zj              do j=myjs_p1,myje_p2
!zj          do i=myis_p1,myie_p2
!zj      hmask1(i,j)=1.
!zj          enddo
!zj              enddo
!
!zj                  if(sigma) then
!zj dy in common dynam
!zj      slopac=slopht*sqrt(2.)*0.5
!zj      rdy=1./(g*dy)
!
!zj              do j=myjs2_p1,myje2_p2
!zj          do i=myis1_p1,myie1_p2
!zj      dh1=abs(fis(i+ihw(j),j-1)-fis(i,j))
!zj      dh2=abs(fis(i+ihe(j),j-1)-fis(i,j))
!zj      dh3=abs(fis(i+ihw(j),j+1)-fis(i,j))
!zj      dh4=abs(fis(i+ihe(j),j+1)-fis(i,j))
!
!zj      dhr=max(dh1,dh2,dh3,dh4)*rdy
!
!zj          if(dhr.gt.slopac) then
!zj      hmask1(i,j)=(1.+dhr)**2
!zj          endif
!zj          enddo
!zj              enddo
!zj                  endif
!-----------------------------------------------------------------------
!$omp parallel do
      DO J=MYJS_P3,MYJE_P3
      DO I=MYIS_P2,MYIE_P2
        EMH (I,J)=ADDT/(08.*DX(I,J))
        dare(i,j)=hbm3(i,j)*dx(i,j)*dy
        E1(I,J,1)=MAX(Q2(I,J,1)*0.5,EPSQ2)
        E2(I,J,1)=E1(I,J,1)
      ENDDO
      ENDDO
C*********************************************************************** 
!$omp parallel do
!$omp& private (destij,dqstij,dvolp,dwstij,hbm2ij,hm,
!$omp&          jfp,jfq,pp,qp,sumne,sumnq,sumnw,sumpe,sumpq,sumpw,
!$omp&          tta,ttb)
      DO 300 L=1,LM
C*********************************************************************** 
      DO J=MYJS_P3,MYJE_P3
      DO I=MYIS_P2,MYIE_P2
        dvol(i,j,l)=dare(i,j)*(deta1(l)*pdtop+deta2(l)*pdsl(i,j)) 
        HM=HTM(I,J,L) 
        Q  (I,J,L)=MAX(Q  (I,J,L),EPSQ)*HM 
        CWM(I,J,L)=MAX(CWM(I,J,L),CLIMIT)*HM 
        Q1 (I,J,L)=Q  (I,J,L) 
        W1 (I,J,L)=CWM(I,J,L) 
      ENDDO 
      ENDDO 
C
      IF(L.GT.1)THEN 
        DO J=MYJS_P3,MYJE_P3
        DO I=MYIS_P2,MYIE_P2
          E1(I,J,L)=MAX((Q2(I,J,L-1)+Q2(I,J,L))*0.5,EPSQ2) 
          E2(I,J,L)=E1(I,J,L) 
        ENDDO 
        ENDDO 
      ENDIF 
C----------------------------------------------------------------------- 
      DO 225 J=MYJS2_P1,MYJE2_P1
      DO 225 I=MYIS1_P1,MYIE1_P1
!
      hm=htm(i,j,l)*hbm2(i,j)
!
      tta=(u(i,j-1,l)+u(i+ihw(j),j,l)+u(i+ihe(j),j,l)+u(i,j+1,l)) 
     2   *emh(i,j)*hm 
      ttb=(v(i,j-1,l)+v(i+ihw(j),j,l)+v(i+ihe(j),j,l)+v(i,j+1,l)) 
     2   *enh*hm 
! 
      spp=-tta-ttb 
      sqp= tta-ttb 
!
          if(spp.lt.0.) then
      jfp=-1
          else
      jfp=1
          endif
!
          if(sqp.lt.0.) then 
      jfq=-1
          else
      jfq=1
          endif 
! 
      IFPA(I,J,L)=IHE(J)+I+( JFP-1)/2 
      IFQA(I,J,L)=IHE(J)+I+(-JFQ-1)/2 
! 
      JFPA(I,J,L)=J+JFP 
      JFQA(I,J,L)=J+JFQ 
! 
      IFPF(I,J,L)=IHE(J)+I+(-JFP-1)/2 
      IFQF(I,J,L)=IHE(J)+I+( JFQ-1)/2 
! 
      JFPF(I,J,L)=J-JFP 
      JFQF(I,J,L)=J-JFQ 
!-----------------------------------------------------------------------
                 if(.not.hydro) then ! z currently not available for hydro=.true.
      dza=(z(ifpa(i,j,l),jfpa(i,j,l),l)-z(i,j,l))*rdy
      dzb=(z(ifqa(i,j,l),jfqa(i,j,l),l)-z(i,j,l))*rdy
              if(abs(dza).gt.slopac) then
          ssa=dza*spp
          if(ssa.gt.crit) then
!zj      spp=crit/dza
      spp=0. !spp*.1
          endif
              endif
              if(abs(dzb).gt.slopac) then
          ssb=dzb*sqp
          if(ssb.gt.crit) then
!zj      sqp=crit/dzb
      sqp=0. !sqp*.1
          endif
              endif
                  endif
!-----------------------------------------------------------------------
      spp=spp*htm(ifpa(i,j,l),jfpa(i,j,l),l) 
      sqp=sqp*htm(ifqa(i,j,l),jfqa(i,j,l),l) 
!
      fpq=spp*sqp*htm(i,j-2,l)*htm(i-1,j,l)
     &           *htm(i+1,j,l)*htm(i,j+2,l)*0.25
!
      pp=abs(spp)
      qp=abs(sqp)
C 
      AFP (I,J,L)=(((FF4*PP+FF3)*PP+FF2)*PP+FF1)*PP 
      AFQ (I,J,L)=(((FF4*QP+FF3)*QP+FF2)*QP+FF1)*QP 
!
      q1(i,j,l)=(q  (ifpa(i,j,l),jfpa(i,j,l),l)-q  (i,j,l))*pp 
     &         +(q  (ifqa(i,j,l),jfqa(i,j,l),l)-q  (i,j,l))*qp 
     &         +(q  (i,j-2,l)+q  (i,j+2,l)-q  (i-1,j,l)-q  (i+1,j,l))
     &         *fpq
     &         +q(i,j,l)
!
      w1(i,j,l)=(cwm(ifpa(i,j,l),jfpa(i,j,l),l)-cwm(i,j,l))*pp 
     &         +(cwm(ifqa(i,j,l),jfqa(i,j,l),l)-cwm(i,j,l))*qp 
     &         +(cwm(i,j-2,l)+cwm(i,j+2,l)-cwm(i-1,j,l)-cwm(i+1,j,l))
     &         *fpq
     &         +cwm(i,j,l)
!
      e2(i,j,l)=(e1 (ifpa(i,j,l),jfpa(i,j,l),l)-e1 (i,j,l))*pp 
     &         +(e1 (ifqa(i,j,l),jfqa(i,j,l),l)-e1 (i,j,l))*qp 
     &         +(e1 (i,j-2,l)+e1 (i,j+2,l)-e1 (i-1,j,l)-e1 (i+1,j,l))
     &         *fpq
     &         +e1(i,j,l)
!
  225 continue
!-----------------------------------------------------------------------
  300 CONTINUE          ! End of Vertical Loop
C--------------ANTI-FILTERING STEP-------------------------------------- 
      do 400 l=1,lm
!-----------------------------------------------------------------------
!zj                  if(deta2(l).gt.0.) then
!zj              do j=myjs_p1,myje_p2
!zj          do i=myis_p1,myie_p2
!zj      hmask(i,j)=hmask1(i,j)
!zj          enddo
!zj              enddo
!zj                  else
!zj              do j=myjs_p1,myje_p2
!zj          do i=myis_p1,myie_p2
!zj      hmask(i,j)=1.0
!zj          enddo
!zj              enddo
!zj                  endif
!-----------------------------------------------------------------------
      SUMPQ=0. 
      SUMNQ=0. 
      SUMPW=0. 
      SUMNW=0. 
      SUMPE=0. 
      SUMNE=0. 
C--------------ANTI-FILTERING LIMITERS---------------------------------- 
      DO 350 J=MYJS2,MYJE2
      DO 350 I=MYIS1,MYIE1
!
      DVOLP=DVOL(I,J,L)
      Q1IJ =Q1(I,J,L)
      W1IJ =W1(I,J,L)
      E2IJ =E2(I,J,L)
!
      hafp=htm(ifpf(i,j,l),jfpf(i,j,l),l)*afp(i,j,l) !zj*hmask(i,j)
      hafq=htm(ifqf(i,j,l),jfqf(i,j,l),l)*afq(i,j,l) !zj*hmask(i,j)
!
      d2pqq=(q1(ifpa(i,j,l),jfpa(i,j,l),l)-q1ij
     2      -q1ij+q1(ifpf(i,j,l),jfpf(i,j,l),l))
     3     *hafp
     4     +(q1(ifqa(i,j,l),jfqa(i,j,l),l)-q1ij
     5      -q1ij+q1(ifqf(i,j,l),jfqf(i,j,l),l))
     6     *hafq
C 
      d2pqw=(w1(ifpa(i,j,l),jfpa(i,j,l),l)-w1ij
     2      -w1ij+w1(ifpf(i,j,l),jfpf(i,j,l),l))
     3     *hafp
     4     +(w1(ifqa(i,j,l),jfqa(i,j,l),l)-w1ij
     5      -w1ij+w1(ifqf(i,j,l),jfqf(i,j,l),l))
     6     *hafq
C 
      d2pqe=(e2(ifpa(i,j,l),jfpa(i,j,l),l)-e2ij
     2      -e2ij+e2(ifpf(i,j,l),jfpf(i,j,l),l))
     3      *hafp
     4     +(e2(ifqa(i,j,l),jfqa(i,j,l),l)-e2ij
     5      -e2ij+e2(ifqf(i,j,l),jfqf(i,j,l),l))
     6      *hafq
!
!zj      if(d2pqq.gt.0.) d2pqq=d2pqq*1.05
!zj      if(d2pqw.gt.0.) d2pqw=d2pqw*1.05
! 
      QSTIJ=Q1IJ-D2PQQ 
      WSTIJ=W1IJ-D2PQW 
      ESTIJ=E2IJ-D2PQE 
C 
      Q00=Q  (I          ,J          ,L) 
      QP0=Q  (IFPA(I,J,L),JFPA(I,J,L),L) 
      Q0Q=Q  (IFQA(I,J,L),JFQA(I,J,L),L) 
C 
      W00=CWM(I          ,J          ,L) 
      WP0=CWM(IFPA(I,J,L),JFPA(I,J,L),L) 
      W0Q=CWM(IFQA(I,J,L),JFQA(I,J,L),L) 
C 
      E00=E1 (I          ,J          ,L) 
      EP0=E1 (IFPA(I,J,L),JFPA(I,J,L),L) 
      E0Q=E1 (IFQA(I,J,L),JFQA(I,J,L),L) 
C 
      QSTIJ=MAX(QSTIJ,MIN(Q00,QP0,Q0Q)) 
      QSTIJ=MIN(QSTIJ,MAX(Q00,QP0,Q0Q)) 
      WSTIJ=MAX(WSTIJ,MIN(W00,WP0,W0Q)) 
      WSTIJ=MIN(WSTIJ,MAX(W00,WP0,W0Q)) 
      ESTIJ=MAX(ESTIJ,MIN(E00,EP0,E0Q)) 
      ESTIJ=MIN(ESTIJ,MAX(E00,EP0,E0Q)) 
C 
      dqstij=qstij-q(i,j,l) 
      dwstij=wstij-cwm(i,j,l) 
      destij=estij-e1(i,j,l)
C 
      DQST(I,J,L)=DQSTIJ 
      DWST(I,J,L)=DWSTIJ 
      DEST(I,J,L)=DESTIJ 
C 
      DQSTIJ=DQSTIJ*DVOLP 
      DWSTIJ=DWSTIJ*DVOLP 
      DESTIJ=DESTIJ*DVOLP 
C 
      IF(DQSTIJ.GT.0.)THEN
        SUMPQ=SUMPQ+DQSTIJ
      ELSE
        SUMNQ=SUMNQ+DQSTIJ
      ENDIF
C
      IF(DWSTIJ.GT.0.)THEN
        SUMPW=SUMPW+DWSTIJ
      ELSE
        SUMNW=SUMNW+DWSTIJ
      ENDIF
C
      IF(DESTIJ.GT.0.)THEN
        SUMPE=SUMPE+DESTIJ
      ELSE
        SUMNE=SUMNE+DESTIJ
      ENDIF
C
  350 CONTINUE
C-----------------------------------------------------------------------
      XSUMS(1,L)=SUMPQ
      XSUMS(2,L)=SUMNQ
      XSUMS(3,L)=SUMPW
      XSUMS(4,L)=SUMNW
      XSUMS(5,L)=SUMPE
      XSUMS(6,L)=SUMNE
C
  400 CONTINUE            ! End of Vertical Loop
C-----------------------------------------------------------------------
C
C***  GLOBAL REDUCTION
C
      CALL MPI_ALLREDUCE(XSUMS,GSUMS,6*LM,MPI_REAL,MPI_SUM,
     1                     MPI_COMM_COMP,IRECV)
      IF(IRECV.NE.0)THEN
         PRINT*, ' RETURN CODE FROM 2nd ALLREDUCE IN ADV2 = ',IRECV
         STOP
      ENDIF
C
C***  END OF GLOBAL REDUCTION
C
C-----------------------------------------------------------------------
      DO 425 L=1,LM
C
      SUMPQ=GSUMS(1,L)
      SUMNQ=GSUMS(2,L)
      SUMPW=GSUMS(3,L)
      SUMNW=GSUMS(4,L)
      SUMPE=GSUMS(5,L)
      SUMNE=GSUMS(6,L)
C
C--------------FIRST MOMENT CONSERVING FACTOR--------------------------- 
C
      IF(SUMPQ.GT.1.)THEN
        RFACQ=-SUMNQ/SUMPQ
      ELSE
        RFACQ=1.
      ENDIF
C
      IF(SUMPW.GT.1.)THEN
        RFACW=-SUMNW/SUMPW
      ELSE
        RFACW=1.
      ENDIF
C
      IF(SUMPE.GT.1.)THEN
        RFACE=-SUMNE/SUMPE
      ELSE
        RFACE=1.
      ENDIF
!-----------------------------------------------------------------------
!      rfacq=1.
!      rfacw=1.
!      rface=1.
!-----------------------------------------------------------------------
      IF(RFACQ.LT.0.9.OR.RFACQ.GT.1.1)RFACQ=1. 
      IF(RFACW.LT.0.9.OR.RFACW.GT.1.1)RFACW=1. 
      IF(RFACE.LT.0.9.OR.RFACE.GT.1.1)RFACE=1. 
C
C--------------IMPOSE CONSERVATION ON ANTI-FILTERING-------------------- 
C
      IF(RFACQ.LT.1.)THEN
!$omp parallel do private (dqstij,rfqij)
        DO J=MYJS2,MYJE2
        DO I=MYIS1,MYIE1
          DQSTIJ=DQST(I,J,L)
          RFQIJ=HBM2(I,J)*(RFACQ-1.)+1.
          IF(DQSTIJ.GE.0.)DQSTIJ=DQSTIJ*RFQIJ
          q(i,j,l)=q(i,j,l)+dqstij 
        ENDDO
        ENDDO
      ELSE
!$omp parallel do private (dqstij,rfqij)
        DO J=MYJS2,MYJE2
        DO I=MYIS1,MYIE1
          DQSTIJ=DQST(I,J,L)
          RFQIJ=HBM2(I,J)*(RFACQ-1.)+1.
          IF(DQSTIJ.LT.0.)DQSTIJ=DQSTIJ/RFQIJ
          q(i,j,l)=q(i,j,l)+dqstij 
        ENDDO
        ENDDO
      ENDIF
C----------------------------------------------------------------------- 
      IF(RFACW.LT.1.)THEN
!$omp parallel do private (dwstij,rfwij)
        DO J=MYJS2,MYJE2
        DO I=MYIS1,MYIE1
          DWSTIJ=DWST(I,J,L)
          RFWIJ=HBM2(I,J)*(RFACW-1.)+1.
          IF(DWSTIJ.GE.0.)DWSTIJ=DWSTIJ*RFWIJ
          cwm(i,j,l)=cwm(i,j,l)+dwstij
        ENDDO
        ENDDO
      ELSE
!$omp parallel do private (dwstij,rfwij)
        DO J=MYJS2,MYJE2
        DO I=MYIS1,MYIE1
          DWSTIJ=DWST(I,J,L)
          RFWIJ=HBM2(I,J)*(RFACW-1.)+1.
          IF(DWSTIJ.LT.0.)DWSTIJ=DWSTIJ/RFWIJ
          cwm(i,j,l)=cwm(i,j,l)+dwstij
        ENDDO
        ENDDO
      ENDIF
C----------------------------------------------------------------------- 
      IF(RFACE.LT.1.)THEN
!$omp parallel do private (destij,rfeij)
        DO J=MYJS2,MYJE2
        DO I=MYIS1,MYIE1
          DESTIJ=DEST(I,J,L)
          RFEIJ=HBM2(I,J)*(RFACE-1.)+1.
          IF(DESTIJ.GE.0.)DESTIJ=DESTIJ*RFEIJ
          e1(i,j,l)=e1(i,j,l)+destij
        ENDDO
        ENDDO
      ELSE
!$omp parallel do private (destij,rfeij)
        DO J=MYJS2,MYJE2
        DO I=MYIS1,MYIE1
          DESTIJ=DEST(I,J,L)
          RFEIJ=HBM2(I,J)*(RFACE-1.)+1.
          IF(DESTIJ.LT.0.)DESTIJ=DESTIJ/RFEIJ
          e1(i,j,l)=e1(i,j,l)+destij
        ENDDO
        ENDDO
      ENDIF
C----------------------------------------------------------------------- 
!$omp parallel do private (hm)
      DO J=MYJS,MYJE
      DO I=MYIS,MYIE
        HM=HTM(I,J,L) 
        Q  (I,J,L)=MAX(Q  (I,J,L),EPSQ)*HM 
        CWM(I,J,L)=MAX(CWM(I,J,L),CLIMIT)*HM 
      ENDDO 
      ENDDO 
C***
      IF(L.EQ.LM)GO TO 425
C***
      IF(L.EQ.1)THEN 
!$omp parallel do 
        DO J=MYJS,MYJE
        DO I=MYIS,MYIE
          Q2(I,J,L)=MAX(E1(I,J,L)+E1(I,J,L)-EPSQ2,EPSQ2)*HTM(I,J,1) 
        ENDDO 
        ENDDO 
      ELSE 
!$omp parallel do 
        DO J=MYJS,MYJE
        DO I=MYIS,MYIE
          IF(L.LT.LMH(I,J))THEN 
            Q2(I,J,L)=MAX(E1(I,J,L)+E1(I,J,L)-Q2(I,J,L-1),EPSQ2)
     1                   *HTM(I,J,L) 
          ELSE
            Q2(I,J,L)=Q2(I,J,L-1)
          ENDIF 
        ENDDO 
        ENDDO 
      ENDIF 
C*********************************************************************** 
  425 CONTINUE
C----------------------------------------------------------------------- 
                             RETURN 
                             END 
