      subroutine eta2hyb(ihinx,ihiny,etop,hf,to,qo,uo,vo,cwmo,q2o
     &, ui,vi,ti,qi,q2i,cldlin)
!-----------------------------------------------------------------------
!23456789112345678921234567893123456789412345678951234567896123456789712
C
C       SUBPROGRAM DOCUMENTAION BLOCK
C
C  SUBROUTINE: ETA2HYB  ETA-TO-HYBRID INTERPOLATION
C
C
C  ABSTRACT:  VERTICALLY INTERPOLATES ETA DATA TO LAYERS.
C             PROGRAM ASSUMES HORIZONTAL INTERPOLATION HAS BEEN
C             PERFORMED EARLIER.
C
C  REFERENCE:
C  BLACK, T. L., 1988: THE STEP-MOUNTAIN, ETA COORDINATE MODEL: A
C          DOCUMENTATION. DEVELOPMEMT DIVISON, NATIONAL METEOROLOGICAL
C          CENTER.
C
C  PROGRAM HISTORY LOG:
C    99-12-22  H CHUANG MODIFIED CODE TO INTERPOLATE IC RESTART
C              FILES FROM ETA TO SIGMA   
C    01-03-01  H CHUANG MODIFIED FOR MPI
!
!    01-07-01  Z. Janjic modified for pressure-sigma hybrid
C    01-10-31  H CHUANG DEBUG AND MODIFIED FOR OPERATIONAL USE
C    01-11-15  T BLACK MODIFIED COMPUTATION OF NEW SLP
C
C USAGE     CALL etatsigma(IHINX,IHINY,ETOP)
C   INPUT ARGUMENT LIST:
C     IHIN     - INDEX OF THE H-POINT ON THE INPUT GRID CLOSEST TO 
C                THE H-POINT ON THE OUTPUT GRID
C     ETOP     - PRESSURE AT THE MODEL TOP IN MB
C
C    CALLED FROM PROGRAM INTRST
C
C---------------------------------------------------------------------
c      use mskout
      INCLUDE "parmeta.resin"
      INCLUDE "parmeta.resout"
      INCLUDE "parmsoil"
      INCLUDE "parmeta.hybint"
      INCLUDE "mpif.h"
      INCLUDE "MPI_PARM.comm"
C---------------------------------------------------------------------
C       IMJM=Number of grid points on output eta grid
C       LMI=Number of vertical levels on input eta grid
C       LMO=Number of vertical levels on output eta grid
C
                             P A R A M E T E R
     1 (IM=IMO,JM=JMO
     3, LMIP=LMI+1,LMIM=LMI-1,LMOP=LMO+1,LMOM=LMO-1
     4, KB=JM+2*IM-3)
c     6, LP1=LMO+1,LONG=IMJM*(6*LMO+3)+KB*(4*LMO*2+2)+2*LMO+LMOP)
C
                             P A R A M E T E R
     1 (CM1=2937.4,CM2=4.9283,CM3=23.5518,EPS=0.622
     2, RD=287.04,G=9.806,RG=1./G,EPSQ=2.E-12,EPSQ2=2.E-2
     3, B1=11.87799326,FCTR=0.75)
      parameter(ii=18,jj=52)
C--------------------------------------------------------------------
      PARAMETER(NBC=55)      
                             L O G I C A L
     1 RUN,FIRST,HF,NEST,same
C
                             C H A R A C T E R
     1 LABEL*32
C
C   TEMPRES CONTAINS THE ENTIRE CONTENTS OF THE RESTART FILE
C   AFTER IT IS HORIZONTALLY INTERPOLATED TO THE TARGET OUTPUT
C   ETA GRID BUT BEFORE THE VERTICAL INTERPOLATION IS DONE
C
cwas      REAL,ALLOCATABLE,DIMENSION(:,:,:)::ui,vi,ti,qi,q2i,cldlin
cwas      REAL,ALLOCATABLE,DIMENSION(:,:,:)::dum3do,TO
cwas      REAL,ALLOCATABLE,DIMENSION(:,:,:)::ZETAI,ZETAO
C--------------------------------------------------------------------
                             C O M M O N /DAT/
     1 IDAT(3) 
C-----------------------------------------------------------------------
C
C   OUTETA CONTAINS THOSE VARIABLES VERTICALLY INTERPOLATED TO THE
C   LMO ETA LEVELS OF THE OUTPUT GRID
C
                             C O M M O N /OUTETA/
     1 PDO(IMO,JMO),PDB(KB,2)
     5,TB (KB,LMO,2),QB (KB,LMO,2),UB (KB,LMO,2),VB (KB,LMO,2)
     6,PT,DETAO(LMO),AETAO(LMO),ETAO(LMOP),DFLO(LMOP)
     7,SM(IMO,JMO),RESO(IMO,JMO),FISO(IMO,JMO)
C--------------------------------------------------------------------
C   MSKIN IS THE TERRAIN MASKS FOR THE INPUT GRID
C   HTMI,VTMI : INPUT GRID HEIGHT MASK AT H,V-POINTS (0=Below ground,
C                 1=above ground)
C   LMHI,LMVI : INDEX OF THE LOWEST INPUT GRID ETA LEVEL ABOVE
C                 GROUND
C
                             C O M M O N /MSKIN/
     1 HTMI(IMI,JMI,LMI),VTMI(IMI,JMI,LMI),LMHI(IMI,JMI)
     2 ,LMVI(IMI,JMI)
C
C--------------------------------------------------------------------
C   MSKOUT IS THE TERRAIN MASKS FOR THE OUTPUT GRID
C   HTM0,VTM0 : OUTPUT GRID HEIGHT MASK AT H,V-POINTS (0=Below ground,
C                 1=above ground)
C   LMH0,LMVO : INDEX OF THE LOWEST OUTPUT GRID ETA LEVEL ABOVE
C                 GROUND
C
c                             C O M M O N /MSKOUT/
c     1 HTMO(IMO,JMO,LMO),VTMO(IMO,JMO,LMO),
c     2 LMHO(IMO,JMO),LMVO(IMO,JMO)
C
C--------------------------------------------------------------------
c                             d i m e n s i o n
      common /HYBCOORD/ pdtop,dsg(lmo),sgml(lmo),sgm(lmo+1)
     &,dsg1(lmo),sgml1(lmo),sg1(lmo+1)
     &,dsg2(lmo),sgml2(lmo),sg2(lmo+1)
!----------------------------------------------------------------------
                             D I M E N S I O N
     1  ETAI(LMIP), LDT1(LMO),DFLI(LMIP)
     2, DETAI(LMI),AETAI(LMI),PMVP(LMI)
     3, PINTI(IM,JM,LMIP),PMIDI(IM,JM,LMI)
c     & ,ALPR(LMIP),ALP(LMIP),ALPSQ(LMIP),DLT(LMO)
     4,ZETAI(IM,JSTA:JEND,LMIP),ZETAO(IM,JSTA:JEND,LMOP)
c     6, ZSFCO(IM,JM),REFO(IM,JM),dum3di(im,jm,lmi)
     7, IHINX(IM,JM),IHINY(IM,JM),PDSLI(IM,JM),zsfcoeta(imi,jmi)
     8, LMHOI(IM,JM),HTMOI(IM,JSTA:JEND,LMI)
     9, PSNEW(IM,JM),TLEV(LMIP),LEVATM(IM,JM),ZSFCB(KB)
     +, dum2do(IM,JM)
     1, TO(IM,JSTA:JEND,LMO),QO(IM,JSTA:JEND,LMO)
     2, UO(IM,JSTA:JEND,LMO),VO(IM,JSTA:JEND,LMO)
     3, CWMO(IM,JSTA:JEND,LMO),Q2O(IM,JSTA:JEND,LMO)
     4, ui(IM,JSTA:JEND,LMI),vi(IM,JSTA:JEND,LMI)
     5, ti(IM,JSTA:JEND,LMI),qi(IM,JSTA:JEND,LMI)
     6, q2i(IM,JSTA:JEND,LMI),cldlin(IM,JSTA:JEND,LMI)
C     +, Q2B(KB,LMO,2),CWMB(KB,LMO,2)
Cmp (try more precision?)
        REAL(8) ALPETA, ARGLOG, DLT(LMO), ALPSFC, ARGLG2
        REAL(8) ALPR(LMIP),ALP(LMIP),ALPSQ(LMIP)
        REAL(8) ALPUB,ALPLB, CLOGES, ESE
C-----------------------------------------------------------------------
      INTEGER IUN1,IUN2,IUN3	
      DATA IUN1 /16/,IUN2 /17/,IUN3 /18/
C-----------------------------------------------------------------------
C&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
C-----------------------------------------------------------------------
                         INCLUDE "TEMPRES"
C***
C***  DEFINITION OF CONSTANTS NEEDED FOR VERTICAL INTERPOLATION
C***

      CP=1004.6
      P00=1.E5
      GAMMA=0.0065
      PRF0=101325.
      T0=288.
      GORG=G/(RD*GAMMA)
      RGOG=1./GORG
      ROG=RD/G
      CPOG=CP/G
      CAPA=RD/CP
      PT=ETOP*100.
      ALPT=ALOG(PT)
C-------------------------------------------------------------
      REWIND IUN1
      REWIND IUN2
      REWIND IUN3
!
!  DETAI = ETA(L)-ETA(L-1) FOR THE INPUT GRID
!  ZSFCO = SIGMA HEIGHT ON OUTPUT GRID
!  SM = LAND/SEA MASK ON OUTPUT GRID
!  REFO = VALUE OF SIGMA AT THE SURFACE ON THE OUTPUT GRID=1
!
      READ(IUN1)DETAI
cwas      READ(IUN2)DETAO
c      
  11  FORMAT(I3,F13.9)    
C---------------------------------------------------------------------
C***
C***  FOR A ETA-TO-ETA
C***
c     READ(IUN3)FISO,SM
c     READ(IUN3)RESO
C---------------------------------------------------------------------
C***
C***  FOR A ETA-TO-HYBRID
C***
      READ(IUN3)FISO,DUM2DO,SM
      if(mype.eq.0)print*,'Debug:Terrain height      ',fiso(ii,jj+1)
      if(mype.eq.0)print*,'Debug:Terrain height ',fiso(ii-1,jj)
     +,fiso(ii,jj),fiso(ii+1,jj)
      if(mype.eq.0)print*,'Debug:Terrain height     ',fiso(ii,jj-1)
      if(mype.eq.0)print*,'Debug:SM      ',sm(ii,jj+1)
      if(mype.eq.0)print*,'Debug:SM ',sm(ii-1,jj),sm(ii,jj)
     +,sm(ii+1,jj)
      if(mype.eq.0)print*,'Debug:SM     ',sm(ii,jj-1) 
C---------------------------------------------------------------------
CCCC
C      WRITE(6,13)(ZSFCO(207,j),j=1,20) 
C      WRITE(6,13)(ZSFCO(I,20),I=11,20)
 13   format(10f8.1)     
CCCCC rewind 19
CCCCC read(19)zsfcoeta
c      do i=1,imi
c       do j=1,jmi
c        if(abs(resi(i,j)-resii(i,j)).gt.0.001)print*,'incon resi at'
c     1,i,j,resi(i,j),resii(i,j)
c       end do
c      end do   	
C
c      PRINT*,'SAMPLE I & OUTPUT TERRAIN BEFORE BUFFER MODIFICATION'
      I=207
c      DO J=50,1,-1
c       print*,'I,J,ZSFCOETA,ZSFCO= ',I,J,ZSFCOETA(I,J),ZSFCO(I,J) 
c      END DO
ccc obtaining eta sea mask from eta nhb file instead
c      call readnhb(smeta)
ccc      
CCC MODIFY BUFER TERRAIN BECAUSE FIELDS AT THESE POINTS ARE 
CCC CALCULATED AS 4 POINT AVERAGE IN THE MODEL
c     IF(MOD(JSTA,2).LT.1)THEN
c      JJ=JSTA
c     ELSE
c      JJ=JSTA+1
c     END IF  
c     DO J=JJ,JEND,2
c      I=IM-1
c      FISO(I,J)=0.25*(FISO(I,J-1)+FISO(I+1,J-1)+
c    + FISO(I+1,JJ+1)+FISO(I,J+1))
c     if(j.eq.54)then
c       write(0,*)' new avg fiso=',fiso(i,j)
c       write(0,*)' avg fiso=',FISO(I,J-1),FISO(I+1,J-1)
c    1,                        FISO(I+1,JJ+1),FISO(I,J+1)
c     endif
c      I=1
c      FISO(I,J)=0.25*(FISO(I,J-1)+FISO(I+1,J-1)+
c    + FISO(I+1,J+1)+FISO(I,J+1))
c     END DO 
c     IF(MYPE.EQ.0)THEN
c      DO I=2,IM-2
c       J=2
c       FISO(I,J)=0.25*(FISO(I,J-1)+FISO(I+1,J-1)+
c    +  FISO(I+1,J+1)+FISO(I,J+1))
c      END DO
c     END IF 
c     IF(MYPE.EQ.(NUM_PROCS-1))THEN
c      DO I=2,IM-2
c       J=JM-1
c       FISO(I,J)=0.25*(FISO(I,J-1)+FISO(I+1,J-1)+
c    +  FISO(I+1,J+1)+FISO(I,J+1))
c      END DO
c     END IF 
CCC
C
      I=207
C      DO J=50,1,-1
C       print*,'I,J,ZSFCO= ',I,J,ZSFCO(I,J) 
C      END DO
C
CC WRITING BOUNDARY TERRAIN FOR MAKING BOUNDARY TENDENCIES 
      CALL COLLECT(FISO)
      IF(MYPE.EQ.0)THEN
       N=1
       DO I=1,IMO
        ZSFCB(N)=FISO(I,1)
        N=N+1
       END DO
       DO I=1,IMO
        ZSFCB(N)=FISO(I,JM)
        N=N+1
       END DO
       DO J=3,JMO-2,2
        ZSFCB(N)=FISO(1,J)
        N=N+1
       END DO
       DO J=3,JMO-2,2
        ZSFCB(N)=FISO(IM,J)
        N=N+1
       END DO 
       WRITE(60)ZSFCB
      END IF 
CCCCC     
      if(mype.eq.0)write(6,*) 'read from IUN1-3'
c MAKE SURE REFO=1 LMHO=LMO HTMO=1 FOR SIGMA COORDINATE
c       ALLOCATE(LMHO(IM,JSTA:JEND))
c       ALLOCATE(HTMO(IM,JSTA:JEND,LMO))
        DO J=JSTA,JEND
	 DO I=1, IMO
	  RESO(I,J)=1.0
c          LMHO(I,J)=LMO
c	  LMVO(I,J)=LMO
c	  DO L=1,LMO
c	    HTMO(I,J,L)=1
c	    VTMO(I,J,L)=1
c	  END DO
	 END DO
	END DO    	  
C
C     CONVERT REFO TO RESO FOR CNSTS AND ZSFCO TO FIS
C   
      DO J=JSTA,JEND
       DO I=1,IM
CC SET OUTPUT TERRAIN HEIGHT TO BE ZERO AT THE BOUNDARIES
C        IF(I.EQ.1 .OR. J.EQ.1 .OR. I.EQ.IM .OR. J.EQ.JM
C     +  .OR.(MOD(J,2).EQ.0.AND.I.EQ.(IM-1)))then
C         zsfco(i,j)=0.0
c         PRINT*,'TERRAIN HT AT BD= ',ZSFCO(I,J)
C	END IF
C	RESO(I,J)=REFO(I,J)
        FISO(I,J)=FISO(I,J)*G
c     if(j.eq.54.and.(i.eq.176.or.i.eq.178))then
c       write(0,*)' to geopot i=',i,' fiso=',fiso(i,j)
c     endif
       END DO	
      ENDDO
!-----------------------------------------------------------------------
!***
!***  compute etai at the interfaces of the model layers
!***
      etai(1)=0.
          do l=2,lmi+1
      etai(l)=etai(l-1)+detai(l-1)
          enddo
      etai(lmi+1)=1.
      detai(lmi)=1.-etai(lmi)
!***
!***  compute aetai in the middles of the layers
!***
          do l=1,lmi
      aetai(l)=0.5*(etai(l)+etai(l+1))
          enddo
!-----------------------------------------------------------------------
!23456789112345678921234567893123456789412345678951234567896123456789712
!  This is the first time the hybrid coordinate appears in the job
!  stream, and that is why it is defined here.  Away from the mountains,
!  the depths of the layers may be the same as those of the input sigma
!  coordinate, or may be defined independently.
!-----------------------------------------------------------------------
      same=.true.
              if(same) then
!-----------------------------------------------------------------------
      READ(IUN2)DSG
      DO L=1,LMO
       DETAO(L)=DSG(L)
      END DO
      SGM(1)=0.0
      DO L=2,LMO+1
       SGM(L)=SGM(L-1)+DSG(L-1)
       ETAO(L)=SGM(L)
      END DO
      SGM(LMO+1)=1.0
      ETAO(LMO+1)=1.0
      DSG(LMO)=1.0-SGM(LMO)
      DETAO(LMO)=1.0-ETAO(LMO)
      DO L=1,LMO
       SGML(L)=0.5*(SGM(L)+SGM(L+1))
       AETAO(L)=SGML(L)
      END DO 
c          do l=1,lmo
c      dsg(l)=detai(l)
c      sgml(l)=aetai(l)
c      detao(l)=detai(l)
c      aetao(l)=aetai(l)
c          enddo
c          do l=1,lmo+1
c      sgm(l)=etai(l)
c      etao(l)=etai(l)
c          enddo
!-----------------------------------------------------------------------
              else
!-------computing of sg levels according to empirical formula-----------
      x0=0.10*pi
      x1=(1.300)*pi
      dx=(x1-x0)/(lmo-1)
!
      aa=1.
      bb=-(aa-1.)/(x1-x0)
!
      x=x0-dx
      sum=0.
          do l=1,lmo
      x=x+dx
      dsgl=aa+bb*(x-x0)+sin(x)
      dsg(l)=dsgl
      sum=sum+dsgl
          enddo
      rsum=1./sum
      sgm(1)=0.
          do l=1,lmo-1
      dsg(l)=dsg(l)*rsum
      sgm(l+1)=sgm(l)+dsg(l)
          enddo
      sgm(lmo+1)=1.
      dsg(lmo)=sgm(lmo+1)-sgm(lmo)
          do l=1,lmo
      sgml(l)=(sgm(l)+sgm(l+1))*0.5
          enddo
!-----------------------------------------------------------------------
              endif
!-----------------------------------------------------------------------
!  ptsgm is the approximate pressure at the boundary between pressure 
!  and sigma in Pa.
!
!  2/8/02 : Now set in parameter file parmeta.hybint
!-----------------------------------------------------------------------
!
!     ptsgm=42000  ! Pa
      lpt2=0
          do l=1,lmo+1
      pl=sgm(l)*(101300.-pt)+pt
      if(pl.lt.ptsgm) lpt2=l
          enddo
          if(lpt2.gt.0) then
      pt2=sgm(lpt2)*(101300.-pt)+pt
          else
      pt2=pt
          endif
      if(mype.eq.0) then
       print*,'*** Sigma system starts at ',pt2,' Pa, from level ',lpt2
      endif
      pdtop=pt2-pt
          do l=1,lmo
          if(l.le.lpt2) then
      dsg1(l)=dsg(l)
          else
      dsg2(l)=dsg(l)
          endif
          enddo
!-----------------------------------------------------------------------
      sg1(1)=0.
          if(lpt2.gt.0) then
          do l=1,lpt2
      sg1(l+1)=sg1(l)+dsg1(l)
          enddo
          do l=2,lpt2
      sg1(l)=sg1(l)/sg1(lpt2+1)
          enddo
      sg1(lpt2+1)=1.
          do l=lpt2+2,lmo+1
      sg1(l)=1.
          enddo
          do l=1,lpt2
      dsg1(l)=sg1(l+1)-sg1(l)
      sgml1(l)=(sg1(l)+sg1(l+1))*0.5
          enddo
          endif
          do l=lpt2+1,lmo
      dsg1(l)=0.
      sgml1(l)=1.
          enddo
!
          do l=1,lpt2+1
      sg2(l)=0.
          enddo
          do l=lpt2+1,lmo
      sg2(l+1)=sg2(l)+dsg2(l)
          enddo
          do l=lpt2+2,lmo
      sg2(l)=sg2(l)/sg2(lmo+1)
          enddo
      sg2(lmo+1)=1.
          do l=lpt2+1,lmo
      dsg2(l)=sg2(l+1)-sg2(l)
      sgml2(l)=(sg2(l)+sg2(l+1))*0.5
          enddo
          do l=1,lpt2
      dsg2(l)=0.
      sgml2(l)=0.
          enddo
!-----------------------------------------------------------------------
      if(mype.eq.0)then
      write(*,*)'dsg=', dsg
      write(*,*)'sgm=', sgm
      write(*,*) 'sgml= ',sgml
      write(*,*) 'dsgl= ',dsg1
      write(*,*) 'sgl= ',sg1
      write(*,*) 'sgml1= ',sgml1
      write(*,*) 'dsg2= ',dsg2
      write(*,*) 'sg2= ',sg2
      write(*,*) 'sgml2= ',sgml2
      end if
!zj      stop
!-----------------------------------------------------------------------
      if(mype.eq.0)then
      open(unit=8,file='deta_hyb'
     &    ,status='unknown',form='unformatted')
      write(8) dsg,sgm,pdtop,dsg1,sg1,dsg2,sg2
      close(8)
      end if
C----------------------------------------------------------------
C***
C***  COMPUTE THE HEIGHTS OF THE STANDARD INTERFACES
C***  FOR THE INPUT DOMAIN
C***
      DO L=1,LMIP
        DFLI(L)=G*T0*(1.-((PT+ETAI(L)*(PRF0-PT))/PRF0)**RGOG)/GAMMA
      ENDDO
C***
C***  COMPUTE THE LMH (INDEX OF LOWEST MODEL LEVEL ABOVE GROUND)
C***  FOR THE INPUT GRID VERTICAL LEVELS ON THE OUTPUT GRID POINTS
C***  BASED SOLEY UPON WHETHER OR NOT ANY PART OF AN OUTPUT
C***  LAYER IS ABOVE GROUND IN ANY INPUT LAYER
C***  USE INPUT GRID TEMPERATURES WHICH ARE ALWAYS < 0 BELOW GROUND
C***
cwas      open(80,file='ti.out',form='unformatted',status='unknown')
cwas      do l=1,lmi 
cwas       read(80)((DUM3DI(I,J,L),I=1,IM),J=1,JM)
cwas      end do
cwas      close(80)
C
CC      ALLOCATE(TI(IMO,JSTA:JEND,LMI))
CC      ALLOCATE(QI(IMO,JSTA:JEND,LMI))
CC      ALLOCATE(UI(IMO,JSTA:JEND,LMI)) 
CC      ALLOCATE(VI(IMO,JSTA:JEND,LMI))
CC      ALLOCATE(Q2I(IMO,JSTA:JEND,LMI))
CC      ALLOCATE(cldlin(IMO,JSTA:JEND,LMI))
      if(mype.eq.0)PRINT*,'H FLAG= ',HF
      IF(HF)THEN
       DO J=JSTA,JEND
       DO I=1,IM
        LMHOI(I,J)=LMHI(I,J)
       END DO
       END DO
      ELSE
       DO 50 J=JSTA,JEND
        DO 50 I=1,IM
         LMHOI(I,J)=LMI
         DO L=2,LMI
c     if(mype.eq.0.and.i.eq.im.and.j.eq.69)then
c       write(0,*)' l=',l,' htmi=',htmi(i,j,l)
c    1,           ' ti=',ti(i,j,l),' etai=',etai(l)
c     endif
          IF(TI(I,J,L).LT.1.)THEN
           LMHOI(I,J)=L-1
c     if(mype.eq.0.and.i.eq.im.and.j.eq.69)then
c       write(0,*)' in loop lmhoi=',lmhoi(i,j)
c     endif
           GO TO 50
          ENDIF
         ENDDO
   50  CONTINUE
      END IF
CCCCC
CCCCC THE FOLLOWING SPECIFICATIONS FOR LMHOI WAS DONE BECAUSE 
CCCCC UNDERGROUND TI WAS NOT EQUAL TO ZERO IN ICS
c      DO I=1,IM
c       DO J=1,JM
c       LMHOI(I,J)=LMHI(I,J)     
c       END DO
c      END DO 
CCCCC
CCC TEST ! USE DIFFERENT WAYS TO COMPUTE LMHOI
c      DO I=1,IM
c       DO J=1,JM
c        PINTI(I,J,1)=PT
c       END DO
c      ENDDO
C
c      DO L=2,LMIP
c        DO I=1,IM
c	 DO J=1,JM
c          PINTI(I,J,L)=PDSLI(I,J)*ETAI(L)+PT
c         END DO
c        ENDDO
c      ENDDO
c      DO J = 1, JM
c       DO I = 1, IM
c         DO K = LMIP,1,-1
c	  PSTAR=PT+PDI(I,J)
c          IF(PSTAR .GE. PINTI(I,J,K)) THEN
c            PDIFF = PSTAR - PINTI(I,J,K)
c            LMHOI(I,J) = K
c            IF(PDIFF.LE.7.5) LMHOI(I,J) = K - 1	    
c	    GO TO 52 
c          ENDIF
c         ENDDO
c 52      CONTINUE
c       ENDDO
c      ENDDO 
C
      IF(HF)THEN
       DO J=JSTA,JEND
        DO I=1,IM
	 DO L=1,LMI
	  HTMOI(I,J,L)=HTMI(I,J,L)
	 END DO
	END DO
       END DO
      ELSE 	        
       DO 60 J=JSTA,JEND
        DO 60 I=1,IM
         DO L=2,LMIP
          HTMOI(I,J,L-1)=1.
          IF((FISO(I,J)-DFLI(L)).GT.1.E-2.AND.DFLI(L).GT.1.E-2)THEN
           DO LL=L,LMI
             HTMOI(I,J,LL)=0.
           ENDDO
           GO TO 60
          ENDIF
         ENDDO
   60  CONTINUE
      END IF  
C
C----------------------------------------------------------------
C***
C***  COMPUTE HEIGHTS OF THE INPUT ETA INTERFACES
C***
cwas      allocate(ZETAI(IM,JM,LMIP))
      DO J=JSTA,JEND
       DO I=1,IM
        PDSLI(I,J)=RESI(I,J)*PDI(I,J)    
c     if(j.eq.401.and.i.ge.1.and.i.le.36)then
c       write(0,*)' i=',i+179,' pdsli=',pdsli(i,j)
c    1,           ' resi=',resi(i,j),' pdi=',pdi(i,j)
c     endif
       END DO
      ENDDO
c      PRINT*,'LMHISM,AND ZSFC(207,7)= ',LMHI(207,7),SM(207,7)
c     + ,ZSFCO(207,7)
      
C       
C***
C***  IF THE OUTPUT POINT IS ABOVE GROUND BUT THE INPUT POINT
C***  IS BELOW GROUND THEN:
C***    (1) IF ADJACENT INPUT POINTS ARE ABOVE GROUND THEN USE
C***        A WEIGHTED AVERAGE OF THOSE ABOVE-GROUND INPUT
C***        TEMPERATURES
C***    (2) IF NO ADJACENT INPUT POINTS ARE ABOVE GROUND THEN
C***        EXTRAPOLATE DOWNWARD FROM THE TWO INPUT LAYERS
C***        ABOVE GROUND AND USE AN ARBITRARY FACTOR TO CHANGE THE
C***        EXTRAPOLATION FROM PURELY LINEAR (FCTR=1 IS LINEAR)
C***
      DO L=1,LMI
         AETAI(L)=0.5*(ETAI(L)+ETAI(L+1))
      ENDDO
CC EXTRAPOLATE INPUT TEMPERATURE DOWNWARD 1 TO 3 LAYERS IN CASE 
CC SIGMA TERRAIN FALLS UNDER ETA TERRAIN
      DO J=JSTA,JEND
       DO I=1,IM
        LMHIK=LMHOI(I,J)
	IF(LMHIK .LT. LMI)THEN
c        IF((FISO(I,J)/g).LE.ZSFCOETA(I,J))then
	 APEL1=ALOG(AETAI(LMHIK-5)*PDSLI(I,J)+PT)
	 APEL2=ALOG(AETAI(LMHIK)*PDSLI(I,J)+PT)
	 LEND=MIN(LMI,LMHIK+4)
         DO L=LMHIK+1,LEND
	  APEL3=ALOG(AETAI(L)*PDSLI(I,J)+PT)
	  TI(I,J,L)=TI(I,J,LMHIK)+(TI(I,J,LMHIK)-
     1           TI(I,J,LMHIK-5))*(APEL3-APEL2)/(APEL2-APEL1)
C USE STANDARD LAPSE RATE TO EXTRAPOLATE TEMPERATURE
cc         TI(I,J,L)=TI(I,J,L-1)+((AETAI(L)-AETAI(L-1))*PDSLI(I,J))
cc     1   *TI(I,J,L-1)/GORG/(PT+ETAI(L)*PDSLI(I,J))	          
          HTMOI(I,J,L)=1.0	  
         END DO
        END IF
       END DO
      END DO 	
CC
CCC TEST!! USE DIFFERENT METHOD TO CALCULATE INPUT HEIGHT
C      DO 59 I=1,IM
C       DO 59 J=1,JM
C        LMHIK=LMHOI(I,J)
C        ZETAI(I,J,LMHIK+1)=ZSFCOETA(I,J)
CC ABOVE GROUND
C        DO L=LMHIK,1,-1
C	 APEL3=AETAI(L)*PDSLI(I,J)+PT
C	 DPDE=DETAI(L)*PDSLI(I,J)
C	 IF(HF)THEN
C	  HWGT=HTMOI(I,J,L)
C         ELSE
C	  IF(MOD(IHINY(I,J),2) .LT. 1)THEN
C           HWGT=HTMI(IHINX(I,J)-1,IHINY(I,J),L)
C     1	      +HTMI(IHINX(I,J),IHINY(I,J)+1,L)
C     2        +HTMI(IHINX(I,J),IHINY(I,J),L)
C     3        +HTMI(IHINX(I,J),IHINY(I,J)-1,L)
C          ELSE
C	   HWGT=HTMI(IHINX(I,J),IHINY(I,J),L)
C     1	      +HTMI(IHINX(I,J),IHINY(I,J)+1,L)
C     2        +HTMI(IHINX(I,J)+1,IHINY(I,J),L)
C     3        +HTMI(IHINX(I,J),IHINY(I,J)-1,L)
C          END IF
C         END IF
C	 IF(HWGT.LT.0.5 .AND.HTMOI(I,J,L).GT.0.5)THEN
C          APEL1=AETAI(LMHIK-1)*PDSLI(I,J)+PT
C          APEL2=AETAI(LMHIK)*PDSLI(I,J)+PT
C          TI(I,J,L)=TI(I,J,LMHIK)+(TI(I,J,LMHIK)-TI(I,J,LMHIK-1))*
C     1            (APEL3-APEL2)/(APEL2-APEL1)*FCTR
C	 ENDIF
C	 RTOP=RD*TI(I,J,L)*(1.+0.608*QI(I,J,L))/APEL3
C         FI=G*ZETAI(I,J,L+1)+RTOP*DPDE
C	 ZETAI(I,J,L)=FI*RG 	 
C	END DO 
CCC BELOW GROUND, ASSUME TEMPERATURE CHANGE WITH STANDARD LAPSE RATE
C        IF(LMHIK.EQ.LMI)GO TO 59
C	PSFC=PT+PDI(I,J)
C	PSEA=PT+PDSLI(I,J)
C	PLMHIK=PT+PDSLI(I,J)*AETAI(LMHIK)
C	TMEAN=G*ZSFCOETA(I,J)/(RD*(LOG(PSEA)-LOG(PSFC)))
C	PMEAN=(PSFC+PSEA)/2.0
C        DO L=LMHIK+2,LMI+1  
C         PETA=PT+PDSLI(I,J)*AETAI(L-1)
C	 DPDE=DETAI(L-1)*PDSLI(I,J)
C	 IF(PETA .LE. PMEAN)THEN
C	  TI(I,J,L-1)=TI(I,J,LMHIK)+(TMEAN-TI(I,J,LMHIK))*(LOG(PETA)
C     1    -LOG(PLMHIK))/(LOG(PMEAN)-LOG(PLMHIK))
C         ELSE
C	  PETA2=PT+PDSLI(I,J)*AETAI(L-2)
C	  PETA3=PT+PDSLI(I,J)*AETAI(L-3)
C	  TI(I,J,L-1)=TI(I,J,L-2)+(TI(I,J,L-2)-TI(I,J,L-3))*(LOG(PETA)
C     1    -LOG(PETA2))/(LOG(PETA2)-LOG(PETA3))
C         END IF
CC	 TI(I,J,L-1)=TMEAN	  	   
C	 RTOP=RD*TI(I,J,L-1)*(1.+0.608*QI(I,J,LMHIK))/PETA
C	 FI=G*ZETAI(I,J,L-1)-RTOP*DPDE
C	 ZETAI(I,J,L)=RG*FI
CC DIFFERENT WAY TO CALCULATE UNDERGROUND ZETAI
cC         ZETAI(I,J,L)=ZETAI(I,J,L-1)-ZSFCOETA(I,J)*(LOG(PT+PDSLI(I,J)*
cC     1	 ETAI(L))-LOG(PT+PDSLI(I,J)*ETAI(L-1)))/(LOG(PSEA)-LOG(PSFC))
C	 if(l.eq.(lmi+1).and.abs(zetai(i,j,l)).gt.10.)
C     1   print*,'nonzero 1st level H ',i,j
C     2,   psfc,pt+pdsli(i,j)*etai(lmhik+1),psea,pt+pdsli(i,j)*etai(l)
C     3,  zetai(i,j,l),zsfcoeta(i,j),zsfco(i,j),resi(i,j)
C     4  ,1./etai(lmhik+1) 
C	END DO 
C 59   CONTINUE	
CCC
CCC
C USE PMIDI FOR QI FOR NOW BECAUSE PMIDI IS NOT USED TILL LATE
cwas      ALLOCATE(PMIDI(IM,JM,LMI))
C USE PMIDI FOR QI FOR NOW BECAUSE PMIDI IS NOT USED TILL LATE
CWAS      open(80,file='qi.out',form='unformatted',status='unknown')
CWAS      do l=1,lmi 
CWAS       read(80)((PMIDI(I,J,L),I=1,IM),J=1,JM)
CWAS      end do
CWAS      close(80)
C            
      DO L=LMI,1,-1
        DO J=JSTA,JEND
	 DO I=1,IM
	  ZETAI(I,J,LMIP)=0.0
	  APEL3=AETAI(L)*PDSLI(I,J)+PT
	  DPDE=DETAI(L)*PDSLI(I,J)
	  IF(HF)GO TO 61
	  IF(IHINY(I,J).EQ.1 .OR. IHINY(I,J).EQ.JMI)THEN  !assume jmi odd
           HWGT=HTMI(IHINX(I,J),IHINY(I,J),L)
     1          +HTMI(IHINX(I,J)+1,IHINY(I,J),L)
          ELSE IF(IHINX(I,J).EQ.1 .OR. IHINX(I,J).EQ.IMI)THEN
           HWGT=HTMI(IHINX(I,J),IHINY(I,J)+1,L)
     1          +HTMI(IHINX(I,J),IHINY(I,J)-1,L)
	  ELSE IF(MOD(IHINY(I,J),2) .LT. 1)THEN
            HWGT=HTMI(IHINX(I,J)-1,IHINY(I,J),L)
     1	      +HTMI(IHINX(I,J),IHINY(I,J)+1,L)
     2        +HTMI(IHINX(I,J),IHINY(I,J),L)
     3        +HTMI(IHINX(I,J),IHINY(I,J)-1,L)
          ELSE
	    HWGT=HTMI(IHINX(I,J),IHINY(I,J),L)
     1	      +HTMI(IHINX(I,J),IHINY(I,J)+1,L)
     2        +HTMI(IHINX(I,J)+1,IHINY(I,J),L)
     3        +HTMI(IHINX(I,J),IHINY(I,J)-1,L)
          END IF
	  LMHIK=LMHOI(I,J)
	  IF(HWGT.LT.0.5 .AND.HTMOI(I,J,L).GT.0.5)THEN
            APEL1=AETAI(LMHIK-5)*PDSLI(I,J)+PT
            APEL2=AETAI(LMHIK)*PDSLI(I,J)+PT
            TI(I,J,L)=TI(I,J,LMHIK)+(TI(I,J,LMHIK)-
     1      TI(I,J,LMHIK-5))*(APEL3-APEL2)/(APEL2-APEL1)*FCTR
	  ENDIF	  
 61       CONTINUE	  
	  RTOP=RD*TI(I,J,L)*(1.+0.608*QI(I,J,L))/APEL3
          FI=G*ZETAI(I,J,L+1)+RTOP*DPDE
          IF(HWGT.GT.0.5)HWGT=1.
          ZETAI(I,J,L)=((FI-DFLI(L))*HTMOI(I,J,L)+DFLI(L))*RG	  
         END DO
	ENDDO
      ENDDO	
C      print*,'sample input height'
C      print*,(zetai(50,50,l),l=1,lmo)
C***
C***  FIND THE PRESSURE ON THE INPUT INTERFACES
C***
cwas      ALLOCATE(PINTI(IM,JM,LMIP))
      DO J=JSTA,JEND
       DO I=1,IM
        PINTI(I,J,1)=PT
       END DO
      ENDDO
C
      DO L=2,LMIP
        DO J=JSTA,JEND
	 DO I=1,IM
          PINTI(I,J,L)=PDSLI(I,J)*ETAI(L)+PT
         END DO
        ENDDO
      ENDDO
C***
C***  FIND THE PRESSURE AT THE INPUT MIDLAYER LOCATIONS
C***
      DO L=1,LMI
        DO J=JSTA,JEND
	 DO I=1,IM
          PMIDI(I,J,L)=PINTI(I,J,L)+0.5*DETAI(L)*PDSLI(I,J)
         END DO
	ENDDO
      ENDDO
C  RECOMPUTE SURFACE PRESSURE BASED ON SIGMA TERRAIN HEIGHT
C  FIRST, FIRST PRESSURE LEVEL IN THE ETA ANALYSIS ABOVE GROUND
C  IF SURFACE IS < 7.5 MB FROM PRESSURE SURFACE
C  USE THE NEXT HIGHEST ONE TO PREVENT EXTRAPOLATION
C
      DO J=JSTA,JEND
       DO I = 1, IM
         DO K = LMIP,1,-1
	  PSTAR=PT+PDI(I,J)
          IF(PSTAR .GE. PINTI(I,J,K)) THEN
            PDIFF = PSTAR - PINTI(I,J,K)
            LEVATM(I,J) = K
            IF(PDIFF.LE.7.5) LEVATM(I,J) = K - 1
c	    IF(LMHOI(I,J).NE.LEVATM(I,J))PRINT*,'INCON MASK ',
c     +      i,j,LMHOI(I,J),LEVATM(I,J) 	    
C            IF(MOD(I,10).EQ.0.AND.MOD(J,10).EQ.0)
C     +      PRINT*,'I,J,PSTAR,PINTI,LEVATM= ',
C     +      I,J,PSTAR,PINTI(I,J,K),LEVATM(I,J)
	    GO TO 65 
          ENDIF
         ENDDO
 65      CONTINUE
       ENDDO
      ENDDO
C
      DO 180 J=JSTA,JEND
       DO 180 I=1,IM
        DO L=1,LMIP
	 ALP(L)=LOG(PINTI(I,J,L)) 	 
	END DO 
	PSTAR=PT+PDI(I,J)
        KSTART = LEVATM(I,J)
	DO K = 1,KSTART
         TLEV(K) = TI(I,J,K)
        ENDDO
        K = LEVATM(I,J)
 170    CONTINUE
C        PLO = EPLEV(K)
        ZLO = ZETAI(I,J,K)        
        TLO = TLEV(K)	 
        K = K - 1
        IF((FISO(I,J)/G) .GT. ZETAI(I,J,K)) GO TO 170
        ALNPB = ALP(K+1)
        ALNPT = ALP(K)
        DLNP = ALNPB - ALNPT
        ALNP3 = 0.5*(ALNPB+ALNPT) 
        A = G * (ZETAI(I,J,K) - ZLO) / DLNP
        B = RD * (TLO - TLEV(K)) / DLNP
        PHI3 = G*0.5*(ZLO+ZETAI(I,J,K)) + 0.125*B*DLNP*DLNP
        RADICL = A*A - 2.*B*(FISO(I,J) - PHI3) 
        IF(RADICL.LE.0.0 ) THEN
          WRITE(6,120) RADICL,I,J,A,B,FISO(I,J)/g,K
  120     FORMAT(' radicl<0,I,J,A,B,ZS,K ',G15.5,2I4,3G15.5,I5)
          GO TO 172
        ENDIF
        ALNPS = ALNP3-2.*(FISO(I,J)-PHI3) /
     1      (A + SQRT(RADICL))
        PSNEW(I,J) = EXP(ALNPS)	
c     if(j.eq.401.and.i.ge.1.and.i.le.36)then
c       write(0,*)' A i=',i+179,' psnew=',psnew(i,j)
c     endif
c     if(mype.eq.0.and.i.eq.im.and.j.eq.69)then
c       write(0,*)' A psnew=',psnew(i,j),' fiso=',fiso(i,j)
c    1,           ' phi3=',phi3
c       write(0,*)' radicl=',radicl,' a=',a,' alnp3=',alnp3
c     endif
C        TSNEW(I,J) = TLO - (ALNPB-ALNPS)*B/R
        GO TO 174
C
C   NGM TERRAIN IS LOWER THAN ETA TERRAIN
C         
C 2500    PRINT *,'SIGMA IS LOWER THAN ETA ',I,J,ZSFCO(I,J),
C     1     PSTAR
 172    PSNEW(I,J)=EXP(ALP(K+1)-(ALP(K)-ALP(K+1))*(ZETAI(I,J,K+1)
     +  -(FISO(I,J)/G))/(ZETAI(I,J,K)-ZETAI(I,J,K+1))) 
c     if(j.eq.401.and.i.ge.1.and.i.le.36)then
c       write(0,*)' B i=',i+179,' psnew=',psnew(i,j)
c     endif
c     if(mype.eq.0.and.i.eq.im.and.j.eq.69)then
c       write(0,*)' B j=',j,' psnew=',psnew(i,j),' fiso=',fiso(i,j)
c       write(0,*)' zeta=',zetai(i,j,k),zetai(i,j,k+1)
c       write(0,*)' alp=',alp(k),alp(k+1)
c     endif
C 172    PSNEW(I,J) = PSTAR 
        PRINT*,'NEW TERRAIN IS LOWER THAN OLD TERRAIN'
	PRINT*,'I,J,ETA SIGMA PSFC= ',I,J,PSTAR,PSNEW(I,J),MYPE
 174    CONTINUE
C
CCCCCCC TEST... FORCE PDO=PDI AT THE BOUNDARIES   CCCCCCCCCC
C        IF((I.EQ.1.AND.MOD(J,2).EQ.1) .OR. J.EQ.1 .OR. 
C     +	(I.EQ.IM.AND.MOD(J,2).EQ.1) .OR. J.EQ.JM)PRINT*,
C     +  'SIGMA PS= ',PSNEW(I,J)	 
C	IF((I.EQ.1.AND.MOD(J,2).EQ.1) .OR. J.EQ.1 .OR. 
C     +	(I.EQ.IM.AND.MOD(J,2).EQ.1) .OR. J.EQ.JM)THEN
C	 PSNEW(I,J) = PDSLI(I,J)+PT
C         PRINT*,'ETA PS= ',PSNEW(I,J)
C	END IF
CCCC
  180 CONTINUE	
C
C-------------------------------------------------------------
C***
C***  VERTICAL INTERPOLATION: QUADRATIC INTERPOLATION OF
C***  HEIGHTS (EQUIVALENT TO TEMPERATURE BEING LINEAR IN LN P)
C***  AND LINEAR INTERPOLATION OF WINDS
C***
C-------------------------------------------------------------
C***
C***  COMPUTATION OF THE 'SEA LEVEL' PRESSURE DIFFERENCE
C***
      DO 200 J=JSTA,JEND
       DO 200 I=1,IM
C
       DO LD=1,LMI
        ALPR(LD)=ALOG(PINTI(I,J,LD+1)/PINTI(I,J,LD))
       ENDDO
C
       DO LD=1,LMIP
        ALP(LD)=ALOG(PINTI(I,J,LD))
        ALPSQ(LD)=ALP(LD)**2
       ENDDO
C
C***
C***  ASSIGN THE VALUE OF PRESSURE DIFFERENCE AT THE NEAREST  
C***  H POINT IN THE INPUT DOMAIN TO PD IN THE OUTPUT DOMAIN
C***  WHERE PD IS EQUAL TO P AT ETA=1 MINUS PT.  
C***
!  the following line is uded when using sigma output terrain
      pdij=psnew(i,j)-pdtop-pt
c     if(mype.eq.0.and.i.eq.im.and.j.eq.69)then
c       write(0,*)' 1st i=',i,' pdo=',pdij
c       write(0,*)' lmhoi=',lmhoi(i,j),' pdi=',pdi(i,j)
c    1,           ' pdtop=',pdtop,' psnew=',psnew(i,j)
c     endif
!  if input eta terrain already on sea level and pdo>pdi =>pdi=pdo
ccccc     if(lmhoi(i,j).eq.lmi.and.fiso(i,j).lt.1.0) then
          if(fisi(i,j).lt.1.0.and.fiso(i,j).lt.1.0) then
      pdij=pdi(i,j)-pdtop
c     if(mype.eq.0.and.i.eq.im.and.j.eq.69)then
c       write(0,*)' 2nd i=',i,' pdo=',pdij
c     endif
          end if
c     if(j.eq.401.and.i.ge.1.and.i.le.36)then
c       write(0,*)' i=',i+179,' pdij=',pdij,' fisi=',fisi(i,j)
c    1,           ' fiso=',fiso(i,j)
c     endif
C----------------------------------------------------------------------
C***
C***  COMPUTATION OF HEIGHTS AT UPPER BOUNDARIES OF ETA LAYERS 
C***  AND TEMPERATURE AND SPEC HUMIDITY AT MIDDLE OF ETA LAYERS
C***
C------------------------------------------------------
C***  THE FOLLOWING LINE IS ONLY TRUE IF PT (PRESSURE AT MODEL TOP IN
C***  PASCALS) IS THE SAME FOR BOTH THE INPUT AND THE OUTPUT
C------------------------------------------------------
      zetao(i,j,1)=zetai(i,j,1)
!
      kntin=1
      peta=pt
      ldt1(1) = 1
!***
!***  LDT1 HOLDS THE INDEX OF THE INPUT ETA INTERFACE THAT IS
!***  DIRECTLY ABOVE THE OUTPUT ETA INTERFACE IN QUESTION.
!***
          do 100 l=2,lmo
      peta=sg1(l)*pdtop+sg2(l)*pdij+pt
   90 continue
          if(peta.gt.pinti(i,j,kntin).and.peta.le.pinti(i,j,kntin+1)
     &      .or.peta.lt.pinti(i,j,2).or.kntin.ge.lmim) then
      ldt1(l)=kntin 
      go to 100
          endif
      kntin=kntin+1
      go to 90
  100     continue
!
          do l=1,lmo
      l1=ldt1(l)
      dlt(l)=alpr(l1+1)*(-alpr(l1))*ALOG(PINTI(I,J,L1+2)/PINTI(I,J,L1))
          enddo
! start height interpolation from bottom up
      l =lmo
!
  125 l1=ldt1(l)
      l2=l1+1
      l3=l1+2
      h1=zetai(i,j,l1)
      h2=zetai(i,j,l2)
      h3=zetai(i,j,l3)
      h3m2=h3-h2
      h3m1=h3-h1
!***
!***  the coefficients for quadratic interpolation of heights.
!***
      b=(h3m2*(alpsq(l3)-alpsq(l1))-h3m1*(alpsq(l3)-alpsq(l2)))/
     &        dlt(l)
      c=(h3m1*(alp(l3)-alp(l2))-h3m2*(alp(l3)-alp(l1)))/dlt(l)
!
  140 arglog=sg1(l)*pdtop+sg2(l)*pdij+pt
      alpeta=dlog(arglog)
      arglg2=sgml1(l)*pdtop+sgml2(l)*pdij+pt  ! p at mid-hyb level
      alpm=dlog(arglg2)
!***
!***  the heights of the output hybrid interfaces.
!***
!      zetao(i,j,l)=h2+b*(alpeta-alp(l2))+c*(alpeta**2-alpsq(l2)) 
!
!***  obtain ZETAO using bi-linear instead of quadratic interpolation
      zetao(i,j,l)=h1+(h1-h2)*log(arglog/pinti(i,j,l1))/
     &log(pinti(i,j,l1)/pinti(i,j,l2))
!
      if(l.eq.2) go to 150
      l=l-1
      if(ldt1(l).eq.l1) go to 140
      go to 125
!
  150 pdo(i,j)=pdij
      if(pdo(i,j).lt. 0.1)print*,'enormal pdo',i,j,pdo(i,j)
      zetao(i,j,lmo+1)=fiso(i,j)/g
!
      if(zetao(i,j,lmo+1).gt.zetao(i,j,lmo))print*,'terrain higher
     & than zetao(lmo) zsfco,zetao(lm) ',i,j,fiso(i,j)/g
     &,zetao(i,j,lmo) 
!
  200 continue
!--------------------------------------------------------------
!***
!***  COMPUTATION OF WINDS WITHIN SIGMA LAYERS (on mid-level)
!***
!--------------------------------------------------------------
      CALL EXCH3D(PMIDI)
      CALL EXCH(PDO)
      CALL EXCH(RESO)
      DO 300 J=JSTA,JEND
      DO 300 I=1,IM-MOD(j,2)
      DO 250 L=1,LMI
      IF(I.EQ.IM .AND. J.EQ.JM)GO TO 250
!***
!***  AVERAGE PRESSURE AT VELOCITY POINTS IN INPUT DATA
!***  ALONG THE LATERAL BOUNDARIES.
!***
      IF(J .EQ. 1 .AND. I .LT. IM)THEN   !SOUTHERN BC
        PMVP(L)=0.5*(PMIDI(I,J,L)+PMIDI(I+1,J,L))
        IF(L.EQ.1)PDVP=0.5*(PDO(I,J)+PDO(I+1,J))
!
      ELSEIF(J.EQ.JM .AND. I.LT.IM)THEN   !NORTHERN BC
        PMVP(L)=0.5*(PMIDI(I,J,L)+PMIDI(I+1,J,L))
        IF(L.EQ.1)PDVP=0.5*(PDO(I,J)+PDO(I+1,J))
      ELSEIF(I .EQ. 1 .AND. MOD(J,2) .EQ. 0) THEN   !WESTERN EVEN BC
        PMVP(L)=0.5*(PMIDI(I,J-1,L)+PMIDI(I,J+1,L))
        IF(L.EQ.1)PDVP=0.5*(PDO(I,J-1)+PDO(I,J+1))
      ELSEIF(I .EQ. IM .AND. MOD(J,2) .EQ. 0) THEN  !EASTERN BC	
        PMVP(L)=0.5*(PMIDI(I,J-1,L)+PMIDI(I,J+1,L))
        IF(L.EQ.1)PDVP=0.5*(PDO(I,J-1)+PDO(I,J+1))
!
      ELSE IF (MOD(J,2) .LT. 1) THEN
C***
C***  PMVP AT ALL INTERIOR POINTS
C***
        PMVP(L)=0.25*(PMIDI(I,J,L)+PMIDI(I-1,J,L)+PMIDI(I,J+1,L)+
     &                PMIDI(I,J-1,L))
        IF(L.EQ.1)PDVP=0.25*(PDO(I,J)+PDO(I-1,J)+PDO(I,J+1)+
     &	PDO(I,J-1))
      ELSE
        PMVP(L)=0.25*(PMIDI(I,J,L)+PMIDI(I+1,J,L)+PMIDI(I,J+1,L)+
     &                PMIDI(I,J-1,L))
        IF(L.EQ.1)PDVP=0.25*(PDO(I,J)+PDO(I+1,J)+PDO(I,J+1)+
     &	PDO(I,J-1)) 
      ENDIF
!
  250 CONTINUE
!
      DO LD=1,LMIM
        ALPR(LD)=ALOG(PMVP(LD+1)/PMVP(LD))
      ENDDO   
      DO LD=1,LMI
        ALP(LD)=ALOG(PMVP(LD))
      ENDDO
!
      alpsfc=alog(pdvp+pdtop+pt)
!***
!***  INTERPOLATE THE WINDS
!***
              do 270 l=1,lmo
          IF(L.EQ.1)ALPLB=ALPT
          ALPUB=ALPLB
          alplb=alog(pt+sg1(l+1)*pdtop+sg2(l+1)*pdvp)
!          alpeta=sqrt(0.5*(alpub**2+alplb**2))
          alpeta=alog(pt+sgml1(l)*pdtop+sgml2(l)*pdvp)
!
          DO 260 LD=2,LMI
      IF(ALPETA.GT.ALP(LD).AND.LD.LT.LMI)GO TO 260
          IF((ALPETA.LE.ALP(LD) .OR. (ALPETA.GT.ALP(LD)
     &       .AND.LD.EQ.LMI)).AND.ABS(UI(I,J,LD)).GT.0.)
     &    THEN
      ULD=UI(I,J,LD)
      LDM1=LD-1
      CF=(ALP(LD)-ALPETA)/ALPR(LDM1)
      UK=ULD+(UI(I,J,LDM1)-ULD)*CF
      UTOP=UK
      ALPTU=ALPETA
      LTOP=L
      UO(I,J,L)=UK
      GO TO 270
C***
C***  IF THE NEW SIGMA GROUND SURFACE IS BELOW THE OLD GROUND, THEN
C***  ASSUME THE WIND COMPONENTS DECREASE TO ZERO AT THE SURFACE.
C***  THE SAME IS TRUE FOR ANY NEW ETA LEVEL BELOW THE LOWEST
C***  OLD ETA SURFACE.
C***  
          ELSEIF(UI(I,J,LD).EQ.0.) THEN
      UDIF=UTOP
      ALPDIF=ALPTU-ALPSFC
      LT1=LTOP+1
              do ll=lt1,lmo
          if(ll.eq.lt1) alplb=alog(pt+sg1(ll)*pdtop+sg2(ll)*pdvp)
          ALPUB=ALPLB
          alplb=alog(pt+sg1(ll+1)*pdtop+sg2(ll+1)*pdvp)
          alpeta=alog(pt+sgml1(ll)*pdtop+sgml2(ll)*pdvp)
          cf=(alpeta-alptu)
     &      /(alptu-alog(pt+sgml1(ltop-1)*pdtop+sgml2(ltop-1)*pdvp))
          UO(I,J,LL)=UTOP-UDIF*(ALPTU-ALPETA)/ALPDIF
              enddo
      GO TO 300
          ENDIF
  260 CONTINUE
C
  270 CONTINUE
C
  300 CONTINUE
      do l=1,lmo  
       do j=jsta,jend
        do i=1,im
         if(abs(Uo(i,j,l)).gt.120.)print*,'enormal u',i,j,l
     +   ,Uo(i,j,l)
        end do 
       end do
      end do 
C***
C***  SOUTHERN V POINTS
C***
      DO L=1,LMO
       CALL COLLECT_LOC(UO(1,JSTA,L),DUM2DO)
       IF(MYPE.EQ.0)THEN
        N=1
        DO I=1,IM-1
         UB(N,L,1)=DUM2DO(I,1)
         UB(N,L,2)=0.
         N=N+1
        END DO
C***
C***  NORTHERN V POINTS
C
        DO I=1,IM-1
         UB(N,L,1)=DUM2DO(I,JM)
         UB(N,L,2)=0.
         N=N+1
        END DO
C***
C***  WESTERN V POINTS
C***
        DO J=2,JM-1,2
         UB(N,L,1)=DUM2DO(1,J)
         UB(N,L,2)=0.
         N=N+1
        END DO
C***
C***  EASTERN V POINTS
C***
        DO J=2,JM-1,2
         UB(N,L,1)=DUM2DO(IM,J)
         UB(N,L,2)=0.
         N=N+1
        END DO 
       END IF
      END DO 	
c
      DO 305 J=JSTA,JEND
      DO 305 I=1,IM-MOD(j,2)
      DO 255 L=1,LMI
      IF(I.EQ.IM .AND. J.EQ.JM)GO TO 255
!***
!***  AVERAGE PRESSURE AT VELOCITY POINTS IN INPUT DATA
!***  ALONG THE LATERAL BOUNDARIES.
!***
      IF(J .EQ. 1 .AND. I .LT. IM)THEN   !SOUTHERN BC
        PMVP(L)=0.5*(PMIDI(I,J,L)+PMIDI(I+1,J,L))
        IF(L.EQ.1)PDVP=0.5*(PDO(I,J)+PDO(I+1,J))
!
      ELSEIF(J.EQ.JM .AND. I.LT.IM)THEN   !NORTHERN BC
        PMVP(L)=0.5*(PMIDI(I,J,L)+PMIDI(I+1,J,L))
        IF(L.EQ.1)PDVP=0.5*(PDO(I,J)+PDO(I+1,J))
      ELSEIF(I .EQ. 1 .AND. MOD(J,2) .EQ. 0) THEN   !WESTERN EVEN BC
        PMVP(L)=0.5*(PMIDI(I,J-1,L)+PMIDI(I,J+1,L))
        IF(L.EQ.1)PDVP=0.5*(PDO(I,J-1)+PDO(I,J+1))
      ELSEIF(I .EQ. IM .AND. MOD(J,2) .EQ. 0 ) THEN  !EASTERN BC	
        PMVP(L)=0.5*(PMIDI(I,J-1,L)+PMIDI(I,J+1,L))
        IF(L.EQ.1)PDVP=0.5*(PDO(I,J-1)+PDO(I,J+1))
!
      ELSE IF (MOD(J,2) .LT. 1)THEN
!***
!***  PMVP AT ALL INTERIOR POINTS
!***
        PMVP(L)=0.25*(PMIDI(I,J,L)+PMIDI(I-1,J,L)+PMIDI(I,J+1,L)+
     &                PMIDI(I,J-1,L))
        IF(L.EQ.1)PDVP=0.25*(PDO(I,J)+PDO(I-1,J)+PDO(I,J+1)+
     &	PDO(I,J-1))
      ELSE
        PMVP(L)=0.25*(PMIDI(I,J,L)+PMIDI(I+1,J,L)+PMIDI(I,J+1,L)+
     &                PMIDI(I,J-1,L))
        IF(L.EQ.1)PDVP=0.25*(PDO(I,J)+PDO(I+1,J)+PDO(I,J+1)+
     &	PDO(I,J-1))      
      ENDIF
  255 CONTINUE
!
      DO LD=1,LMIM
        ALPR(LD)=ALOG(PMVP(LD+1)/PMVP(LD))
      ENDDO   
      DO LD=1,LMI
        ALP(LD)=ALOG(PMVP(LD))
      ENDDO
C
      ALPSFC=ALOG(PDVP+PT+PDTOP)
!***
!***  INTERPOLATE THE WINDS
!***
              DO 275 L=1,LMO
          IF(L.EQ.1)ALPLB=ALPT
          ALPUB=ALPLB
          alplb=alog(pt+sg1(l+1)*pdtop+sg2(l+1)*pdvp)
!          alpeta=sqrt(0.5*(alpub**2+alplb**2))
          alpeta=alog(pt+sgml1(l)*pdtop+sgml2(l)*pdvp)
! 
          DO 265 LD=2,LMI
      IF(ALPETA.GT.ALP(LD).AND.LD.LT.LMI)GO TO 265
          IF((ALPETA.LE.ALP(LD) .OR. (ALPETA.GT.ALP(LD)
     &       .AND.LD.EQ.LMI)).AND.ABS(VI(I,J,LD)).GT.0.)
     &    THEN
      VLD=VI(I,J,LD)
      LDM1=LD-1
      CF=(ALP(LD)-ALPETA)/ALPR(LDM1)
      VK=VLD+(VI(I,J,LDM1)-VLD)*CF
      VTOP=VK
      ALPTU=ALPETA
      LTOP=L
      VO(I,J,L)=VK
      GO TO 275
C***
C***  IF THE NEW SIGMA GROUND SURFACE IS BELOW THE OLD GROUND, THEN
C***  ASSUME THE WIND COMPONENTS DECREASE TO ZERO AT THE SURFACE.
C***  THE SAME IS TRUE FOR ANY NEW ETA LEVEL BELOW THE LOWEST
C***  OLD ETA SURFACE.
C***  
          ELSEIF(VI(I,J,LD).EQ.0.)THEN
c      PRINT*,'INTERPOLATE WIND FROM UNDERGROUND AT',I,J,L
c      PRINT*,'ALPETA,ALP(LMI),VI= ',ALPETA,ALP(LMI),VI(I,J,LD)
      VDIF=VTOP
      ALPDIF=ALPTU-ALPSFC
      LT1=LTOP+1
              do ll=lt1,lmo
          if(ll.eq.lt1) alplb=alog(pt+sg1(ll)*pdtop+sg2(ll)*pdvp)
          ALPUB=ALPLB
          alplb=alog(pt+sg1(ll+1)+sg2(ll+1)*pdvp)
!          alpeta=sqrt(0.5*(alpub**2+alplb**2))
	  alpeta=alog(pt+sgml1(ll)*pdtop+sgml2(ll)*pdvp)
          cf=(alpeta-alptu)
     &      /(alptu-alog(pt+sgml1(ltop-1)*pdtop+sgml2(ltop-1)*pdvp))
          VO(I,J,LL)=VTOP-VDIF*(ALPTU-ALPETA)/ALPDIF
              enddo
      GO TO 305
          ENDIF
  265     CONTINUE
C
  275 CONTINUE
C
  305 CONTINUE
      do l=1,lmo  
       do J=JSTA,JEND
        do I=1,Im
         if(abs(Vo(i,j,l)).gt.120.)print*,'enormal v',i,j,l
     +   ,Vo(i,j,l)
        end do 
       end do
      end do
C***
C***  SOUTHERN V POINTS
      DO L=1,LMO
       CALL COLLECT_LOC(VO(1,JSTA,L),DUM2DO)
       IF(MYPE.EQ.0)THEN
        N=1
        DO I=1,IM-1
         VB(N,L,1)=DUM2DO(I,1)
         VB(N,L,2)=0.
         N=N+1
        END DO
C***
C***  NORTHERN V POINTS
C
        DO I=1,IM-1
         VB(N,L,1)=DUM2DO(I,JM)
         VB(N,L,2)=0.
         N=N+1
        END DO
C***
C***  WESTERN V POINTS
C***
        DO J=2,JM-1,2
         VB(N,L,1)=DUM2DO(1,J)
         VB(N,L,2)=0.
         N=N+1
        END DO
C***
C***  EASTERN V POINTS
C***
        DO J=2,JM-1,2
         VB(N,L,1)=DUM2DO(IM,J)
         VB(N,L,2)=0.
         N=N+1
        END DO 
       END IF
      END DO
C--------------------------------------------------------------------
C OBTAIN TEMPERATURE AT MID-SIGMA LAYER BASED ON HYDROSTATIC 
C RELATIONSHIP FROM BOTTOM UP
C THE LOWEST LAYER TEMP IS EXTRAPOLATED FROM TWO LAYER ABOVE
C FOR NOW DUE TO INCONSISTANT LOWER LEVEL HEIGHT
C
      DO 325 J=JSTA,JEND
      DO 325 I=1,IM
        PHUB=G*ZETAO(I,J,LMO)
        DO L=LMO-1,1,-1
        PHLB=PHUB
        PHUB=G*ZETAO(I,J,L)
        to(i,j,l)=-(phlb-phub)*(pt+sgml1(l)*pdtop+sgml2(l)*pdo(i,j))
     &            /(rd*(dsg1(l)*pdtop+dsg2(l)*pdo(i,j)))
        if(i.eq.ii.and.j.eq.jj)print*,'Debug 1: L,TO(',ii,','
     &  ,jj,')= ',L,to(ii,jj,l)-273.
c     if(i.eq.im.and.j.eq.5)then
c       write(0,*)' 0 l=',l,' to=',to(i,j,l),' phlb=',phlb
c    1,           ' phub=',phub,' pt=',pt
c       write(0,*)' 0 sgml1=',sgml1(l),' sgml2=',sgml2(l)
c    1,           ' dsg1=',dsg1(l),' dsg2=',dsg2(l)
c       write(0,*)' 0 pdtop=',pdtop,' pdo=',pdo(i,j)
c     endif
        IF(TO(I,J,L).LT.150.)TO(I,J,L)=150.
        ENDDO
  325 CONTINUE
CCC THE FOLLOWING LINES OBTAIN TEMP(LMO) USING TWO LEVELS ABOVE
      DO 326 J=JSTA,JEND
       DO 326 I=1,IM
        apel1=log(pt+sgml1(lmo-5)*pdtop+sgml2(lmo-5)*pdo(i,j))
        apel2=log(pt+sgml1(lmo-1)*pdtop+sgml2(lmo-1)*pdo(i,j))
c        print*,
c     +  sgml1(lmo-5),pdtop,sgml2(lmo-5),pdo(i,j),sgml1(lmo-1)
c     +,sgml2(lmo-1)
        apel3=log(pt+sgml1(lmo)*pdtop+sgml2(lmo)*pdo(i,j))
        TO(I,J,LMO)=TO(I,J,LMO-1)+(TO(I,J,LMO-1)-TO(I,J,LMO-5))
     &                           *(APEL3-APEL2)/(APEL2-APEL1)	
c       if(i.eq.ii.and.j.eq.jj)print*,'Debug 1: LMO,TO(',ii,','
c    &  ,jj,')= ',LMO,TO(ii,jj,LMO)-273.
c     if(i.eq.im.and.j.eq.5)then
c       write(0,*)' 1 lmo=',lmo,' to=',to(i,j,lmo)
c    1,           to(i,j,lmo-1),to(i,j,lmo-5)
c       write(0,*)' 1 apel3=',apel3,' apel2=',apel2
c    1,           ' apel1=',apel1
c     endif
 326  CONTINUE      
      do l=1,lmo
       do j=JSTA,JEND
        do i=1,im
	 if(to(i,j,l).gt.325.0.or.to(i,j,l).lt.150.)print*,
     +   'enormal temperature',i,j,l,to(i,j,l),mype
        end do
       end do
      end do 		 
C IF SIGMA GROUND FALLS UNDER ETA GROUND, EXTRAPOLATE TO
      DO 328 J=JSTA,JEND
       DO 328 I=1,IM
        PIN=PT+ETAI(LMHOI(I,J)-1)*PDSLI(I,J)
        DO L=1,LMO
         psigma=pt+sg1(l)*pdtop+sg2(l)*pdo(i,j)
	 IF(PSIGMA .GT. PIN)THEN 
	  DO LL=L-1,LMO
           apel3=log(pt+sgml1(ll-5)*pdtop+sgml2(ll-5)*pdo(i,j))
           apel2=log(pt+sgml1(ll-1)*pdtop+sgml2(ll-1)*pdo(i,j))
           apel1=log(pt+sgml1(ll)*pdtop+sgml2(ll)*pdo(i,j))
	   TO(I,J,LL)=TO(I,J,LL-1)+(TO(I,J,LL-1)-TO(I,J,LL-5))
     &                            *(APEL1-APEL2)/(APEL2-APEL3)
c     if(i.eq.im.and.j.eq.5)then
c       write(0,*)' 2 ll=',ll,' to=',to(i,j,ll)
c    1,           to(i,j,ll-1),to(i,j,ll-5)
c       write(0,*)' 2 apel3=',apel3,' apel2=',apel2
c    1,           ' apel1=',apel1
c     endif
          END DO
	  GO TO 328
	 END IF 	
	END DO
  328 CONTINUE	    
C CHECK ON ENORMAL LAPSE RATE
      DO L=1,LMO-1
       DO J=JSTA,JEND
        DO I=1,IM
	 IF(L.GT.30.AND.ABS(TO(I,J,L)-TO(I,J,L+1)).GT.5.0)PRINT*,
     + 'LARGE LAPSE RATE AT ',I,J,L,'DT= ',TO(I,J,L)-TO(I,J,L+1)
        END DO
       END DO
      END DO 		 
C***
C***  INTERPOLATE Q AND CLD WATER LINEARLY IN LN(P)
C***
CWAS      open(80,file='cldi.out',form='unformatted',status='unknown')
CWAS      dum3do=0.
CWAS      do l=1,lmi 
CWAS       read(80)((DUM3DI(I,J,L),I=1,IM),J=1,JM)
CWAS      end do
CWAS      close(80) 
C 
C
      DO 355 J=JSTA,JEND
       DO 355 I=1,IM
C
       DO LD=1,LMIM
        ALPR(LD)=ALOG(PMIDI(I,J,LD+1)/PMIDI(I,J,LD))
       ENDDO
       DO LD=1,LMI
        ALP(LD)=ALOG(PMIDI(I,J,LD))
       ENDDO
C
       LKO=LMO
       ALPLB=ALPT
       DO 345 L=1,LKO
       ALPUB=ALPLB
       alplb=alog(pt+sg1(l+1)*pdtop+sg2(l+1)*pdo(i,j))
!       alpeta=sqrt(0.5*(alpub**2+alplb**2))
       alpeta=alog(pt+sgml1(l)*pdtop+sgml2(l)*pdo(i,j))
       pmido=pt+sgml1(l)*pdtop+sgml2(l)*pdo(i,j)
       DO 335 LD=2,LMI
        IF(HF)THEN 
	 HWGT=HTMI(I,J,LD)
	ELSE 
	 IF(IHINY(I,J).EQ.1 .OR. IHINY(I,J).EQ.JMI)THEN  !assume jmi odd
          HWGT=HTMI(IHINX(I,J),IHINY(I,J),LD)
     1       +HTMI(IHINX(I,J)+1,IHINY(I,J),LD)
         ELSE IF(IHINX(I,J).EQ.1 .OR. IHINX(I,J).EQ.IMI)THEN
          HWGT=HTMI(IHINX(I,J),IHINY(I,J)+1,LD)
     1       +HTMI(IHINX(I,J),IHINY(I,J)-1,LD)
	 ELSE IF(MOD(IHINY(I,J),2) .LT. 1)THEN
          HWGT=HTMI(IHINX(I,J)-1,IHINY(I,J),LD)+HTMI(IHINX(I,J),
     1    IHINY(I,J)+1,LD)+HTMI(IHINX(I,J),IHINY(I,J),LD)
     2    +HTMI(IHINX(I,J),IHINY(I,J)-1,LD)
         ELSE
	  HWGT=HTMI(IHINX(I,J),IHINY(I,J),LD)+HTMI(IHINX(I,J),
     1    IHINY(I,J)+1,LD)+HTMI(IHINX(I,J)+1,IHINY(I,J),LD)
     2    +HTMI(IHINX(I,J),IHINY(I,J)-1,LD)
         END IF
	END IF
        IF(HWGT.GT.0.5.AND.(ALPETA.LE.ALP(LD).OR.
     1                 (ALPETA.GT.ALP(LD).AND.LD.EQ.LMI)))THEN
        CWMLD=CLDLIN(I,J,LD)
        LDM1=LD-1
        CF=(ALP(LD)-ALPETA)/ALPR(LDM1)
        CWMO(I,J,L)=CWMLD+(cldlin(I,J,LDM1)-CWMLD)*CF
        CWMO(I,J,L)=AMAX1(CWMO(I,J,L),0.)
        LKI=LD
        GO TO 345
C***
C***  IF THE NEW SIGMA GROUND SURFACE IS BELOW THE OLD GROUND THEN
C***  ASSUME THE RH (GIVEN T) IS EQUAL TO THAT ON THE LOWEST
C***  OLD SURFACE AND THAT THE CLOUD WATER IS ZERO.
C***
C*** IF ALL NEIGHBORING POINTS (BRFORE HORIZONTAL INTERP)FOR THE 
C*** OUTPUT GRID ARE UNDERGROUND THEN THE INPUT QI IS ZERO 
C
      ELSEIF(HWGT.LT.0.5)THEN
        CWMO(I,J,L)=0.
        GO TO 345
      ENDIF
  335 CONTINUE
  345 CONTINUE
C
  355 CONTINUE
C  
      do l=1,lmo
       do j=JSTA,JEND
        do I=1,Imo
	 if(cwmo(i,j,l).gt.0.01)print*,'enormal cwmo',i,j,l
     +  ,cwmo(i,j,l)
        end do
       end do
      end do 		 
C
C            
      print*,'start computing qi'
      DO 350 J=JSTA,JEND
       DO 350 I=1,IM
C
       DO LD=1,LMIM
        ALPR(LD)=ALOG(PMIDI(I,J,LD+1)/PMIDI(I,J,LD))
       ENDDO
       DO LD=1,LMI
        ALP(LD)=ALOG(PMIDI(I,J,LD))
       ENDDO
C
       LKO=LMO
       ALPLB=ALPT
       DO 340 L=1,LKO
       ALPUB=ALPLB
       alplb=log(pt+sg1(l+1)*pdtop+sg2(l+1)*pdo(i,j))
!       alpeta=sqrt(0.5*(alpub**2+alplb**2))
       alpeta=log(pt+sgml1(l)*pdtop+sgml2(l)*pdo(i,j))
       pmido=pt+sgml1(l)*pdtop+sgml2(l)*pdo(i,j)
       DO 330 LD=2,LMI
        IF(HF)THEN 
	 HWGT=HTMI(I,J,LD)
	ELSE 
         IF(IHINY(I,J).EQ.1 .OR. IHINY(I,J).EQ.JMI)THEN  !assume jmi odd
          HWGT=HTMI(IHINX(I,J),IHINY(I,J),LD)
     1       +HTMI(IHINX(I,J)+1,IHINY(I,J),LD)
         ELSE IF(IHINX(I,J).EQ.1 .OR. IHINX(I,J).EQ.IMI)THEN
          HWGT=HTMI(IHINX(I,J),IHINY(I,J)+1,LD)
     1       +HTMI(IHINX(I,J),IHINY(I,J)-1,LD) 
	 ELSE IF(MOD(IHINY(I,J),2) .LT. 1)THEN
          HWGT=HTMI(IHINX(I,J)-1,IHINY(I,J),LD)+HTMI(IHINX(I,J),
     1    IHINY(I,J)+1,LD)+HTMI(IHINX(I,J),IHINY(I,J),LD)
     2    +HTMI(IHINX(I,J),IHINY(I,J)-1,LD)
         ELSE
	  HWGT=HTMI(IHINX(I,J),IHINY(I,J),LD)+HTMI(IHINX(I,J),
     1    IHINY(I,J)+1,LD)+HTMI(IHINX(I,J)+1,IHINY(I,J),LD)
     2    +HTMI(IHINX(I,J),IHINY(I,J)-1,LD)
         END IF
	END IF
        IF(HWGT.GT.0.5.AND.(ALPETA.LE.ALP(LD).OR.
     1                 (ALPETA.GT.ALP(LD).AND.LD.EQ.LMI)))THEN
        QLD=QI(I,J,LD)
        LDM1=LD-1
        CF=(ALP(LD)-ALPETA)/ALPR(LDM1)
        QETAIJ=QLD+(QI(I,J,LDM1)-QLD)*CF
        CLOGES=-CM1/TO(I,J,L)-CM2*ALOG10(TO(I,J,L))+CM3
        ESE=10.**(CLOGES+2.)
        QSX=EPS*ESE/(PMIDO-ESE*(1.-EPS))
        QSMX=0.98*QSX
        QO(I,J,L)=AMIN1(QETAIJ,QSMX)
        QO(I,J,L)=AMAX1(QO(I,J,L),EPSQ)
        if(i.eq.ii.and.j.eq.jj)
     +print*,'Debug:QO computation when HWGT GT 0.5'
     +,',l,IHINX,IHINY,HTMI1,HTMI2,HTMI3,HTMI4 = '
     +,l,IHINX(I,J),IHINY(I,J),HTMI(IHINX(I,J)-1,IHINY(I,J),LD)
     +,HTMI(IHINX(I,J),IHINY(I,J)+1,LD)
     +,HTMI(IHINX(I,J),IHINY(I,J),LD)
     +,HTMI(IHINX(I,J),IHINY(I,J)-1,LD)
        if(i.eq.ii.and.j.eq.jj)
     +print*,'Debug:QO computation when HWGT GT 0.5'
     +,' ,l,ld,QI(I,J,LD),QI(I,J,LDM1),qetaij,qo= ',l,ld,QI(I,J,LD)
     +,QI(I,J,LDM1),qetaij,qo(i,j,l)  
C       IF(QO(I,J,L).LT.QO(I,J,L-1).AND.LD.EQ.LMHO(I,J))QO(I,J,L)=QO(I,J,L-1)
        LKI=LD
        GO TO 340
C***
C***  IF THE NEW SIGMA GROUND SURFACE IS BELOW THE OLD GROUND THEN
C***  ASSUME THE RH (GIVEN T) IS EQUAL TO THAT ON THE LOWEST
C***  OLD SURFACE AND THAT THE CLOUD WATER IS ZERO.
C***
C*** IF ALL NEIGHBORING POINTS (BRFORE HORIZONTAL INTERP)FOR THE 
C*** OUTPUT GRID ARE UNDERGROUND THEN THE INPUT QI IS ZERO 
C
      ELSEIF(HWGT.LT.0.5)THEN
c     if(TI(I,J,lki).lt.1.e-2)stop666
      if(TI(I,J,lki).lt.1.e-2)then
       if(i.eq.ii.and.j.eq.jj)
     +  print*,'Debug:QO computation when TI lt 1.e-2)'
     +  ,'ti,qi = ',TI(I,J,lki),qi(i,j,l-1)
          qo(i,j,l)=qi(i,j,l-1)
          go to 340
      endif
        CLOGES=-CM1/TI(I,J,LKI)-CM2*ALOG10(TI(I,J,LKI))+CM3
        ESS=10.**(CLOGES+2.)
        QSLDM=EPS*ESS/(PMIDI(I,J,LKI)-ESS*(1.-EPS))
        RHLDM=QI(I,J,LKI)/QSLDM
        RHLDM=AMIN1(RHLDM,0.98)
        CLOGES=-CM1/TO(I,J,L)-CM2*ALOG10(TO(I,J,L))+CM3
        ESL=10.**(CLOGES+2.)
        qsl=eps*esl/(pt+sgml1(l)*pdtop+sgml2(l)*pdo(i,j)-esl*(1.-eps))
        QO(I,J,L)=RHLDM*QSL
        if(i.eq.ii.and.j.eq.jj)
     +print*,'Debug:QO computation when HWGT LT 0.5'
     +,' l,RHLDM,QSL,QO at 217,487= ',l,RHLDM,QSL,QO(I,J,L)
        GO TO 340
      ENDIF
  330 CONTINUE
  340 CONTINUE
C
CC Tempory fix for QO
C COMMENT OUT FROM NOW
C
      qmin=1.0e-10
      do l=1,lmo
       if(qo(i,j,l).le.qmin)then
        print*,'fixing qo at i,j,l,qold',i,j,l,qo(i,j,l)
        ifind=0
        do ll=l-1,1
         if(qo(i,j,ll).gt.qmin)then
          ifind=ifind+1
C         qo(i,j,l)=qo(i,j,ll)
          go to 342
         end if
        end do
        if(ifind.lt.1)then
         do ll=l+1,lmo
          if(qo(i,j,ll).gt.qmin)then
           ifind=ifind+1
C          qo(i,j,l)=qo(i,j,ll)
           go to 342
          end if
         end do
        end if
c       if(ifind.lt.1)qo(i,j,l)=qmin
        print*,'fixed qo at i,j,l,qnew= ',i,j,l,qo(i,j,l)
       end if
 342   continue
      end do
C
      DO L=1,LMO
        IF(QO(I,J,L).LT.0.)QO(I,J,L)=0.
        IF(QO(I,J,L).GT.25.E-3)QO(I,J,L)=25.E-3
      ENDDO   
C
  350 CONTINUE
C***
C***  CONVERT FROM VIRTUAL TO TRUE TEMPERATURE.
C***
      DO 375 L=1,LMO
      DO 375 J=JSTA,JEND
      DO 375 I=1,IM
      IF(TO(I,J,L).GT.325.)THEN
       PRINT*,'TEMP > 325',I,J,L,TO(I,J,L)
       TO(I,J,L)=325.
      END IF 
      IF(TO(I,J,L).LT.150.)TO(I,J,L)=150.
      TV=TO(I,J,L)
      DO ITER=1,3
        TO(I,J,L)=TV/(1.+0.608*QO(I,J,L))
c     if(i.eq.im.and.j.eq.5)then
c       write(0,*)' 3 iter=',iter,' l=',l,' to=',to(i,j,l)
c    1,           ' tv=',tv,' qo=',qo(i,j,l)
c     endif
        CLOGES=-CM1/TO(I,J,L)-CM2*ALOG10(TO(I,J,L))+CM3
        ESE=10.**(CLOGES+2.)
        denom2=pt+sgml1(l)*pdtop+sgml2(l)*pdo(i,j)-ese*(1.-eps)
        QSX=EPS*ESE/DENOM2
        QSMX=0.98*QSX
        IF(QO(I,J,L).LT.QSMX)GO TO 375
        QO(I,J,L)=QSMX
      ENDDO
  375 CONTINUE
      do l=1,lmo
       do j=JSTA,JEND
        do i=1,im
	 if(to(i,j,l).gt.325.0.or.to(i,j,l).lt.150.)print*,
     +   'enormal temperature',i,j,l,to(i,j,l)
         if(i.eq.2.and.j.eq.73.and.l.eq.17)print*,'T AT 2 73 17'
     +    ,i,j,l,TO(I,J,L)
         if(qo(i,j,l).gt.0.025.or.qo(i,j,l).le.qmin)
     +print*,'enormal q',i,j
     +   ,qo(i,j,l)
        end do
       end do
      end do
C COLLECT PDB
      CALL COLLECT(PDO)
      IF(MYPE.EQ.0)THEN
       N=1
       DO I=1,IM
        PDB(N,1)=PDO(I,1)
        PDB(N,2)=0.
        N=N+1
       END DO
       DO I=1,IM
        PDB(N,1)=PDO(I,JM)
        PDB(N,2)=0
        N=N+1
       END DO
       DO J=3,JM-2,2
        PDB(N,1)=PDO(1,J)
        PDB(N,2)=0.
	N=N+1
       END DO
       DO J=3,JM-2,2
        PDB(N,1)=PDO(IM,J)
        PDB(N,2)=0.
	N=N+1
       END DO	
      END IF
C COLLECT TB
      DO L=1,LMO
       CALL COLLECT_LOC(TO(1,JSTA,L),DUM2DO)
       IF(MYPE.EQ.0)THEN
        N=1
        DO I=1,IM
         TB(N,L,1)=DUM2DO(I,1)
         TB(N,L,2)=0.
         N=N+1
        END DO
C***
C***  NORTHERN H POINTS
C***
        DO I=1,IM
         TB(N,L,1)=DUM2DO(I,JM)
         TB(N,L,2)=0.
         N=N+1
        END DO
C***
C***  WESTERN H POINTS
C***
        DO J=3,JM-2,2
         TB(N,L,1)=DUM2DO(1,J)
         TB(N,L,2)=0.
         N=N+1
        END DO
C***
C***  EASTERN H POINTS
C***
        DO J=3,JM-2,2
         TB(N,L,1)=DUM2DO(IM,J)
         TB(N,L,2)=0.
	 N=N+1
        END DO
       END IF
      END DO 
C COLLECT QB
      DO L=1,LMO
       CALL COLLECT_LOC(QO(1,JSTA,L),DUM2DO)
       IF(MYPE.EQ.0)THEN
        N=1
        DO I=1,IM
         QB(N,L,1)=DUM2DO(I,1)
         QB(N,L,2)=0.  
         N=N+1
        END DO
C***
C***  NORTHERN H POINTS
C***
        DO I=1,IM
         QB(N,L,1)=DUM2DO(I,JM)
         QB(N,L,2)=0.
         N=N+1
        END DO
C***
C***  WESTERN H POINTS
C***
        DO J=3,JM-2,2
         QB(N,L,1)=DUM2DO(1,J)
         QB(N,L,2)=0.
         N=N+1
        END DO
C***
C***  EASTERN H POINTS
C***
        DO J=3,JM-2,2
         QB(N,L,1)=DUM2DO(IM,J)
         QB(N,L,2)=0.
	 N=N+1
        END DO
       END IF
      END DO
C      
C--------------------------------------------------------------------
C***
C***  INTERPOLATE Q2 LINEARLY IN LN(P) (BUT ON INTERFACES)
C***
      DO 450 J=JSTA,JEND
      DO 450 I=1,IM
C
      DO LD=1,LMI
        ALPR(LD)=ALOG(PINTI(I,J,LD+1)/PINTI(I,J,LD))
      ENDDO
      DO LD=1,LMIP
        ALP(LD)=ALOG(PINTI(I,J,LD))
      ENDDO
C
      LKO=LMO-1
      Q2SFC=AMAX1(B1**(2./3.)*USTAR(I,J)*USTAR(I,J),EPSQ2)
      Q2O(I,J,LKO+1)=Q2SFC
      alpsfc=alog(pdo(i,j)+pdtop+pt)
C
      DO 440 L=1,LKO
      alpeta=alog(pt+sg1(l+1)*pdtop+sg2(l+1)*pdo(i,j))
C
      DO 430 LD=2,LMI
       IF(HF)THEN
        HWGT=HTMI(I,J,LD)
       ELSE	
        IF(IHINY(I,J).EQ.1 .OR. IHINY(I,J).EQ.JMI)THEN  !assume jmi odd
         HWGT=HTMI(IHINX(I,J),IHINY(I,J),LD)
     1       +HTMI(IHINX(I,J)+1,IHINY(I,J),LD)
        ELSE IF(IHINX(I,J).EQ.1 .OR. IHINX(I,J).EQ.IMI)THEN
         HWGT=HTMI(IHINX(I,J),IHINY(I,J)+1,LD)
     1       +HTMI(IHINX(I,J),IHINY(I,J)-1,LD)
        ELSE IF(MOD(IHINY(I,J),2) .LT. 1)THEN
         HWGT=HTMI(IHINX(I,J)-1,IHINY(I,J),LD)+HTMI(IHINX(I,J),
     1   IHINY(I,J)+1,LD)+HTMI(IHINX(I,J),IHINY(I,J),LD)
     2   +HTMI(IHINX(I,J),IHINY(I,J)-1,LD)
        ELSE
	 HWGT=HTMI(IHINX(I,J),IHINY(I,J),LD)+HTMI(IHINX(I,J),
     1   IHINY(I,J)+1,LD)+HTMI(IHINX(I,J)+1,IHINY(I,J),LD)
     2   +HTMI(IHINX(I,J),IHINY(I,J)-1,LD)
        END IF
       END IF
       IF(HWGT.GT.0.5.AND.(ALPETA.LE.ALP(LD) .OR.
     1                     ALPETA.GT.ALP(LMI)))THEN
        Q2LD=Q2I(I,J,LD)
        LDM1=LD-1
        CF=(ALP(LD)-ALPETA)/ALPR(LDM1)
        Q2IJ=Q2LD+(Q2I(I,J,LDM1)-Q2LD)*CF
        Q2TOP=AMAX1(Q2IJ,EPSQ2)
        ALPT=ALPETA
        LTOP=L
        Q2O(I,J,L)=Q2TOP
        GO TO 440
C***
C***  IF THE NEW ETA GROUND SURFACE IS BELOW THE OLD GROUND THEN
C***  ASSUME Q2 DECREASES LINEARLY TO Q2 AT THE NEW GROUND.
C***
       ELSEIF(HWGT.LT.0.5)THEN
        Q2DIF=Q2TOP-Q2SFC
        ALPDIF=ALPT-ALPSFC
        LT1=LTOP+1
        DO LL=LT1,LKO
          alpeta=log(pt+sg1(ll)*pdtop+sg2(ll)*pdo(i,j))
          Q2O(I,J,LL)=Q2TOP-Q2DIF*(ALPT-ALPETA)/ALPDIF
        ENDDO
        GO TO 450
      ENDIF
  430 CONTINUE
  440 CONTINUE
C
  450 CONTINUE
      PRINT*,'SAMPLE Q2'
      DO L=1,LMO
       J=49
       PRINT*,(Q2O(I,J,L),I=35,40)
      END DO   
CWAS      deallocate(dum3do)
C-----------------------------------------------------------
C***
C***  NOW REDEFINE PD TO HAVE IT EQUAL TO PSFC-PT.
C***
      DO J=JSTA,JEND
       DO I=1,IM
        PDO(I,J)=PDO(I,J)*RESO(I,J)
       END DO
      ENDDO
C
      DO L=1,LMOP
        dflo(l)=t0*(1.-((pt+sg1(l)*pdtop+sg2(l)*(prf0-pt2))
     &                  /prf0)**rgog)/gamma
      ENDDO
C----------------------------------------------------------------
C***
C***  TRANSLATE HBOT AND HTOP FROM INPUT LEVELS TO OUTPUT LEVELS
C***
      DO J=JSTA,JEND
       DO I=1,IM
        HBOTK=HBOT(I,J)
        HTOPK=HTOP(I,J)
        LMHOK=LMO
C
        DO 475 L=1,LMO
        IF(HBOTK.LT.LDT1(L))THEN
          HBOT(I,J)=L
C          IF(HBOT(I,J).GE.LMHOK-1)HBOT(I,J)=LMHOK-1
          GO TO 480
        ENDIF
  475   CONTINUE
C            
  480   DO 485 L=1,LMO
        IF(HTOPK.LT.LDT1(L))THEN
          HTOP(I,J)=L
          GO TO 485
        ENDIF
  485   CONTINUE
C            
       END DO
      ENDDO
!---------------------------------------------------------------------
CCC WRITING BOUNDARY CONDITIONS TO NBC FILE FOR BCTEND TO USE LATER
      IF(MYPE.EQ.0)THEN
       WRITE(NBC)RUN,IDAT,IHRST
       WRITE(NBC)PDB,TB,QB,UB,VB
      END IF 
!---------------------------------------------------------------------
      RETURN
      END
