      SUBROUTINE ETALL(IM,JM,TLM0D,TPH0D,DLMD,DPHD,
     &                 HLAT,HLON,VLAT,VLON)
C$$$  SUBPROGRAM DOCUMENTATION BLOCK
C                .      .    .                                       .
C SUBPROGRAM: ETALL         COMPUTE EARTH LATITUDE & LONIGTUDE OF
C                           ETA GRID POINTS
C   PRGMMR: ROGERS          ORG: W/NMC22    DATE: 90-06-13
C
C ABSTRACT: COMPUTES THE EARTH LATITUDE AND LONGITUDE OF ETA GRID
C   POINTS (BOTH H AND V POINTS)
C
C PROGRAM HISTORY LOG:
C   90-06-13  E.ROGERS
C
C USAGE:    CALL ETALL(HLAT,HLON,VLAT,VLON)
C   INPUT ARGUMENT LIST:
C     NONE
C
C   OUTPUT ARGUMENT LIST:
C     HLAT     - LATITUDE OF H GRID POINTS IN RADIANS (NEG=S)
C     HLON     - LONGITUDE OF H GRID POINTS IN RADIANS (E)
C     VLAT     - LATITUDE OF V GRID POINTS IN RADIANS (NEG=S)
C     VLON     - LONGITUDE OF V GRID POINTS IN RADIANS (E)
C
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN 77
C   MACHINE:  CRAY
c
      include "mpif.h"
      include "MPI_PARM.comm"
C-----------------------------------------------------------------------
                             D I M E N S I O N
     & IDAT  (3)
C
C     &,KHL0  (1000),KHH0  (1000), KVL0 (1000), KVH0 (1000)
C
C     &,GLATH (99999),GLONH (99999),GLATV (99999),GLONV (99999)
     &,GLATH (IM,JM),GLONH (IM,JM),GLATV (IM,JM),GLONV (IM,JM)
C
C     &,HLAT (99999),HLON (99999),VLAT (99999),VLON (99999)
     &,HLAT (IM,JM),HLON (IM,JM),VLAT (IM,JM),VLON (IM,JM)
C--------------LOGICAL FILE NAMES---------------------------------------
                             D A T A
     &  LIST  /03/
C--------------UNIVERSAL CONSTANTS--------------------------------------
                             D A T A
     & PI/3.141592654/
C-----------------------------------------------------------------------
 1000 FORMAT(' K=',I4,' GLAT=',E12.5,' GLON=',E12.5,' ELAT=',E12.5
     &,' ELON=',E12.5,' WLON = ',E12.5)
C----------------------------------------------------------------------
C--------------DERIVED GEOMETRICAL CONSTANTS----------------------------
C----------------------------------------------------------------------
      IMJM = IM * JM - JM/2
      DTR = PI / 180.0
      TPH0 = TPH0D * DTR
      WBD=-(IM-1)*DLMD
      SBD=-((JM-1)/2)*DPHD
      WB = WBD * DTR
      SB = SBD * DTR
      DLM = DLMD * DTR
      DPH = DPHD * DTR
      TDLM = DLM + DLM
      TDPH = DPH + DPH
C
      STPH0 = SIN(TPH0)
      CTPH0 = COS(TPH0)
C
C      DO 100 J = 1, JM
C        KHL0(J) = IM * (J-1) - (J-1)/2 + 1
C        KHH0(J) = IM * J - J / 2
C        IF(MOD(J,2) .EQ. 0) THEN
C          KVL0(J) = KHL0(J) - 1
C          KVH0(J) = KHH0(J)
C        ELSE
C          KVL0(J) = KHL0(J)
C          KVH0(J) = KHH0(J) - 1
C        END IF
C       WRITE(LIST,20) J,KHL0(J),KHH0(J),KVL0(J),KVH0(J)
C  20 FORMAT(2X,5I10)
C  100 CONTINUE
C-----------------------------------------------------------------------
C---COMPUTE GEOGRAPHIC LAT AND LONG OF ETA GRID POINTS (H & V POINTS)---
C-----------------------------------------------------------------------
      TPHH = SB - DPH
      DO 200 J = 1, JM
C         KHL = KHL0(J)
C         KHH = KHH0(J)
C         KVL = KVL0(J)
C         KVH = KVH0(J)
C
         TLMH = WB - TDLM + MOD(J+1,2) * DLM
         TPHH = TPHH + DPH
         TLMV = WB - TDLM + MOD(J,2) * DLM
         TPHV = TPHH
         STPH = SIN(TPHH)
         CTPH = COS(TPHH)
         STPV = SIN(TPHV)
         CTPV = COS(TPHV)
C----------------------------------------------------------------------
C---------- COMPUTE EARTH LATITUDE/LONGITUDE OF H POINTS --------------
C----------------------------------------------------------------------
c         DO 201 K = KHL, KHH
c           TLMH = TLMH + TDLM
c           SPHH = CTPH0 * STPH + STPH0 * CTPH * COS(TLMH)
c           GLATH(K) = ASIN(SPHH)
c           CLMH = CTPH * COS(TLMH) / (COS(GLATH(K)) * CTPH0)
c     1               - TAN(GLATH(K)) * TAN(TPH0)
c           IF(CLMH .GT. 1.) CLMH = 1.0
c           IF(CLMH .LT. -1.) CLMH = -1.0
c           FACTH = 1.
c           IF(TLMH .GT. 0.) FACTH = -1.
C          WRITE(3,88888) K, CLMH
C8888      FORMAT(2X,I6,1X,E12.5)
c           GLONH(K) = -TLM0D * DTR + FACTH * ACOS(CLMH)
C          IF(K .EQ. 1. OR. MOD(K,50). EQ. 0) THEN
C           WRITE(LIST,99995) J,K,GLATH(K),GLONH(K)
C9995       FORMAT(2X,2(I6,1X),2(E12.5,1X))
C          END IF
C
C    CONVERT INTO DEGREES AND EAST LONGITUDE
C
c           HLAT(K) = GLATH(K) / DTR
c           HLON(K) = 360.0 - GLONH(K) / DTR
c           IF(HLON(K) .GT. 360.) HLON(K) = HLON(K) - 360.
c  201    CONTINUE
         DO 201 I = 1, IM
           TLMH = TLMH + TDLM
           SPHH = CTPH0 * STPH + STPH0 * CTPH * COS(TLMH)
           GLATH(I,J) = ASIN(SPHH)
           CLMH = CTPH * COS(TLMH) / (COS(GLATH(I,J)) * CTPH0)
     1               - TAN(GLATH(I,J)) * TAN(TPH0)
           IF(CLMH .GT. 1.) CLMH = 1.0
           IF(CLMH .LT. -1.) CLMH = -1.0
           FACTH = 1.
           IF(TLMH .GT. 0.) FACTH = -1.
C          WRITE(3,88888) K, CLMH
C8888      FORMAT(2X,I6,1X,E12.5)
           GLONH(I,J) = (-TLM0D) * DTR + FACTH * ACOS(CLMH)
C          IF(K .EQ. 1. OR. MOD(K,50). EQ. 0) THEN
C           WRITE(LIST,99995) J,K,GLATH(K),GLONH(K)
C9995       FORMAT(2X,2(I6,1X),2(E12.5,1X))
C          END IF
C
C    CONVERT INTO DEGREES AND EAST LONGITUDE
C
           HLAT(I,J) = GLATH(I,J) / DTR
           HLON(I,J) = 360.0 - GLONH(I,J) / DTR
           IF(HLON(I,J) .GT. 360.) HLON(I,J) = HLON(I,J) - 360.
  201    CONTINUE
C----------------------------------------------------------------------
C---------- COMPUTE EARTH LATITUDE/LONGITUDE OF V POINTS --------------
C----------------------------------------------------------------------
c         DO 202 K = KVL, KVH
c           TLMV = TLMV + TDLM
c           SPHV = CTPH0 * STPV + STPH0 * CTPV * COS(TLMV)
c           GLATV(K) = ASIN(SPHV)
c           CLMV = CTPV * COS(TLMV) / (COS(GLATV(K)) * CTPH0)
c     1          - TAN(GLATV(K)) * TAN(TPH0)
c           IF(CLMV .GT. 1.) CLMV = 1.
c           IF(CLMV .LT. -1.) CLMV = -1.
c           FACTV = 1.
c           IF(TLMV .GT. 0.) FACTV = -1.
c           GLONV(K) = -TLM0D * DTR + FACTV * ACOS(CLMV)
C          IF(K .EQ. 1. OR. MOD(K,50). EQ. 0) THEN
C           WRITE(LIST,99995) J,K,GLATV(K),GLONV(K)
C          END IF
C
C    CONVERT INTO DEGREES AND EAST LONGITUDE
C
c           VLAT(K) = GLATV(K) / DTR
c           VLON(K) = 360.0 - GLONV(K) / DTR
c           IF(VLON(K) .GT. 360.) VLON(K) = VLON(K) - 360.
c  202    CONTINUE
         DO 202 I = 1, IM
           TLMV = TLMV + TDLM
           SPHV = CTPH0 * STPV + STPH0 * CTPV * COS(TLMV)
           GLATV(I,J) = ASIN(SPHV)
           CLMV = CTPV * COS(TLMV) / (COS(GLATV(I,J)) * CTPH0)
     1          - TAN(GLATV(I,J)) * TAN(TPH0)
           IF(CLMV .GT. 1.) CLMV = 1.
           IF(CLMV .LT. -1.) CLMV = -1.
           FACTV = 1.
           IF(TLMV .GT. 0.) FACTV = -1.
           GLONV(I,J) = (-TLM0D) * DTR + FACTV * ACOS(CLMV)
C          IF(K .EQ. 1. OR. MOD(K,50). EQ. 0) THEN
C           WRITE(LIST,99995) J,K,GLATV(K),GLONV(K)
C          END IF
C
C    CONVERT INTO DEGREES AND EAST LONGITUDE
C
           VLAT(I,J) = GLATV(I,J) / DTR
           VLON(I,J) = 360.0 - GLONV(I,J) / DTR
           IF(VLON(I,J) .GT. 360.) VLON(I,J) = VLON(I,J) - 360.
  202    CONTINUE
  200 CONTINUE
C
      RETURN
      END
