      SUBROUTINE PTETAE(IHINX,IHINY,ETOP,HF,TO,QO,UO,VO,CWMO,Q2O
     +, ui,vi,ti,qi,q2i,cldlin,htmo,vtmo,lmho,lmvo)
C
C       SUBPROGRAM DOCUMENTAION BLOCK
C
C    SUBROUTINE:  PTETAE              ETA-TO-ETA INTERPOLATION PROGRAM
C    AUTHOR: T BLACK                  ORG: W/NMC22        DATE: 95-07-14
C
C
C  ABSTRACT:  VERTICALLY INTERPOLATES ETA DATA AT ONE VERTICAL 
C             RESOLUTION TO ETA LAYERS AT ANOTHER RESOLUTION.
C             PROGRAM ASSUMES HORIZONTAL INTERPOLATION HAS BEEN
C             PERFORMED EARLIER.
C
C  REFERENCE:
C  BLACK, T. L., 1988: THE STEP-MOUNTAIN, ETA COORDINATE MODEL: A
C          DOCUMENTATION. DEVELOPMEMT DIVISON, NATIONAL METEOROLOGICAL
C          CENTER.
C
C  PROGRAM HISTORY LOG:
C    95-07-14  T BLACK WROTE CODE FOR ETA RESTRT-->ETA RESTRT ABILITY
C    01-01-30  H CHUANG MODIFY CODE TO ACCOMENDATE INTERPOLATION WHEN
C                       BDYS OF INPUT AND OUTPUT DOMAINS OVERLAP  
C    01-03-01  H CHUANG MODIFIED FOR MPI
C
C USAGE     CALL PTETAE(IHIN,ETOP)
C   INPUT ARGUMENT LIST:
C     IHIN     - INDEX OF THE H-POINT ON THE INPUT GRID CLOSEST TO 
C                THE H-POINT ON THE OUTPUT GRID
C     ETOP     - PRESSURE AT THE MODEL TOP IN MB
C
C    CALLED FROM PROGRAM INTRST
C
C---------------------------------------------------------------------
c      use mskout
      INCLUDE "parmeta.resin"
      INCLUDE "parmeta.resout"
      INCLUDE "griddef.out"
      INCLUDE "mpif.h"
      INCLUDE "MPI_PARM.comm"
C---------------------------------------------------------------------
C       IMJM=Number of grid points on output eta grid
C       LMI=Number of vertical levels on input eta grid
C       LMO=Number of vertical levels on output eta grid
C
                             P A R A M E T E R
     1 (IM=IMO,JM=JMO
     2, IMJM=IM*JM-JM/2,IMT=2*IM-1
     3, LMIP=LMI+1,LMIM=LMI-1,LMOP=LMO+1,LMOM=LMO-1
     4, KB=JM+2*IM-3
     6, LP1=LMO+1,LONG=IMJM*(6*LMO+3)+KB*(4*LMO*2+2)+2*LMO+LMOP)
C
                             P A R A M E T E R
     1 (CM1=2937.4,CM2=4.9283,CM3=23.5518,EPS=0.622
     2, RD=287.04,G=9.806,RG=1./G,EPSQ=2.E-12,EPSQ2=2.E-2
     3, B1=11.87799326,FCTR=0.75)
C--------------------------------------------------------------------
      PARAMETER(NBC=55)
                             L O G I C A L
     1 RUN,FIRST,HF
C
                             C H A R A C T E R
     1 LABEL*32
C
C   TEMPRES CONTAINS THE ENTIRE CONTENTS OF THE RESTART FILE
C   AFTER IT IS HORIZONTALLY INTERPOLATED TO THE TARGET OUTPUT
C   ETA GRID BUT BEFORE THE VERTICAL INTERPOLATION IS DONE
C
c      real,allocatable::ui(:,:,:)
c     +,vi,ti,qi,q2i,cldlin
c      REAL,ALLOCATABLE,DIMENSION(:,:,:)::PINTI,PMIDI
CWAS      REAL,ALLOCATABLE,DIMENSION(:,:,:)::dum3do,TO
cwas      REAL,ALLOCATABLE,DIMENSION(:,:,:)::ZETAI,ZETAO
c      REAL,ALLOCATABLE,DIMENSION(:,:)::LMHOI
c      REAL,ALLOCATABLE,DIMENSION(:,:,:)::HTMOI,dum3di
C--------------------------------------------------------------------
                             C O M M O N /DAT/
     1 IDAT(3) 
C-----------------------------------------------------------------------
C
C   OUTETA CONTAINS THOSE VARIABLES VERTICALLY INTERPOLATED TO THE
C   LMO ETA LEVELS OF THE OUTPUT GRID
C
                             C O M M O N /OUTETA/
     1 PDO(IMO,JMO),PDB(KB,2)
     5,TB (KB,LMO,2),QB (KB,LMO,2),UB (KB,LMO,2),VB (KB,LMO,2)
     6,PT,DETAO(LMO),AETAO(LMO),ETAO(LMOP),DFLO(LMOP)
     7,SM(IMO,JMO),RESO(IMO,JMO),FISO(IMO,JMO)
C--------------------------------------------------------------------
C   MSKIN IS THE TERRAIN MASKS FOR THE INPUT GRID
C   HTMI0,VTMI0 : INPUT GRID HEIGHT MASK AT H,V-POINTS (0=Below ground,
C                 1=above ground)
C   LMHI0,LMVI0 : INDEX OF THE LOWEST INPUT GRID ETA LEVEL ABOVE
C                 GROUND
C
                             C O M M O N /MSKIN/
     1 HTMI(IMI,JMI,LMI),VTMI(IMI,JMI,LMI),LMHI(IMI,JMI)
     2 ,LMVI(IMI,JMI)
C
C--------------------------------------------------------------------
C   MSKOUT IS THE TERRAIN MASKS FOR THE OUTPUT GRID
C   HTM0,VTM0 : OUTPUT GRID HEIGHT MASK AT H,V-POINTS (0=Below ground,
C                 1=above ground)
C   LMH0,LMV0 : INDEX OF THE LOWEST OUTPUT GRID ETA LEVEL ABOVE
C                 GROUND

c                             C O M M O N /MSKOUT/
c     1 HTMO(IMO,JMO,LMO),VTMO(IMO,JMO,LMO),
c     2 LMHO(IMO,JMO),LMVO(IMO,JMO)
C
C--------------------------------------------------------------------
                             D I M E N S I O N
     1  ETAI(LMIP), LDT1(LMO),DFLI(LMIP)
     2, DETAI(LMI),AETAI(LMI),PMVP(LMI)
     3, PINTI(IM,JM,LMIP),PMIDI(IM,JM,LMI)
c     &,ALPR(LMIP),ALP(LMIP),ALPSQ(LMIP),DLT(LMO)
     4,ZETAI(IM,JSTA:JEND,LMIP),ZETAO(IM,JSTA:JEND,LMOP)
     5,dum2do(im,jm)
C     6, ZSFCO(IM,JM),dum3di(im,jm,lmi)
     7, IHINX(IM,JM),IHINY(IM,JM), PDSLI(IM,JM)
     8,LMHOI(IM,JM),LMHO(IM,JSTA:JEND),LMVO(IM,JSTA:JEND)
     9,HTMOI(IM,JSTA:JEND,LMI)
     1, TO(IM,JSTA:JEND,LMO),QO(IM,JSTA:JEND,LMO)
     2, UO(IM,JSTA:JEND,LMO),VO(IM,JSTA:JEND,LMO)
     3, CWMO(IM,JSTA:JEND,LMO),Q2O(IM,JSTA:JEND,LMO)
     4, ui(IM,JSTA:JEND,LMI),vi(IM,JSTA:JEND,LMI)
     5, ti(IM,JSTA:JEND,LMI),qi(IM,JSTA:JEND,LMI)
     6, q2i(IM,JSTA:JEND,LMI),cldlin(IM,JSTA:JEND,LMI)
     7, HTMO(IM,JSTA:JEND,LMO),VTMO(IM,JSTA:JEND,LMO)
c     9, OUTCOM(LONG)
C-----------------------------------------------------------------------
Cmp (try more precision?)
        REAL(8) ALPETA, ARGLOG, DLT(LMO), ALPSFC, ARGLG2
        REAL(8) ALPR(LMIP),ALP(LMIP),ALPSQ(LMIP)
        REAL(8) ALPUB,ALPLB, CLOGES, ESE
c        REAL(8) PMIDI(IM,JM,LMI),PMVP(LMI), PINTI(IM,JM,LMIP)
c                             E Q U I V A L E N C E
c     & (PDO,OUTCOM)
C-----------------------------------------------------------------------
Cmp                             D A T A
	INTEGER IUN1,IUN2,IUN3	
      DATA IUN1 /16/,IUN2 /17/,IUN3 /18/
C      DATA IUN1 /16/,IUN2 /17/,IUN3 /48/
                        INCLUDE "TEMPRES"
C
C-----------------------------------------------------------------------
C&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
C-----------------------------------------------------------------------
C***
C***  DEFINITION OF CONSTANTS NEEDED FOR VERTICAL INTERPOLATION
C***

	write(6,*) 'IUN1,IUN2,IUN3= ', IUN1,IUN2,IUN3
      CP=1004.6
      P00=1.E5
      GAMMA=0.0065
      PRF0=101325.
      T0=288.
      GORG=G/(RD*GAMMA)
      RGOG=1./GORG
      ROG=RD/G
      CPOG=CP/G
      CAPA=RD/CP
      PT=ETOP*100.
      ALPT=ALOG(PT)
C-------------------------------------------------------------
c      REWIND IUN1
      REWIND IUN2
      REWIND IUN3
	write(6,*) 'IUN#s = ', IUN1,IUN2,IUN3
C
C  DETAI = ETA(L)-ETA(L-1) FOR THE INPUT GRID
C  DETAO = ETA(L)-ETA(L-1) FOR THE OUTPUT GRID
C  ZSFCO = STEP MOUNTAIN HEIGHT ON OUTPUT GRID
C  SM = LAND/SEA MASK ON OUTPUT GRID
C  REFO = VALUE OF ETA AT THE SURFACE ON THE OUTPUT GRID
C
      READ(IUN1)DETAI
      READ(IUN2)DETAO
      READ(IUN3)FISO,SM
      READ(IUN3)RESO
c      PRINT*,'TERRAIN HT AT (10,10)= ',ZSFCO(10,10)
	write(6,*) 'read from IUN1-3'
C
C     CONVERT REFO TO RESO FOR CNSTS AND ZSFCO TO FIS
C   
      DO J=JSTA,JEND
       DO I=1,IM
c        RESO(I,J)=REFO(I,J)
c	IF(I.EQ.1 .OR. J.EQ.1 .OR. I.EQ.IM .OR. J.EQ.JM)
c     +  PRINT*,'BDY RESO ZSFCO= ',I,J,RESO(I,J),ZSFCO(I,J)	
c        IF(I.EQ.1 .OR. J.EQ.1 .OR. I.EQ.IM .OR. J.EQ.JM)THEN
c         RESO(I,J)=1
c         REFO(I,J)=1
c         ZSFCO(I,J)=0.0
c        END IF 
c        FISO(I,J)=ZSFCO(I,J)*G
        FISO(I,J)=FISO(I,J)*G
       END DO	
      ENDDO
C   PRINT TI
c      PRINT*,'PRINTING TI'
c      DO I=1,IM
c      DO J=1,JM
c      DO K=1,LMI
c      IF(TI(I,J,K).LT.1.0 .and. k.lt.20)PRINT*,I,J,K,TI(I,J,K)
c      END DO
c      END DO
c      END DO
C-------------------------------------------------------------
C***
C***  COMPUTE ETA AT THE INTERFACES OF THE MODEL LAYERS
C***
      ETAI(1)=0.
C
      DO L=2,LMIP
        ETAI(L)=ETAI(L-1)+DETAI(L-1)
      ENDDO
C
      ETAI(LMIP)=1.
      DETAI(LMI)=1.-ETAI(LMI)
C
      ETAO(1)=0.
C
      DO L=2,LMOP
        ETAO(L)=ETAO(L-1)+DETAO(L-1)
      ENDDO
C
      ETAO(LMOP)=1.
      DETAO(LMO)=1.-ETAO(LMO)
C***
C***  COMPUTE AETA IN THE MIDDLES OF THE LAYERS
C***
      DO L=1,LMI
         AETAI(L)=0.5*(ETAI(L)+ETAI(L+1))
         print*,'AETAI =',L,AETAI(L)
      ENDDO
      DO L=1,LMO
         AETAO(L)=0.5*(ETAO(L)+ETAO(L+1))
         PRINT*,'AETAO =',L,AETAO(L)
      ENDDO
C----------------------------------------------------------------
C***
C***  COMPUTE THE HEIGHTS OF THE STANDARD INTERFACES
C***  FOR THE INPUT DOMAIN
C***
      DO L=1,LMIP
        DFLI(L)=G*T0*(1.-((PT+ETAI(L)*(PRF0-PT))/PRF0)**RGOG)/GAMMA
      ENDDO
C***
C***  COMPUTE THE LMH (INDEX OF LOWEST MODEL LEVEL ABOVE GROUND)
C***  FOR THE INPUT GRID VERTICAL LEVELS ON THE OUTPUT GRID POINTS
C***  BASED SOLEY UPON WHETHER OR NOT ANY PART OF AN OUTPUT
C***  LAYER IS ABOVE GROUND IN ANY INPUT LAYER
C***  USE INPUT GRID TEMPERATURES WHICH ARE ALWAYS < 0 BELOW GROUND
C***
cc      ALLOCATE(TI(IMO,JSTA:JEND,LMI))
cc      ALLOCATE(QI(IMO,JSTA:JEND,LMI))
cc      ALLOCATE(UI(IMO,JSTA:JEND,LMI)) 
cc      ALLOCATE(VI(IMO,JSTA:JEND,LMI))
cc      ALLOCATE(Q2I(IMO,JSTA:JEND,LMI))
cc      ALLOCATE(cldlin(IMO,JSTA:JEND,LMI))
c      ALLOCATE(dum3di(im,jm,lmi))
CWAS      open(80,file='ti.out',form='unformatted',status='unknown')
CWAS      do l=1,lmi 
CWAS       read(80)((DUM3DI(I,J,L),I=1,IM),J=1,JM)
CWAS      end do
CWAS      close(80) 
C      
c      ALLOCATE(LMHO(IM,JSTA:JEND))
c      ALLOCATE(LMVO(IM,JSTA:JEND))
      PRINT*,'H FLAG= ',HF
      IF(HF)THEN
       DO J=JSTA,JEND
       DO I=1,IM
        LMHOI(I,J)=LMHI(I,J)
       END DO
       END DO
      ELSE
       DO 50 J=JSTA,JEND
        DO 50 I=1,IM
         LMHOI(I,J)=LMI
         DO L=2,LMI
CWAS          IF(DUM3DI(I,J,L).LT.1.)THEN
          IF(TI(I,J,L).LT.1.)THEN
           LMHOI(I,J)=L-1
           GO TO 50
          ENDIF
         ENDDO
   50  CONTINUE
      END IF
      print*,'done specifying lmhoi'
C
      IF(HF)THEN
       DO J=JSTA,JEND
        DO I=1,IM
	 DO L=1,LMI
	  HTMOI(I,J,L)=HTMI(I,J,L)
	 END DO
	END DO
       END DO
      ELSE 	        
       DO 60 J=JSTA,JEND
        DO 60 I=1,IM
         DO L=2,LMIP
          HTMOI(I,J,L-1)=1.
          IF((FISO(I,J)-DFLI(L)).GT.1.E-2.AND.DFLI(L).GT.1.E-2)THEN
           DO LL=L,LMI
             HTMOI(I,J,LL)=0.
           ENDDO
           GO TO 60
          ENDIF
         ENDDO
   60  CONTINUE
      END IF  
      print*,'done specifying HTMOI'
C
C----------------------------------------------------------------
C***
C***  COMPUTE HEIGHTS OF THE INPUT ETA INTERFACES
C***
cwas      allocate(ZETAI(IM,JM,LMIP))
      DO J=JSTA,JEND
       DO I=1,IM
        PDSLI(I,J)=RESI(I,J)*PDI(I,J)
        ZETAI(I,J,LMIP)=0.0
       END DO
      ENDDO
c
C       
C***
C***  IF THE OUTPUT POINT IS ABOVE GROUND BUT THE INPUT POINT
C***  IS BELOW GROUND THEN:
C***    (1) IF ADJACENT INPUT POINTS ARE ABOVE GROUND THEN USE
C***        A WEIGHTED AVERAGE OF THOSE ABOVE-GROUND INPUT
C***        TEMPERATURES
C***    (2) IF NO ADJACENT INPUT POINTS ARE ABOVE GROUND THEN
C***        EXTRAPOLATE DOWNWARD FROM THE TWO INPUT LAYERS
C***        ABOVE GROUND AND USE AN ARBITRARY FACTOR TO CHANGE THE
C***        EXTRAPOLATION FROM PURELY LINEAR (FCTR=1 IS LINEAR)
C***
C USE PMIDI FOR QI FOR NOW BECAUSE PMIDI IS NOT USED TILL LATER
CWAS      ALLOCATE(PMIDI(IM,JM,LMI))
CWAS      open(80,file='qi.out',form='unformatted',status='unknown')
CWAS      do l=1,lmi 
CWAS       read(80)((PMIDI(I,J,L),I=1,IM),J=1,JM)
CWAS      end do
CWAS      close(80)
      DO L=1,LMI
         AETAI(L)=0.5*(ETAI(L)+ETAI(L+1))
      ENDDO
      DO L=LMI,1,-1
        DO J=JSTA,JEND
	 DO I=1,IM
	  APEL3=AETAI(L)*PDSLI(I,J)+PT
	  DPDE=DETAI(L)*PDSLI(I,J)
	  IF(HF)GO TO 61	 
          LMHIK=LMHOI(I,J)
          IF(IHINY(I,J).EQ.1 .OR. IHINY(I,J).EQ.JMI)THEN  !assume jmi odd
           HWGT=HTMI(IHINX(I,J),IHINY(I,J),L)
     1          +HTMI(IHINX(I,J)+1,IHINY(I,J),L)
          ELSE IF(IHINX(I,J).EQ.1 .OR. IHINX(I,J).EQ.IMI)THEN
           HWGT=HTMI(IHINX(I,J),IHINY(I,J)+1,L)
     1          +HTMI(IHINX(I,J),IHINY(I,J)-1,L)
	  ELSE IF(MOD(IHINY(I,J),2) .LT. 1)THEN
           HWGT=HTMI(IHINX(I,J)-1,IHINY(I,J),L)
     1	      +HTMI(IHINX(I,J),IHINY(I,J)+1,L)
     2        +HTMI(IHINX(I,J),IHINY(I,J),L)
     3        +HTMI(IHINX(I,J),IHINY(I,J)-1,L)
          ELSE
	   HWGT=HTMI(IHINX(I,J),IHINY(I,J),L)
     1	      +HTMI(IHINX(I,J),IHINY(I,J)+1,L)
     2        +HTMI(IHINX(I,J)+1,IHINY(I,J),L)
     3        +HTMI(IHINX(I,J),IHINY(I,J)-1,L)
          END IF
          IF(HWGT.LT.0.5.AND.HTMOI(I,J,L).GT.0.5)THEN
            APEL1=AETAI(LMHIK-1)*PDSLI(I,J)+PT
            APEL2=AETAI(LMHIK)*PDSLI(I,J)+PT
            TI(I,J,L)=TI(I,J,LMHIK)+(TI(I,J,LMHIK)-
     1      TI(I,J,LMHIK-1))*(APEL3-APEL2)/(APEL2-APEL1)*FCTR
          ENDIF
 61       CONTINUE	  
          RTOP=RD*TI(I,J,L)*(1.+0.608*QI(I,J,L))/APEL3
          FI=G*ZETAI(I,J,L+1)+RTOP*DPDE
          IF(HWGT.GT.0.5)HWGT=1.
          ZETAI(I,J,L)=((FI-DFLI(L))*HTMOI(I,J,L)+DFLI(L))*RG
         END DO
	ENDDO
      ENDDO
      print*,'done computing input height'
C
C
C      PRINT*,'PRINTING TI'
C      DO I=1,IM
C      DO J=1,JM
C      DO K=1,LMI
C      IF(TI(I,J,K).LT.1.0 .and. k.lt.20)PRINT*,I,J,K,TI(I,J,K)
C      END DO
C      END DO
C      END DO
C***
C***  FIND THE PRESSURE ON THE INPUT INTERFACES
C***
CWAS      ALLOCATE(PINTI(IM,JM,LMIP))
      DO J=JSTA,JEND
       DO I=1,IM
        PINTI(I,J,1)=PT
       END DO
      ENDDO
C
      DO L=2,LMIP
        DO J=JSTA,JEND
	 DO I=1,IM
          PINTI(I,J,L)=PDSLI(I,J)*ETAI(L)+PT
         END DO
        ENDDO
      ENDDO
C***
C***  FIND THE PRESSURE AT THE INPUT MIDLAYER LOCATIONS
C***
      DO L=1,LMI
        DO J=JSTA,JEND
	 DO I=1,IM
          PMIDI(I,J,L)=PINTI(I,J,L)+0.5*DETAI(L)*PDSLI(I,J)
         END DO
	ENDDO
      ENDDO
C-------------------------------------------------------------
C***
C***  VERTICAL INTERPOLATION: QUADRATIC INTERPOLATION OF
C***  HEIGHTS (EQUIVALENT TO TEMPERATURE BEING LINEAR IN LN P)
C***  AND LINEAR INTERPOLATION OF WINDS
C***
C-------------------------------------------------------------
C***
C***  COMPUTATION OF THE 'SEA LEVEL' PRESSURE DIFFERENCE
C***
CWAS      allocate(ZETAO(IM,JM,LMOP))
      DO 200 J=JSTA,JEND
       DO 200 I=1,IM
        DO LD=1,LMI
         ALPR(LD)=ALOG(PINTI(I,J,LD+1)/PINTI(I,J,LD))
c        ALPR(LD)=DLOG(PINTI(I,J,LD+1)/PINTI(I,J,LD))
        ENDDO
C
        DO LD=1,LMIP
         ALP(LD)=ALOG(PINTI(I,J,LD))
c         ALP(LD)=DLOG(PINTI(I,J,LD))
         ALPSQ(LD)=ALP(LD)**2
        ENDDO
C
C***
C***  ASSIGN THE VALUE OF PDSL AT THE NEAREST H POINT IN THE 
C***  INPUT DOMAIN TO PD IN THE OUTPUT DOMAIN
C***  WHERE PD IS EQUAL TO P AT ETA=1 MINUS PT.  
C***
C
      PDIJ=PDSLI(I,J)
C
C----------------------------------------------------------------------
C***
C***  COMPUTATION OF HEIGHTS AT UPPER BOUNDARIES OF ETA LAYERS 
C***  AND TEMPERATURE AND SPEC HUMIDITY AT MIDDLE OF ETA LAYERS
C***
C------------------------------------------------------
C***  THE FOLLOWING LINE IS ONLY TRUE IF PT (PRESSURE AT MODEL TOP IN
C***  PASCALS) IS THE SAME FOR BOTH THE INPUT AND THE OUTPUT
C------------------------------------------------------
      ZETAO(I,J,1)=ZETAI(I,J,1)
C
      KNTIN=1
      PETA=PT
      LDT1(1) = 1
C***
C***  LDT1 HOLDS THE INDEX OF THE INPUT ETA INTERFACE THAT IS
C***  DIRECTLY ABOVE THE OUTPUT ETA INTERFACE IN QUESTION.
C***
C      IF(I.EQ.10.AND.J.EQ.10)PRINT*,'L,ETA,PETA,PDIJ= ',1
C     + ,ETAO(1),PETA,PDIJ
      DO 100 L=2,LMO
        DETA1=DETAO(L-1)
	PETA=PETA+DETA1*PDIJ
C        IF(I.EQ.10.AND.J.EQ.10)PRINT*,'L,ETA,PETA,PDIJ= '
C     +	,L,ETAO(L),PETA,PDIJ
   90   CONTINUE
        IF(PETA.GT.PINTI(I,J,KNTIN).AND.PETA.LE.PINTI(I,J,KNTIN+1)
     1     .OR.PETA.LT.PINTI(I,J,2).OR.KNTIN.GE.LMIM)THEN
          LDT1(L)=KNTIN 
          GO TO 100
        ENDIF
        KNTIN=KNTIN+1
        GO TO 90
  100 CONTINUE
C
      DO L=1,LMO
        L1=LDT1(L)
        DLT(L)=ALPR(L1+1)*(-ALPR(L1))
     1	*ALOG(PINTI(I,J,L1+2)/PINTI(I,J,L1))
c        DLT(L)=ALPR(L1+1)*(-ALPR(L1))
c     1 *DLOG(PINTI(I,J,L1+2)/PINTI(I,J,L1))
      ENDDO    
C
      L =LMO
  125 L1=LDT1(L)
      L2=L1+1
      L3=L1+2
      H1=ZETAI(I,J,L1)
      H2=ZETAI(I,J,L2)
      H3=ZETAI(I,J,L3)
      H3M2=H3-H2
      H3M1=H3-H1
C***
C***  THE COEFFICIENTS FOR QUADRATIC INTERPOLATION OF HEIGHTS.
C***
      B=(H3M2*(ALPSQ(L3)-ALPSQ(L1))-H3M1*(ALPSQ(L3)-ALPSQ(L2)))/
     1        DLT(L)
      C=(H3M1*(ALP(L3)-ALP(L2))-H3M2*(ALP(L3)-ALP(L1)))/DLT(L)
C
  140 ARGLOG=PT+PDIJ*ETAO(L)
c      ALPETA=ALOG(ARGLOG)
      ALPETA=DLOG(ARGLOG)
      ARGLG2=PT+PDIJ*(ETAO(L)+0.5*DETAO(L))
c      ALPM=ALOG(ARGLG2)
      ALPM=DLOG(ARGLG2)
C***
C***  THE HEIGHTS OF THE OUTPUT ETA INTERFACES.
C***
      ZETAO(I,J,L)=H2+B*(ALPETA-ALP(L2))+C*(ALPETA**2-ALPSQ(L2))
C
      IF(L.EQ.2)GO TO 150
      L=L-1
      IF(LDT1(L).EQ.L1)GO TO 140
      GO TO 125
C
  150 PDO(I,J)=PDIJ
      ZETAO(I,J,LMOP)=0.
C
  200 CONTINUE
C COMPUTE TEMPERATURE HYDROSTATICLY
      DO 325 J=JSTA,JEND
       DO 325 I=1,IM
c       PHUB=0.
        PHUB=G*ZETAO(I,J,LMO)
C
c        DO L=LMO,1,-1
        DO L=LMO-1,1,-1
        PHLB=PHUB
        PHUB=G*ZETAO(I,J,L)
        TO(I,J,L)=-(PHLB-PHUB)*(PT+AETAO(L)*PDO(I,J))
     1          /(RD*DETAO(L)*PDO(I,J))
c        IF(I.EQ.81.AND.J.EQ.81)PRINT*,'H, TO IN PTETAE '
c     1	,L,DETAO(L),PDO(I,J),PHLB,PHUB,TO(I,J,L)
        if(TO(I,J,L).LT.150. .or. TO(I,J,L).gt.325.)print*,
     1  'enormal T',i,j,l,to(i,j,l)
        if(i.eq.48.and.j.eq.3.and.l.eq.23)
     1  print*,'HYDRO T(48,3,23)= ',TO(48,3,23)
	IF(TO(I,J,L).LT.150.)TO(I,J,L)=150.
        ENDDO
  325 CONTINUE
      print*,'done computing to in ptetae'
cwas      deallocate(ZETAO) 
C TEST...EXTRAPOLATE TO OBTAIN LOWEST LAYER TEMPERATURE
      DO 326 J=JSTA,JEND
       DO 326 I=1,IM
        APEL1=LOG(PT+AETAO(LMO-2)*PDO(I,J))
        APEL2=LOG(PT+AETAO(LMO-1)*PDO(I,J))
        APEL3=LOG(PT+AETAO(LMO)*PDO(I,J))
        TO(I,J,LMO)=TO(I,J,LMO-1)+(TO(I,J,LMO-1)-
     +  TO(I,J,LMO-2))*(APEL3-APEL2)/(APEL2-APEL1)
 326  CONTINUE
C--------------------------------------------------------------
C***
C***  COMPUTATION OF WINDS WITHIN ETA LAYERS
C***
C--------------------------------------------------------------
CWAS      open(80,file='ui.out',form='unformatted',status='unknown')
CWAS      do l=1,lmi 
CWAS       read(80)((DUM3DI(I,J,L),I=1,IM),J=1,JM)
CWAS      end do
CWAS      close(80)
CWAS      DO L=1,LMI
CWAS      print*,'L,UI(10,10,L)= ',L,DUM3DI(10,10,L)
CWAS      END DO
c      allocate(dum3do(im,jm,lmo))
CWAS      dum3do=0.
      print*,'start exchanging pmidi,pdo,reso'
      DO L=1,LMI
       CALL EXCH(PMIDI(1,1,L))
      END DO
c      if(mype.eq.0)then
c       do l=1,lmi
c       do i=1,im
c        pmidi(i,jend+1,l)=pmidi(i,jend,l)
c       end do
c       end do
c      else if(mype.eq.(num_procs-1))then
c       do l=1,lmi
c       do i=1,im
c        pmidi(i,jsta-1,l)=pmidi(i,jsta,l)
c       end do
c       end do
c      else
c       do l=1,lmi
c       do i=1,im
c        pmidi(i,jend+1,l)=pmidi(i,jend,l)
c        pmidi(i,jsta-1,l)=pmidi(i,jsta,l)
c       end do
c       end do
c      end if
      CALL EXCH(PDO)
      CALL EXCH(RESO)
      print*,'done exchanging pmidi,pdo,reso'
C DOESN'T HAVE TO USE JSTA_P BECAUSE SPECIAL ATTENTION IS TAKEN AT
C BDYS       
      DO 300 J=JSTA,JEND  
      DO 300 I=1,IM-MOD(j,2)
C
      DO 250 L=1,LMI
      IF(I.EQ.IM .AND. J.EQ.JM)GO TO 250
C***
C***  AVERAGE PRESSURE AT VELOCITY POINTS IN INPUT DATA
C***  ALONG THE LATERAL BOUNDARIES.
C***
C
      IF(J .EQ. 1 .AND. I .LT. IM)THEN
        PMVP(L)=0.5*(PMIDI(I,J,L)+PMIDI(I+1,J,L))
        IF(L.EQ.1)PDVP=0.5*(PDO(I,J)+PDO(I+1,J))
        RESV=0.5*(RESO(i,j)+RESO(i+1,j))
C
      ELSEIF(J.EQ.JM .AND. I.LT.IM)THEN
        PMVP(L)=0.5*(PMIDI(I,J,L)+PMIDI(I+1,J,L))
        IF(L.EQ.1)PDVP=0.5*(PDO(I,J)+PDO(I+1,J))
        RESV=0.5*(RESO(i,j)+RESO(i+1,j))
C
      ELSEIF(I .EQ. 1 .AND. MOD(J,2) .EQ. 0) THEN   !WESTERN BC
        PMVP(L)=0.5*(PMIDI(I,J-1,L)+PMIDI(I,J+1,L))
        IF(L.EQ.1)PDVP=0.5*(PDO(I,J-1)+PDO(I,J+1))
        RESV=0.5*(RESO(i,j-1)+RESO(i,j+1))
      ELSEIF(I .EQ. IM .AND. MOD(J,2) .EQ. 0) THEN  !EASTERN BC	
        PMVP(L)=0.5*(PMIDI(I,J-1,L)+PMIDI(I,J+1,L))
        IF(L.EQ.1)PDVP=0.5*(PDO(I,J-1)+PDO(I,J+1))
        RESV=0.5*(RESO(i,j-1)+RESO(i,j+1))
C
      ELSE IF (MOD(J,2) .LT. 1)THEN
C***
C***  PMVP AT ALL INTERIOR POINTS
C***
        PMVP(L)=0.25*(PMIDI(I,J,L)+PMIDI(I-1,J,L)+PMIDI(I,J+1,L)+
     1                PMIDI(I,J-1,L))
        IF(L.EQ.1)PDVP=0.25*(PDO(I,J)+PDO(I-1,J)+PDO(I,J+1)+
     1	PDO(I,J-1))
        RESV=0.25*(RESO(I,J)+RESO(I-1,J)+RESO(I,J+1)+
     1   RESO(I,J-1))
      ELSE
        PMVP(L)=0.25*(PMIDI(I,J,L)+PMIDI(I+1,J,L)+PMIDI(I,J+1,L)+
     1                PMIDI(I,J-1,L))
        IF(L.EQ.1)PDVP=0.25*(PDO(I,J)+PDO(I+1,J)+PDO(I,J+1)+
     1	PDO(I,J-1)) 
        RESV=0.25*(RESO(I,J)+RESO(I+1,J)+RESO(I,J+1)
     1  +RESO(I,J-1))
      ENDIF
C      IF(ABS(PDI(I,J)-PDO(I,J)).GT.5000.)PRINT*,'BAD TERRAIN MATCH '
C     + ,I,J,PDI(I,J),PDO(I,J),ZSFCO(I,J),pdvp
C
  250 CONTINUE
C
      DO LD=1,LMIM
        ALPR(LD)=ALOG(PMVP(LD+1)/PMVP(LD))
      ENDDO   
      DO LD=1,LMI
        ALP(LD)=ALOG(PMVP(LD))
      ENDDO
C
      ALPSFC=ALOG(ETAO(LMVO(I,J)+1)*PDVP+PT)
c      ALPSFC=ALOG(ETAO(LMHOI(I,J)+1)*PDVP+PT)
c      ALPSFC=ALOG(PT+PDVP*RESV)
C***
C***  INTERPOLATE THE WINDS
C***
      DO L=1,LMO
       UO(I,J,L)=0.0
      END DO
      DO 270 L=1,LMO
      IF(L.EQ.1)ALPLB=ALPT
      ALPUB=ALPLB
      ALPLB=ALOG(PT+PDVP*ETAO(L+1))
      ALPETA=SQRT(0.5*(ALPUB**2+ALPLB**2))
C
      DO 260 LD=2,LMI
      IF(ALPETA.GT.ALP(LD).AND.LD.LT.LMI)GO TO 260
      IF(ALPETA.LT.ALP(LD).AND.ABS(UI(I,J,LD)).GT.0.)THEN
        ULD=UI(I,J,LD)
        LDM1=LD-1
        CF=(ALP(LD)-ALPETA)/ALPR(LDM1)
        UK=ULD+(UI(I,J,LDM1)-ULD)*CF
        UTOP=UK
        ALPTU=ALPETA
        LTOP=L
        UO(I,J,L)=UK
        if(i.eq.149.and.j.eq.30)print*,'L,U(149,30,L)= '
     +, l,uo(i,j,l)
C        VO(I,J,L)=VK
        GO TO 270
C***
C***  IF THE NEW ETA GROUND SURFACE IS BELOW THE OLD GROUND, THEN
C***  ASSUME THE WIND COMPONENTS DECREASE TO ZERO AT THE SURFACE.
C***  THE SAME IS TRUE FOR ANY NEW ETA LEVEL BELOW THE LOWEST
C***  OLD ETA SURFACE.
C***  
      ELSEIF(ALPETA.GT.ALP(LMI).OR.UI(I,J,LD).EQ.0.)THEN
        if(i.eq.149.and.j.eq.30)print*,'underg at (149,30)',
     +  'l,ld,ui= ',l,ld,UI(I,J,LD)
        UDIF=UTOP
        ALPDIF=ALPTU-ALPSFC
        LT1=LTOP+1
        DO LL=LT1,LMVO(I,J)
c        DO LL=LT1,LMHOI(I,J)
          IF(LL.EQ.LT1)ALPLB=ALOG(PT+PDVP*ETAO(LL))
          ALPUB=ALPLB
          ALPLB=ALOG(PT+PDVP*ETAO(LL+1))
          ALPETA=SQRT(0.5*(ALPUB**2+ALPLB**2))
          UO(I,J,LL)=UTOP-UDIF*(ALPTU-ALPETA)/ALPDIF
	  UO(I,J,LL)=UO(I,J,LL)
          if(i.eq.149.and.j.eq.30)print*,'L,U(149,30,L)= '
     +,   ll,UO(I,J,LL)
C          VO(I,J,LL)=VTOP-VDIF*(ALPTU-ALPETA)/ALPDIF
        ENDDO
        GO TO 300
      ENDIF
  260 CONTINUE
C
  270 CONTINUE
C
  300 CONTINUE
ccc The following lines are commented out because vtmo is 
ccc not defined here to save memory
c      ALLOCATE(VTMO(IMO,JSTA:JEND,LMO))
      DO L=1,LMO
       DO J=JSTA,JEND
        DO I=1,IM
         if(i.eq.149.and.j.eq.30.and.l.eq.23)
     1   print*,'U(149,30,23) in PTETAE= ',UO(i,j,l)
          UO(I,J,L)=UO(I,J,L)*VTMO(I,J,L)
         ENDDO
	ENDDO
      ENDDO  
C
CCCC
CWAS      open(80,file='vi.out',form='unformatted',status='unknown')
CWAS      do l=1,lmi 
CWAS       read(80)((DUM3DI(I,J,L),I=1,IM),J=1,JM)
CWAS      end do
CWAS      do l=1,lmi
CWAS      print*,'L,VI(10,10,L)= ',L,DUM3DI(10,10,L)
CWAS      end do
CWAS      dum3do=0.
CC      CALL EXCH3D(PMIDI)
CC      CALL EXCH(PDO)
CC      CALL EXCH(RESO)
      DO 305 J=JSTA,JEND
      DO 305 I=1,IM-MOD(j,2)
C
      DO 255 L=1,LMI
      IF(I.EQ.IM .AND. J.EQ.JM)GO TO 255
C***
C***  AVERAGE PRESSURE AT VELOCITY POINTS IN INPUT DATA
C***  ALONG THE LATERAL BOUNDARIES.
C***
C
      IF(J .EQ. 1 .AND. I .LT. IM)THEN
        PMVP(L)=0.5*(PMIDI(I,J,L)+PMIDI(I+1,J,L))
        IF(L.EQ.1)PDVP=0.5*(PDO(I,J)+PDO(I+1,J))
        RESV=0.5*(RESO(i,j)+RESO(i+1,j))
C
      ELSEIF(J.EQ.JM .AND. I.LT.IM)THEN
        PMVP(L)=0.5*(PMIDI(I,J,L)+PMIDI(I+1,J,L))
        IF(L.EQ.1)PDVP=0.5*(PDO(I,J)+PDO(I+1,J))
        RESV=0.5*(RESO(i,j)+RESO(i+1,j))
C
      ELSEIF(I .EQ. 1 .AND. MOD(J,2) .EQ. 0) THEN   !WESTERN BC
        PMVP(L)=0.5*(PMIDI(I,J-1,L)+PMIDI(I,J+1,L))
        IF(L.EQ.1)PDVP=0.5*(PDO(I,J-1)+PDO(I,J+1))
        RESV=0.5*(RESO(i,j-1)+RESO(i,j+1))
      ELSEIF(I .EQ. IM .AND. MOD(J,2) .EQ. 0) THEN  !EASTERN BC	
        PMVP(L)=0.5*(PMIDI(I,J-1,L)+PMIDI(I,J+1,L))
        IF(L.EQ.1)PDVP=0.5*(PDO(I,J-1)+PDO(I,J+1))
        RESV=0.5*(RESO(i,j-1)+RESO(i,j+1))
C
      ELSE IF (MOD(J,2) .LT. 1)THEN
C***
C***  PMVP AT ALL INTERIOR POINTS
C***
        PMVP(L)=0.25*(PMIDI(I,J,L)+PMIDI(I-1,J,L)+PMIDI(I,J+1,L)+
     1                PMIDI(I,J-1,L))
        IF(L.EQ.1)PDVP=0.25*(PDO(I,J)+PDO(I-1,J)+PDO(I,J+1)+
     1	PDO(I,J-1))
        RESV=0.25*(RESO(I,J)+RESO(I-1,J)+RESO(I,J+1)+
     1   RESO(I,J-1))
      ELSE
        PMVP(L)=0.25*(PMIDI(I,J,L)+PMIDI(I+1,J,L)+PMIDI(I,J+1,L)+
     1                PMIDI(I,J-1,L))
        IF(L.EQ.1)PDVP=0.25*(PDO(I,J)+PDO(I+1,J)+PDO(I,J+1)+
     1	PDO(I,J-1)) 
        RESV=0.25*(RESO(I,J)+RESO(I+1,J)+RESO(I,J+1)
     1  +RESO(I,J-1))
      ENDIF
C      IF(ABS(PDI(I,J)-PDO(I,J)).GT.5000.)PRINT*,'BAD TERRAIN MATCH '
C     + ,I,J,PDI(I,J),PDO(I,J),ZSFCO(I,J),pdvp
C
  255 CONTINUE
C
      DO LD=1,LMIM
        ALPR(LD)=ALOG(PMVP(LD+1)/PMVP(LD))
      ENDDO   
      DO LD=1,LMI
        ALP(LD)=ALOG(PMVP(LD))
        if(i.eq.359.and.j.eq.361)print*,'alp at 259,361='
     +,ld,alp(ld)
      ENDDO
C
      ALPSFC=ALOG(ETAO(LMVO(I,J)+1)*PDVP+PT)
c      ALPSFC=ALOG(ETAO(LMHOI(I,J)+1)*PDVP+PT)
c      ALPSFC=ALOG(PT+PDVP*RESV) 
C***
C***  INTERPOLATE THE WINDS
C***
      DO L=1,LMO
       VO(I,J,L)=0.0
      END DO
      DO 275 L=1,LMO
      VO(I,J,L)=0.0
      IF(L.EQ.1)ALPLB=ALPT
      ALPUB=ALPLB
      ALPLB=ALOG(PT+PDVP*ETAO(L+1))
      ALPETA=SQRT(0.5*(ALPUB**2+ALPLB**2))
      if(l.eq.26.and.i.eq.359.and.j.eq.361)print*,
     +'alpeta at359,361,26=',alpeta
C
      DO 265 LD=2,LMI
      IF(ALPETA.GT.ALP(LD).AND.LD.LT.LMI)GO TO 265
      IF(ALPETA.LT.ALP(LD).AND.ABS(VI(I,J,LD)).GT.0.)THEN
        VLD=VI(I,J,LD)
        LDM1=LD-1
        CF=(ALP(LD)-ALPETA)/ALPR(LDM1)
        VK=VLD+(VI(I,J,LDM1)-VLD)*CF
        VTOP=VK
        ALPTU=ALPETA
        LTOP=L
        VO(I,J,L)=VK
C        if(i.eq.359.and.j.eq.360)print*,'v at 359,360=',l
C     +  ,alp(ld),alpeta,ALPR(LDM1),VLD,DUM3DI(I,J,LDM1),vk
C        if(i.eq.359.and.j.eq.361)print*,'v at 359,361=',l
C     +  ,alp(ld),alpeta,ALPR(LDM1),VLD,DUM3DI(I,J,LDM1),vk
        GO TO 275
C***
C***  IF THE NEW ETA GROUND SURFACE IS BELOW THE OLD GROUND, THEN
C***  ASSUME THE WIND COMPONENTS DECREASE TO ZERO AT THE SURFACE.
C***  THE SAME IS TRUE FOR ANY NEW ETA LEVEL BELOW THE LOWEST
C***  OLD ETA SURFACE.
C***  
      ELSEIF(ALPETA.GT.ALP(LMI).OR.VI(I,J,LD).EQ.0.)THEN
        VDIF=VTOP
        ALPDIF=ALPTU-ALPSFC
        LT1=LTOP+1
        DO LL=LT1,LMVO(I,J)
c        DO LL=LT1,LMHOI(I,J)
          IF(LL.EQ.LT1)ALPLB=ALOG(PT+PDVP*ETAO(LL))
          ALPUB=ALPLB
          ALPLB=ALOG(PT+PDVP*ETAO(LL+1))
          ALPETA=SQRT(0.5*(ALPUB**2+ALPLB**2))
          VO(I,J,LL)=VTOP-VDIF*(ALPTU-ALPETA)/ALPDIF
	  VO(I,J,LL)=VO(I,J,LL)
C          if(i.eq.359.and.j.eq.360)print*,'v- at 359,360=',ll
C     +,   alptu,alpeta,alpdif,vtop,vdif,dum3do(i,j,ll)
C          if(i.eq.359.and.j.eq.361)print*,'v- at 359,361=',ll
C     +,   alptu,alpeta,alpdif,vtop,vdif,dum3do(i,j,ll)
        ENDDO
        GO TO 305
      ENDIF
  265 CONTINUE
C
  275 CONTINUE
C
  305 CONTINUE  
C
      DO L=1,LMO
       DO J=JSTA,JEND
        DO I=1,IM
          VO(I,J,L)=VO(I,J,L)*VTMO(I,J,L)
         ENDDO
        ENDDO
      ENDDO
c      DEALLOCATE(VTMO)
C--------------------------------------------------------------------
C***
C***  INTERPOLATE Q AND CLD WATER LINEARLY IN LN(P)
C***
C
C        
C
      print*,'start computing cwm'
C
      DO 355 J=JSTA,JEND
       DO 355 I=1,IM
C
       DO LD=1,LMIM
        ALPR(LD)=ALOG(PMIDI(I,J,LD+1)/PMIDI(I,J,LD))
       ENDDO
       DO LD=1,LMI
        ALP(LD)=ALOG(PMIDI(I,J,LD))
       ENDDO
C
      LKO=LMHO(I,J)
c      LKO=LMO
      PSFCI=PDSLI(I,J)/RESI(I,J)+PT
      ALPLB=ALPT
      DO 345 L=1,LKO
      ALPUB=ALPLB
      ALPLB=ALOG(PT+PDO(I,J)*ETAO(L+1))
      ALPETA=SQRT(0.5*(ALPUB**2+ALPLB**2))
      PMIDO=AETAO(L)*PDO(I,J)+PT
      DO 335 LD=2,LMI
       IF(HF)THEN 
	HWGT=HTMI(I,J,LD)
       ELSE 
        IF(IHINY(I,J).EQ.1 .OR. IHINY(I,J).EQ.JMI)THEN  !assume jmi odd
         HWGT=HTMI(IHINX(I,J),IHINY(I,J),LD)
     1       +HTMI(IHINX(I,J)+1,IHINY(I,J),LD)
        ELSE IF(IHINX(I,J).EQ.1 .OR. IHINX(I,J).EQ.IMI)THEN
         HWGT=HTMI(IHINX(I,J),IHINY(I,J)+1,LD)
     1       +HTMI(IHINX(I,J),IHINY(I,J)-1,LD)        
        ELSE IF(MOD(IHINY(I,J),2) .LT. 1)THEN
         HWGT=HTMI(IHINX(I,J)-1,IHINY(I,J),LD)+HTMI(IHINX(I,J),
     1    IHINY(I,J)+1,LD)+HTMI(IHINX(I,J),IHINY(I,J),LD)
     2    +HTMI(IHINX(I,J),IHINY(I,J)-1,LD)
        ELSE
	 HWGT=HTMI(IHINX(I,J),IHINY(I,J),LD)+HTMI(IHINX(I,J),
     1    IHINY(I,J)+1,LD)+HTMI(IHINX(I,J)+1,IHINY(I,J),LD)
     2    +HTMI(IHINX(I,J),IHINY(I,J)-1,LD)
        END IF
       END IF 
	IF(HWGT.GT.0.5.AND.(ALPETA.LT.ALP(LD).OR.
     1                      ALPETA.GT.ALP(LD).AND.LD.EQ.LMI))THEN
        CWMLD=cldlin(I,J,LD)
        LDM1=LD-1
        CF=(ALP(LD)-ALPETA)/ALPR(LDM1)
        CWMO(I,J,L)=CWMLD+(cldlin(I,J,LDM1)-CWMLD)*CF
c        if(i.eq.71.and.j.eq.88.and.l.gt.20)print*,
c     1  'large cwmo at(71,88,l)',l,cwmld,DUM3DI(I,J,LDM1),CF,
c     2  ALP(LD),ALPETA,ALPR(LDM1),DUM3DO(I,J,L)
        CWMO(I,J,L)=AMAX1(CWMO(I,J,L),0.)
        LKI=LD
        GO TO 345
C***
C***  IF THE NEW ETA GROUND SURFACE IS BELOW THE OLD GROUND THEN
C***  ASSUME THE RH (GIVEN T) IS EQUAL TO THAT ON THE LOWEST
C***  OLD SURFACE AND THAT THE CLOUD WATER IS ZERO.
C***
      ELSEIF(HWGT.LT.0.5)THEN
        CWMO(I,J,L)=0.
c        if(i.eq.71.and.j.eq.88.and.l.gt.20)print*,
c     1  'large cwmo at(71,88,l)',l,cwmld,DUM3DI(I,J,LDM1),CF,
c     2  ALP(LD),ALPETA,ALPR(LDM1),DUM3DO(I,J,L)
        GO TO 345
      ENDIF
  335 CONTINUE
  345 CONTINUE
C  
C
  355 CONTINUE
      print*,'finish writing cwm'
c      ALLOCATE(HTMO(IMO,JSTA:JEND,LMO))
      DO L=1,LMO
       DO J=JSTA,JEND
        DO I=1,IMO
          CWMO(I,J,L)=CWMO(I,J,L)*HTMO(I,J,L)
         ENDDO
        ENDDO
      ENDDO
C
      print*,'start computing qi'
      DO 350 J=JSTA,JEND
       DO 350 I=1,IM
C
       DO LD=1,LMIM
        ALPR(LD)=ALOG(PMIDI(I,J,LD+1)/PMIDI(I,J,LD))
       ENDDO
       DO LD=1,LMI
        ALP(LD)=ALOG(PMIDI(I,J,LD))
       ENDDO
C
      LKO=LMHO(I,J)
c      LKO=LMO 
      PSFCI=PDSLI(I,J)/RESI(I,J)+PT
      ALPLB=ALPT
      DO 340 L=1,LKO
      ALPUB=ALPLB
      ALPLB=ALOG(PT+PDO(I,J)*ETAO(L+1))
      ALPETA=SQRT(0.5*(ALPUB**2+ALPLB**2))
      PMIDO=AETAO(L)*PDO(I,J)+PT
      DO 330 LD=2,LMI
       IF(HF)THEN 
	HWGT=HTMI(I,J,LD)
       ELSE
        IF(IHINY(I,J).EQ.1 .OR. IHINY(I,J).EQ.JMI)THEN  !assume jmi odd
         HWGT=HTMI(IHINX(I,J),IHINY(I,J),LD)
     1       +HTMI(IHINX(I,J)+1,IHINY(I,J),LD)
        ELSE IF(IHINX(I,J).EQ.1 .OR. IHINX(I,J).EQ.IMI)THEN
         HWGT=HTMI(IHINX(I,J),IHINY(I,J)+1,LD)
     1       +HTMI(IHINX(I,J),IHINY(I,J)-1,LD) 
        ELSE IF(MOD(IHINY(I,J),2) .LT. 1)THEN
         HWGT=HTMI(IHINX(I,J)-1,IHINY(I,J),LD)+HTMI(IHINX(I,J),
     1    IHINY(I,J)+1,LD)+HTMI(IHINX(I,J),IHINY(I,J),LD)
     2    +HTMI(IHINX(I,J),IHINY(I,J)-1,LD)
        ELSE
	 HWGT=HTMI(IHINX(I,J),IHINY(I,J),LD)+HTMI(IHINX(I,J),
     1    IHINY(I,J)+1,LD)+HTMI(IHINX(I,J)+1,IHINY(I,J),LD)
     2    +HTMI(IHINX(I,J),IHINY(I,J)-1,LD)
        END IF
       END IF 
	IF(HWGT.GT.0.5.AND.(ALPETA.LT.ALP(LD).OR.
     1                      ALPETA.GT.ALP(LD).AND.LD.EQ.LMI))THEN
        QLD=QI(I,J,LD)
        LDM1=LD-1
        CF=(ALP(LD)-ALPETA)/ALPR(LDM1)
        QETAIJ=QLD+(QI(I,J,LDM1)-QLD)*CF
        CLOGES=-CM1/TO(I,J,L)-CM2*ALOG10(TO(I,J,L))+CM3
        ESE=10.**(CLOGES+2.)
        QSX=EPS*ESE/(PMIDO-ESE*(1.-EPS))
        QSMX=0.98*QSX
        QO(I,J,L)=AMIN1(QETAIJ,QSMX)
        QO(I,J,L)=AMAX1(QO(I,J,L),EPSQ)
C       IF(QO(I,J,L).LT.QO(I,J,L-1).AND.LD.EQ.LMHI(I,J))QO(I,J,L)=QO(I,J,L-1)
C        CWMO(I,J,L)=AMAX1(CWMO(I,J,L),0.)
        LKI=LD
        GO TO 340
C***
C***  IF THE NEW ETA GROUND SURFACE IS BELOW THE OLD GROUND THEN
C***  ASSUME THE RH (GIVEN T) IS EQUAL TO THAT ON THE LOWEST
C***  OLD SURFACE AND THAT THE CLOUD WATER IS ZERO.
C***
      ELSEIF(HWGT.LT.0.5)THEN
      if(ti(I,J,LKI).lt.1.e-2)stop666
        CLOGES=-CM1/TI(I,J,LKI)-CM2*ALOG10(TI(I,J,LKI))+CM3
        ESS=10.**(CLOGES+2.)
        QSLDM=EPS*ESS/(PMIDI(I,J,LKI)-ESS*(1.-EPS))
        RHLDM=QI(I,J,LKI)/QSLDM
        RHLDM=AMIN1(RHLDM,0.98)
        CLOGES=-CM1/TO(I,J,L)-CM2*ALOG10(TO(I,J,L))+CM3
        ESL=10.**(CLOGES+2.)
        QSL=EPS*ESL/(AETAO(L)*PDO(I,J)+PT-ESL*(1.-EPS))
        QO(I,J,L)=RHLDM*QSL
C        CWMO(I,J,L)=0.
        GO TO 340
      ENDIF
  330 CONTINUE
  340 CONTINUE
C
      DO L=1,LMO
        IF(QO(I,J,L).LT.0..OR.L.GT.LKO)QO(I,J,L)=0.
        IF(QO(I,J,L).GT.25.E-3)QO(I,J,L)=25.E-3
      ENDDO   
C
  350 CONTINUE
      DO L=1,LMO
       DO J=JSTA,JEND
        DO I=1,IM
          QO(I,J,L)=QO(I,J,L)*HTMO(I,J,L)
         ENDDO
        ENDDO
      ENDDO
C***
C***  CONVERT FROM VIRTUAL TO TRUE TEMPERATURE.
C***
      DO 375 L=1,LMO
      DO 375 J=JSTA,JEND
      DO 375 I=1,IM
      IF(TO(I,J,L).GT.350.)TO(I,J,L)=350.
      IF(TO(I,J,L).LT.150.)TO(I,J,L)=150.
      TV=TO(I,J,L)
      DO ITER=1,3
        TO(I,J,L)=TV/(1.+0.608*QO(I,J,L))
        CLOGES=-CM1/TO(I,J,L)-CM2*ALOG10(TO(I,J,L))+CM3
        ESE=10.**(CLOGES+2.)
        DENOM2=AETAO(L)*PDO(I,J)+PT-ESE*(1.-EPS)
        QSX=EPS*ESE/DENOM2
        QSMX=0.98*QSX
        IF(QO(I,J,L).LT.QSMX)GO TO 375
        QO(I,J,L)=QSMX
      ENDDO
  375 CONTINUE
      if(mype.eq.0)print*,'after Q fix T(48,3,23)= ',TO(48,3,23)
      DO L=1,LMO
        DO J=JSTA,JEND 
	 DO I=1,IM
	  TO(I,J,L)=TO(I,J,L)*HTMO(I,J,L)
          if(l.eq.17)print*,'T AT L=17',i,j,l,TO(I,J,L)
         ENDDO
	ENDDO
      ENDDO  
      if(mype.eq.0)print*,'after terrain mask HTMO T(48,3,23)= ',
     + HTMO(48,3,23),TO(48,3,23)
CWAS      OPEN(70,file='qo.out',form='unformatted',status='unknown')
      do l=1,lmo  
       do J=JSTA,JEND
        do I=1,Im
         if(Qo(i,j,l).gt.0.1)print*,'enormal q',i,j
     +   ,Qo(i,j,l)
        end do 
       end do
CWAS       write(70)((DUM3DO(I,J,L),i=1,im),j=1,jm)
      end do
CWAS      close(70) 
CWAS      OPEN(70,file='to.out',form='unformatted',status='unknown')
      do l=1,lmo  
       do J=JSTA,JEND
        do I=1,Im
         if(to(i,j,l).gt.325.)print*,'enormal T',i,j,to(i,j,l)
        end do 
       end do 
CWAS       write(70)((TO(I,J,L),i=1,im),j=1,jm)
      end do
CWAS      close(70) 
C            
C
C***  SOUTHERN U POINTS
C***
      DO L=1,LMO
       CALL COLLECT_LOC(UO(1,JSTA,L),dum2do)
       IF(MYPE.EQ.0)THEN
       N=1
       DO I=1,IM-1
        UB(N,L,1)=DUM2DO(I,1)
        UB(N,L,2)=0.
        N=N+1
       END DO
C***
C***  NORTHERN U POINTS
C
       DO I=1,IM-1
        UB(N,L,1)=DUM2DO(I,JM)
        UB(N,L,2)=0.
        N=N+1
       END DO
C***
C***  WESTERN U POINTS
C***
       DO J=2,JM-1,2
        UB(N,L,1)=DUM2DO(1,J)
        UB(N,L,2)=0.
        N=N+1
       END DO
C***
C***  EASTERN U POINTS
C***
       DO J=2,JM-1,2
        UB(N,L,1)=DUM2DO(IM,J)
        UB(N,L,2)=0.
        N=N+1
       END DO
      END IF
      END DO
C
C***  COLLECT VB, SOUTHERN V POINTS
C***
      DO L=1,LMO
       CALL COLLECT_LOC(VO(1,JSTA,L),dum2do)
       IF(MYPE.EQ.0)THEN
       N=1
       DO I=1,IM-1
	VB(N,L,1)=DUM2DO(I,1)
        VB(N,L,2)=0.   
        N=N+1
       END DO
C***
C***  NORTHERN V POINTS
C
       DO I=1,IM-1
	VB(N,L,1)=DUM2DO(I,JM)
        VB(N,L,2)=0.
        N=N+1
       END DO
C***
C***  WESTERN V POINTS
C***
       DO J=2,JM-1,2
	VB(N,L,1)=DUM2DO(1,J)
        VB(N,L,2)=0.
        N=N+1
       END DO
C***
C***  EASTERN V POINTS
C***
       DO J=2,JM-1,2
	VB(N,L,1)=DUM2DO(IM,J)
        VB(N,L,2)=0.
        N=N+1
       END DO
      END IF
      END DO
C COLLECT PDB
      CALL COLLECT(PDO)
      IF(MYPE.EQ.0)THEN
       N=1
       DO I=1,IM
        PDB(N,1)=PDO(I,1)
        PDB(N,2)=0.
        N=N+1
       END DO
       DO I=1,IM
        PDB(N,1)=PDO(I,JM)
        PDB(N,2)=0
        N=N+1
       END DO
       DO J=3,JM-2,2
        PDB(N,1)=PDO(1,J)
        PDB(N,2)=0.
	N=N+1
       END DO
       DO J=3,JM-2,2
        PDB(N,1)=PDO(IM,J)
        PDB(N,2)=0.
	N=N+1
       END DO	
      END IF
C COLLECT TB
      DO L=1,LMO
       CALL COLLECT_LOC(TO(1,JSTA,L),DUM2DO)
       IF(MYPE.EQ.0)THEN
        N=1
        DO I=1,IM
         TB(N,L,1)=DUM2DO(I,1)
         TB(N,L,2)=0.
         N=N+1
        END DO
C***
C***  NORTHERN H POINTS
C***
        DO I=1,IM
         TB(N,L,1)=DUM2DO(I,JM)
         TB(N,L,2)=0.
         N=N+1
        END DO
C***
C***  WESTERN H POINTS
C***
        DO J=3,JM-2,2
         TB(N,L,1)=DUM2DO(1,J)
         TB(N,L,2)=0.
         N=N+1
        END DO
C***
C***  EASTERN H POINTS
C***
        DO J=3,JM-2,2
         TB(N,L,1)=DUM2DO(IM,J)
         TB(N,L,2)=0.
	 N=N+1
        END DO
       END IF
      END DO 
C COLLECT QB
      DO L=1,LMO
       CALL COLLECT_LOC(QO(1,JSTA,L),DUM2DO)
       IF(MYPE.EQ.0)THEN
        N=1
        DO I=1,IM
         QB(N,L,1)=DUM2DO(I,1)
         QB(N,L,2)=0.  
         N=N+1
        END DO
C***
C***  NORTHERN H POINTS
C***
        DO I=1,IM
         QB(N,L,1)=DUM2DO(I,JM)
         QB(N,L,2)=0.
         N=N+1
        END DO
C***
C***  WESTERN H POINTS
C***
        DO J=3,JM-2,2
         QB(N,L,1)=DUM2DO(1,J)
         QB(N,L,2)=0.
         N=N+1
        END DO
C***
C***  EASTERN H POINTS
C***
        DO J=3,JM-2,2
         QB(N,L,1)=DUM2DO(IM,J)
         QB(N,L,2)=0.
	 N=N+1
        END DO
       END IF
      END DO
C--------------------------------------------------------------------
C***
C***  INTERPOLATE Q2 LINEARLY IN LN(P) (BUT ON INTERFACES)
C***
      DO 450 J=JSTA,JEND
      DO 450 I=1,IM
C
      DO LD=1,LMI
        ALPR(LD)=ALOG(PINTI(I,J,LD+1)/PINTI(I,J,LD))
      ENDDO
      DO LD=1,LMIP
        ALP(LD)=ALOG(PINTI(I,J,LD))
      ENDDO
C
      LKO=LMHO(I,J)-1
c      LKO=LMO-1
      Q2SFC=AMAX1(B1**(2./3.)*USTAR(I,J)*USTAR(I,J),EPSQ2)
      Q2O(I,J,LKO+1)=Q2SFC
      ALPSFC=ALOG(PDO(I,J)+PT)
C      IF(I.EQ.37 .AND. J.EQ.49)
C     + PRINT*,'EPSQ2,Q2 AT SFC= ',LKO+1,EPSQ2,Q2O(I,J,LKO+1)
C
      DO 440 L=1,LKO
      ALPETA=ALOG(ETAO(L)*PDO(I,J)+PT)
C
      DO 430 LD=2,LMI
       IF(HF)THEN
        HWGT=HTMI(I,J,LD)
       ELSE
        IF(IHINY(I,J).EQ.1 .OR. IHINY(I,J).EQ.JMI)THEN  !assume jmi odd
         HWGT=HTMI(IHINX(I,J),IHINY(I,J),LD)
     1       +HTMI(IHINX(I,J)+1,IHINY(I,J),LD)
        ELSE IF(IHINX(I,J).EQ.1 .OR. IHINX(I,J).EQ.IMI)THEN
         HWGT=HTMI(IHINX(I,J),IHINY(I,J)+1,LD)
     1       +HTMI(IHINX(I,J),IHINY(I,J)-1,LD)
        ELSE IF(MOD(IHINY(I,J),2) .LT. 1)THEN
         HWGT=HTMI(IHINX(I,J)-1,IHINY(I,J),LD)+HTMI(IHINX(I,J),
     1   IHINY(I,J)+1,LD)+HTMI(IHINX(I,J),IHINY(I,J),LD)
     2   +HTMI(IHINX(I,J),IHINY(I,J)-1,LD)
        ELSE
	 HWGT=HTMI(IHINX(I,J),IHINY(I,J),LD)+HTMI(IHINX(I,J),
     1    IHINY(I,J)+1,LD)+HTMI(IHINX(I,J)+1,IHINY(I,J),LD)
     2    +HTMI(IHINX(I,J),IHINY(I,J)-1,LD)
        END IF
       END IF
      IF(HWGT.GT.0.5.AND.ALPETA.LT.ALP(LD))THEN
        Q2LD=Q2I(I,J,LD)
        LDM1=LD-1
        CF=(ALP(LD)-ALPETA)/ALPR(LDM1)
        Q2IJ=Q2LD+(Q2I(I,J,LDM1)-Q2LD)*CF
        Q2TOP=AMAX1(Q2IJ,EPSQ2)
        ALPT=ALPETA
        LTOP=L
        Q2O(I,J,L)=Q2TOP
        GO TO 440
C***
C***  IF THE NEW ETA GROUND SURFACE IS BELOW THE OLD GROUND THEN
C***  ASSUME Q2 DECREASES LINEARLY TO Q2 AT THE NEW GROUND.
C***
      ELSEIF(HWGT.LT.0.5)THEN
        Q2DIF=Q2TOP-Q2SFC
        ALPDIF=ALPT-ALPSFC
        LT1=LTOP+1
        DO LL=LT1,LKO
	  ALPETA=ALOG(ETAO(LL)*PDO(I,J)+PT)
          Q2O(I,J,LL)=Q2TOP-Q2DIF*(ALPT-ALPETA)/ALPDIF
        ENDDO
        GO TO 450
      ENDIF
  430 CONTINUE
  440 CONTINUE
C
  450 CONTINUE
c      DEALLOCATE(DUM3DI)
CWAS      DEALLOCATE(PINTI)
CWAS      DEALLOCATE(PMIDI)
C***  SET TO ZERO AT POINTS BELOW THE GROUND
C***
      DO L=1,LMO
       DO J=JSTA,JEND
        DO I=1,IM  
          Q2O(I,J,L)=Q2O(I,J,L)*HTMO(I,J,L)
         ENDDO
	ENDDO
      ENDDO
c      DEALLOCATE(HTMO)
C-----------------------------------------------------------
C***
C***  NOW REDEFINE PD TO HAVE IT EQUAL TO PSFC-PT.
C***
      DO J=JSTA,JEND
       DO I=1,IM
        PDO(I,J)=PDO(I,J)*RESO(I,J)
       END DO
      ENDDO
C-----------------------------------------------------------
C***
C
C      DO I=50,60
C       j=1
C       PRINT*,'PD, T, Q, U, V, AND Q2 ALONG S BOUNDARY ',I,J,PDO(I,J)
C     +  ,TO(I,1,50),QO(I,1,50),UO(I,1,50),VO(I,1,50),Q2O(I,1,50)
C       j=jm
C       PRINT*,'PD, T, Q, U, V, AND Q2 ALONG N BOUNDARY ',I,J,PDO(I,J)
C     + ,TO(I,JM,50),QO(I,JM,50),UO(I,JM,50),VO(I,JM,50),Q2O(I,JM,50)
C      END DO
C      DO J=50,60
C       i=1
C       PRINT*,'PD, T, Q, U, V, AND Q2 ALONG W BOUNDARY ',I,J,PDO(I,J)
C     +  ,TO(1,J,50),QO(1,J,50),UO(1,J,50),VO(1,J,50),Q2O(1,J,50)
C       i=im
C       PRINT*,'PD, T, Q, U, V, AND Q2 ALONG E BOUNDARY ',I,J,PDO(I,J)
C     + ,TO(IM,J,50),QO(IM,J,50),UO(IM,J,50),VO(IM,J,50),Q2O(IM,J,50)
C      END DO
C-----------------------------------------------------------
C***
C***  COMPUTE THE HEIGHTS OF THE STANDARD INTERFACES
C***  FOR THE OUTPUT DOMAIN
C***
      DO L=1,LMOP
        DFLO(L)=T0*(1.-((PT+ETAO(L)*(PRF0-PT))/PRF0)**RGOG)/GAMMA
      ENDDO
C----------------------------------------------------------------
C***
C***  TRANSLATE HBOT AND HTOP FROM INPUT LEVELS TO OUTPUT LEVELS
C***
      DO J=JSTA,JEND
       DO I=1,IM
        HBOTK=HBOT(I,J)
        HTOPK=HTOP(I,J)
        LMHOK=LMHO(I,J)
c        LMHOK=LMO
C
        DO 475 L=1,LMO
        IF(HBOTK.LT.LDT1(L))THEN
          HBOT(I,J)=L
          IF(HBOT(I,J).GE.LMHOK-1)HBOT(I,J)=LMHOK-1
          GO TO 480
        ENDIF
  475   CONTINUE
C            
  480   DO 485 L=1,LMO
        IF(HTOPK.LT.LDT1(L))THEN
          HTOP(I,J)=L
          GO TO 485
        ENDIF
  485   CONTINUE
C            
       END DO
      ENDDO
CCC WRITING BOUNDARY CONDITIONS TO NBC FILE FOR BCTEND TO USE LATER
      IF(MYPE.EQ.0)THEN
       WRITE(NBC)RUN,IDAT,IHRST
       WRITE(NBC)PDB,TB,QB,UB,VB
      END IF
CCCC      
C------------------------------------------------------------
c      DEALLOCATE(LMHO)
c      DEALLOCATE(LMVO)
cc      DEALLOCATE(TI)
cc      DEALLOCATE(QI)
cc      DEALLOCATE(UI) 
cc      DEALLOCATE(VI)
cc      DEALLOCATE(Q2I)
cc      DEALLOCATE(cldlin)
      RETURN
      END
