      SUBROUTINE ROTLL1(U,V,VLAT,VLON,UOUT,VOUT)
C
C SUBPROGRAM: ROTLLE        ROTATE WINDS ON LAT/LONG GRID TO E-GRID
C   PRGMMR: T.BLACK         ORG: W/NMC22    DATE: ??-??-??
C
C ABSTRACT: ROTATES WINDS ON THE LAT/LONG GRID TO THE ETA MODEL GRID
C
C PROGRAM HISTORY LOG:
C   ??-??-??  T.BLACK
C
C USAGE     CALL ROTLLE(U,V,VLAT,VLON)
C   INPUT ARGUMENT LIST:
C     U        - LAT/LONG U-COMPONENT
C     V        - LAT/LONG V-COMPONENT
C     VLAT     - LATITUDE OF E-GRID V POINTS (DEGREES)
C     VLON     - LONGITUDE OF E-GRID V POINTS (DEGREES)
C
C   OUTPUT ARGUMENT LIST:
C     U        - ETA GRID U-COMPONENT
C     V        - ETA GRID V-COMPONENT
C
C ATTRIBUTES:
C   LANGUAGE: FORTRAN 77
C   MACHINE:  CRAY
C
C***
C*** ROTATE THE LAT-LON WINDS TO THE E-GRID
C***
C
C    N O T E : INPUT LAT/LONG MUST BE CONVERTED TO RADIANS !!!
C
            include "parmeta.resin"
            include "parmeta.resout"
            include "griddef.out"
	    INCLUDE "mpif.h"
            INCLUDE "MPI_PARM.comm"
                         P A R A M E T E R
     1 (LMAX=LMI)
                         D I M E N S I O N
     1 U(IMO,JMO),V(IMO,JMO),VLAT(IMO,JMO),VLON(IMO,JMO)
     2,UOUT(IMO,JMO),VOUT(IMO,JMO)
C----------------------------------------------------------------------
      pi=3.141592654
      d2r=pi/180.
      orphi0 = tph0do * d2r
      orlam0 = (360. + tlm0do) * d2r
      SPHI0 = SIN(ORPHI0)
      CPHI0 = COS(ORPHI0)
      DO 100 J = JSTA, JEND
       DO 100 I = 1, IMO
        TLAT = VLAT(I,J) * D2R
        TLON = VLON(I,J) * D2R
        RELM = TLON - ORLAM0
        SRLM = SIN(RELM)
        CRLM = COS(RELM)
        SPH = SIN(TLAT)
        CPH = COS(TLAT)
        CC = CPH * CRLM
        TPH = ASIN(CPHI0 * SPH - SPHI0 * CC)
        RCTPH = 1.0 / COS(TPH)
        CRAY = SPHI0 * SRLM * RCTPH
        DRAY = (CPHI0 * CPH + SPHI0 * SPH * CRLM) * RCTPH
C        RU = DRAY * U(L) - CRAY * V(L)
C        RV = CRAY * U(L) + DRAY * V(L)
C        UOUT(L) = RU
C        VOUT(L) = RV
        RU = DRAY * U(I,J) - CRAY * V(I,J)
        RV = CRAY * U(I,J) + DRAY * V(I,J)
        UOUT(I,J) = RU
        VOUT(I,J) = RV
  100 CONTINUE
C----------------------------------------------------------------------
      RETURN
      END
