      SUBROUTINE ETA2HYB(PTOP,KBOMX,LMOMX,NBCEX
     1,                  IMOUTPUT,JMOUTPUT,LMOUTPUT
     2,                  DETAI,IUDETAO,IUFISO
     3,                  PDBI,FISBI,TBI,QBI,UBI,VBI,Q2BI,CWMBI
     4,                  HTMBI,USTARBI
     5,                  PDBO,TBO,QBO,UBO,VBO,Q2BO,CWMBO)
C----------------------------------------------------------------------
C
C$$$  SUBPROGRAM DOCUMENTAION BLOCK
C
C   SUBROUTINE:  ETA2HYB     ETA-TO-HYBRID BC INTERPOLATION
C   PRGRMMR: BLACK           ORG: W/NP22     DATE: 01-10-16
C
C  ABSTRACT:  VERTICALLY INTERPOLATES ETA DATA TO HYBRID LAYERS.
C             PROGRAM ASSUMES HORIZONTAL INTERPOLATION HAS BEEN
C             PERFORMED EARLIER.
C
C  REFERENCE:
C  BLACK, T. L., 1988: THE STEP-MOUNTAIN, ETA COORDINATE MODEL: A
C          DOCUMENTATION. DEVELOPMEMT DIVISION, NATIONAL METEOROLOGICAL
C          CENTER.
C
C  PROGRAM HISTORY LOG:
C    99-12-22  H CHUANG MODIFIED CODE TO INTERPOLATE IC RESTART
C              FILES FROM ETA TO SIGMA   
C    01-03-01  H CHUANG - MODIFIED FOR MPI
C    01-07-01  Z JANJIC - MODIFIED FOR PRESSURE-SIGMA HYBRID
C    01-10-16  T BLACK - MODIFIED FOR BOUNDARY EXTRACTION
C
C USAGE:    CALL ETA2HYB FROM BCEX
C
C   INPUT ARGUMENT LIST:
C       PTOP - THE TOP PRESSURE OF THE DOMAIN IN PASCALS
C      KBOMX - THE MAXIMUM LENGTH OF ANY NEST BOUNDARY'S HORIZONTAL
C              EXTENT
C      LMOMX - THE MAXIMUM NUMBER OF MODEL LEVELS FOR ANY NEST
C      NBCEX - THE NUMBER OF NESTS
C        IMO - THE ARRAY OF IM's FOR ALL NESTS
C        JMO - THE ARRAY OF JM's FOR ALL NESTS
C       LMO0 - THE ARRAY OF LM's FOR ALL NESTS
C      DETAI - DELTA ETAs FOR PARENT GRID
C    IUDETAO - THE UNIT NUMBER OF THE NEST DETA's
C       PDBI - THE PD ARRAY FOR ALL NEST BOUNDARIES AFTER
C              HORIZONTAL INTERPOLATION FROM THE OPERATIONAL GRID
C      FISBI - THE FIS ARRAY FOR ALL NEST BOUNDARIES AFTER
C              HORIZONTAL INTERPOLATION FROM THE OPERATIONAL GRID
C        TBI - THE TEMPERATURE ARRAY FOR ALL NEST BOUNDARIES AFTER
C              HORIZONTAL INTERPOLATION FROM THE OPERATIONAL GRID
C        QBI - THE SPEC HUMIDITY ARRAY FOR ALL NEST BOUNDARIES AFTER
C              HORIZONTAL INTERPOLATION FROM THE OPERATIONAL GRID
C        UBI - THE U COMPONENT ARRAY FOR ALL NEST BOUNDARIES AFTER
C              HORIZONTAL INTERPOLATION FROM THE OPERATIONAL GRID
C        VBI - THE V COMPONENT ARRAY FOR ALL NEST BOUNDARIES AFTER
C              HORIZONTAL INTERPOLATION FROM THE OPERATIONAL GRID
C       Q2BI - THE TKE ARRAY FOR ALL NEST BOUNDARIES AFTER
C              HORIZONTAL INTERPOLATION FROM THE OPERATIONAL GRID
C      CWMBI - THE CLOUD WATER ARRAY FOR ALL NEST BOUNDARIES AFTER
C              HORIZONTAL INTERPOLATION FROM THE OPERATIONAL GRID
C      HTMBI - THE HEIGHT MASK ARRAY FOR ALL NEST BOUNDARIES AFTER
C              HORIZONTAL INTERPOLATION FROM THE OPERATIONAL GRID
C    USTARBI - THE USTAR ARRAY FOR ALL NEST BOUNDARIES AFTER
C              HORIZONTAL INTERPOLATION FROM THE OPERATIONAL GRID
C
C   OUTPUT ARGUMENT LIST:
C       PDBO - THE FINAL PD ARRAY FOR ALL NEST BOUNDARIES
C        TBO - THE FINAL TEMP ARRAY FOR ALL NEST BOUNDARIES
C        QBO - THE FINAL SPEC HUMIDITY ARRAY FOR ALL NEST BOUNDARIES
C        UBO - THE FINAL U COMPONENT ARRAY FOR ALL NEST BOUNDARIES
C        VBO - THE FINAL V COMPONENT ARRAY FOR ALL NEST BOUNDARIES
C       Q2BO - THE FINAL TKE ARRAY FOR ALL NEST BOUNDARIES
C      CWMBO - THE FINAL CLOUD WATER ARRAY FOR ALL NEST BOUNDARIES
C
C----------------------------------------------------------------------
      INCLUDE "parmeta"
C----------------------------------------------------------------------
                             P A R A M E T E R
     1 (LMI=LM,LMIP=LM+1,LMIM=LM-1)
C
                             P A R A M E T E R
     1 (CM1=2937.4,CM2=4.9283,CM3=23.5518,EPS=0.622
     2, RD=287.04,G=9.806,RG=1./G,EPSQ=2.E-12,EPSQ2=2.E-2
     3, B1=11.87799326,FCTR=0.75)
C----------------------------------------------------------------------
                             L O G I C A L
     1 SAME
C----------------------------------------------------------------------
                             R E A L
     1 DSG (LMOMX),SGML (LMOMX),SGM(LMOMX+1)
     2,DSG1(LMOMX),SGML1(LMOMX),SG1(LMOMX+1)
     3,DSG2(LMOMX),SGML2(LMOMX),SG2(LMOMX+1)
C----------------------------------------------------------------------
                             R E A L
ccc  1  PDBO(KBOMX,NBCEX),PMIDO(KBOMX,NBCEX)
     1  PDBO(KBOMX,NBCEX)
     2, TBO(KBOMX,LMOMX,NBCEX),QBO(KBOMX,LMOMX,NBCEX)
     3, UBO(KBOMX,LMOMX,NBCEX),VBO(KBOMX,LMOMX,NBCEX)
     4, Q2BO(KBOMX,LMOMX,NBCEX),CWMBO(KBOMX,LMOMX,NBCEX)
C
                               R E A L
     1  PDBI(KBOMX,NBCEX),FISBI(KBOMX,NBCEX)
     2, TBI(KBOMX,LM,NBCEX),QBI(KBOMX,LM,NBCEX)
     3, UBI(KBOMX,LM,NBCEX),VBI(KBOMX,LM,NBCEX)
     4, Q2BI(KBOMX,LM,NBCEX),CWMBI(KBOMX,LM,NBCEX)
     5, HTMBI(KBOMX,LM,NBCEX),USTARBI(KBOMX,NBCEX)
C
                             R E A L
     1  ETAI(LMIP),DFLI(LMIP)
     2, ETAO(LMOMX),DETAO(LMOMX),AETAO(LMOMX)
     2, DETAI(LMI),AETAI(LMI)
     3, PINTI(KBOMX,LMIP),PMIDI(KBOMX,LMI)
     4, ZETAI(KBOMX,LMIP),ZETAO(KBOMX,LMOMX+1)
     5, PDSLI(KBOMX),PSNEW(KBOMX)
     6, RESI(KBOMX),HTMOI(KBOMX,LMI)
     7, TLEV(LMIP)
     8, PMVP(KBOMX,2),PDVP(KBOMX)
C
                             R E A L
     1  FISBO(KBOMX),RESO(KBOMX)
C
                             I N T E G E R
     1  LDT1(LMOMX),LMHOI(KBOMX),LEVATM(KBOMX)
     2, K1(KBOMX),K2(KBOMX)
C
                             I N T E G E R
     1  IMOUTPUT(NBCEX),JMOUTPUT(NBCEX),LMOUTPUT(NBCEX)
C
        REAL(8) ALPETA, ARGLOG, DLT(LMOMX), ALPSFC, ARGLG2
        REAL(8) ALPR(KBOMX,LMIP),ALP(KBOMX,LMIP),ALPSQ(KBOMX,LMIP)
        REAL(8) ALPUB,ALPLB, CLOGES, ESE
C
        CHARACTER(8) :: DETAFIL,FISFIL
        real tsum(lmomx)
C----------------------------------------------------------------------
        REAL,ALLOCATABLE,DIMENSION(:,:) :: FISO
C----------------------------------------------------------------------
C&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&
C----------------------------------------------------------------------
C***
C***  DEFINITION OF CONSTANTS NEEDED FOR VERTICAL INTERPOLATION
C***
C----------------------------------------------------------------------
      GAMMA=0.0065
      PRF0=101325.
      T0=288.
      GORG=G/(RD*GAMMA)
      RGOG=1./GORG
      PT=PTOP
      ALPT=ALOG(PT)
C----------------------------------------------------------------------
C----------------------------------------------------------------------
C***
C***  THE GRAND LOOP OVER EACH OF THE NESTS
C***
C----------------------------------------------------------------------
C----------------------------------------------------------------------
      DO 1000 NB=1,NBCEX
C----------------------------------------------------------------------
      IMO=IMOUTPUT(NB)
      JMO=JMOUTPUT(NB)
      LMO=LMOUTPUT(NB)
      KBO=2*IMO+JMO-3
C
      ALLOCATE(FISO(IMO,JMO),STAT=I)
C----------------------------------------------------------------------
      CLOSE(IUDETAO)
      WRITE(DETAFIL,10)NB
   10 FORMAT('deta.',I2.2)
      OPEN(UNIT=IUDETAO,FILE=DETAFIL,FORM='UNFORMATTED',IOSTAT=IER)
      REWIND IUDETAO
      READ(IUDETAO)(DSG(L),L=1,LMO)
C
      CLOSE(IUFISO)
      WRITE(FISFIL,12)NB
   12 FORMAT('fis.',I2.2)
      OPEN(UNIT=IUFISO,FILE=FISFIL,FORM='UNFORMATTED',IOSTAT=IER)
      REWIND IUFISO
      READ(IUFISO)FISO
C
C***  OUTPUT IS SIGMA SO RESO SHOULD EQUAL 1.0 EVERYWHERE
C
      DO K=1,KBO
        RESO(K)=1.
      ENDDO
C----------------------------------------------------------------------
C
C***  EXTRACT FIS ALONG THE NEST BOUNDARY
C
      N=1
C
      DO I=1,IMO
        FISBO(N)=FISO(I,1)*G
        N=N+1
      ENDDO
C
      DO I=1,IMO
        FISBO(N)=FISO(I,JMO)*G
      if(i.ge.1.and.i.le.36)then
        write(0,*)' i=',i+179,' fisbo=',fiso(i,jmo)
      endif
        N=N+1
      ENDDO
C
      DO J=3,JMO-2,2
        FISBO(N)=FISO(1,J)*G
        N=N+1
      ENDDO
C
      DO J=3,JMO-2,2
        FISBO(N)=FISO(IMO,J)*G
        N=N+1
      ENDDO
C
      DEALLOCATE(FISO,STAT=IER)
!----------------------------------------------------------------------
!***
!***  COMPUTE ETAI AT THE INTERFACES OF THE MODEL LAYERS
!***
      ETAI(1)=0.
C
      DO L=2,LMI+1
        ETAI(L)=ETAI(L-1)+DETAI(L-1)
      ENDDO
C
      ETAI(LMI+1)=1.
      DETAI(LMI)=1.-ETAI(LMI)
!***
!***  COMPUTE AETAI IN THE MIDDLES OF THE LAYERS
!***
      DO L=1,LMI
        AETAI(L)=0.5*(ETAI(L)+ETAI(L+1))
      ENDDO
!----------------------------------------------------------------------
!  This is the first time the hybrid coordinate appears in the job
!  stream, and that is why it is defined here.  Away from the mountains,
!  the depths of the layers may be the same as those of the input sigma
!  coordinate, or may be defined independently.
!----------------------------------------------------------------------
      SAME=.TRUE.
!----------------------------------------------------------------------
      IF(SAME)THEN
!----------------------------------------------------------------------
        DO L=1,LMO
          DETAO(L)=DSG(L)
        ENDDO
C
        SGM(1)=0.0
C
        DO L=2,LMO+1
          SGM(L)=SGM(L-1)+DSG(L-1)
          ETAO(L)=SGM(L)
        ENDDO
C
        SGM(LMO+1)=1.0
        ETAO(LMO+1)=1.0
        DSG(LMO)=1.0-SGM(LMO)
        DETAO(LMO)=1.0-ETAO(LMO)
C
        DO L=1,LMO
          SGML(L)=0.5*(SGM(L)+SGM(L+1))
          AETAO(L)=SGML(L)
        ENDDO 
!-----------------------------------------------------------------------
      ELSE
!-------COMPUTING OF SG LEVELS ACCORDING TO EMPIRICAL FORMULA-----------
        X0=0.10*PI
        X1=(1.300)*PI
        DX=(X1-X0)/(LMO-1)
!
        AA=1.
        BB=-(AA-1.)/(X1-X0)
!
        X=X0-DX
        SUM=0.
C
        DO L=1,LMO
          X=X+DX
          DSGL=AA+BB*(X-X0)+SIN(X)
          DSG(L)=DSGL
          SUM=SUM+DSGL
        ENDDO
C
        RSUM=1./SUM
        SGM(1)=0.
C
        DO L=1,LMO-1
          DSG(L)=DSG(L)*RSUM
          SGM(L+1)=SGM(L)+DSG(L)
        ENDDO
C
        SGM(LMO+1)=1.
        DSG(LMO)=SGM(LMO+1)-SGM(LMO)
C
        DO L=1,LMO
          SGML(L)=(SGM(L)+SGM(L+1))*0.5
        ENDDO
!-----------------------------------------------------------------------
      ENDIF
!-----------------------------------------------------------------------
!***  PTSGM IS THE APPROXIMATE PRESSURE AT THE BOUNDARY BETWEEN PRESSURE 
!***  AND SIGMA IN Pa.
!-----------------------------------------------------------------------
      PTSGM=42000. ! Pa
      LPT2=0
C
      DO L=1,LMO+1
        PL=SGM(L)*(101300.-PT)+PT
        IF(PL.LT.PTSGM)LPT2=l
      ENDDO
C
      IF(LPT2.GT.0)THEN
        PT2=SGM(LPT2)*(101300.-PT)+PT
      ELSE
        PT2=PT
      ENDIF
C
      print*,'*** Sigma system starts at ',pt2,' Pa, from level ',lpt2
C
      PDTOP=PT2-PT
C
      DO L=1,LMO
        IF(L.LE.LPT2)THEN
          DSG1(L)=DSG(L)
        ELSE
          DSG2(L)=DSG(L)
        ENDIF
      ENDDO
!-----------------------------------------------------------------------
      SG1(1)=0.
C
      IF(LPT2.GT.0)THEN
        DO L=1,LPT2
          SG1(L+1)=SG1(L)+DSG1(L)
        ENDDO
C
        DO L=2,LPT2
          SG1(L)=SG1(L)/SG1(LPT2+1)
        ENDDO
C
        SG1(LPT2+1)=1.
C
        DO L=LPT2+2,LMO+1
          SG1(L)=1.
        ENDDO
C
        DO L=1,LPT2
          DSG1(L)=SG1(L+1)-SG1(L)
          SGML1(L)=(SG1(L)+SG1(L+1))*0.5
        ENDDO
      ENDIF
C
      DO L=LPT2+1,LMO
        DSG1(l)=0.
        SGML1(l)=1.
      ENDDO
!
      DO L=1,LPT2+1
        SG2(L)=0.
      ENDDO
C
      DO L=LPT2+1,LMO
        SG2(L+1)=SG2(L)+DSG2(L)
      ENDDO
C
      DO L=LPT2+2,LMO
        SG2(L)=SG2(L)/SG2(LMO+1)
      ENDDO
C
      SG2(LMO+1)=1.
C
      DO L=LPT2+1,LMO
        DSG2(L)=SG2(L+1)-SG2(L)
        SGML2(L)=(SG2(L)+SG2(L+1))*0.5
      ENDDO
C
      DO L=1,LPT2
        DSG2(L)=0.
        SGML2(L)=0.
      ENDDO
!-----------------------------------------------------------------------
c     write(*,*)' nb=', nb
c     write(*,*)'dsg=', dsg
c     write(*,*)'sgm=', sgm
c     write(*,*) 'sgml= ',sgml
c     write(*,*) 'dsgl= ',dsg1
c     write(*,*) 'sgl= ',sg1
c     write(*,*) 'sgml1= ',sgml1
c     write(*,*) 'dsg2= ',dsg2
c     write(*,*) 'sg2= ',sg2
c     write(*,*) 'sgml2= ',sgml2
!-----------------------------------------------------------------------
c     open(unit=8,file='deta_hyb'
c    1    ,status='unknown',form='unformatted')
c     write(8) dsg,sgm,pdtop,dsg1,sg1,dsg2,sg2
c     close(8)
C----------------------------------------------------------------
C***
C***  COMPUTE THE HEIGHTS OF THE STANDARD INTERFACES
C***  FOR THE INPUT DOMAIN
C***
      DO L=1,LMIP
        DFLI(L)=G*T0*(1.-((PT+ETAI(L)*(PRF0-PT))/PRF0)**RGOG)/GAMMA
      ENDDO
C***
C***  COMPUTE THE LMH (INDEX OF LOWEST MODEL LEVEL ABOVE GROUND)
C***  FOR THE INPUT GRID VERTICAL LEVELS ON THE OUTPUT GRID POINTS
C***  BASED SOLEY UPON WHETHER OR NOT ANY PART OF AN OUTPUT
C***  LAYER IS ABOVE GROUND IN ANY INPUT LAYER
C***  USE INPUT GRID TEMPERATURES WHICH ARE ALWAYS < 0 BELOW GROUND
C***
C
      DO 50 K=1,KBO
      LMHOI(K)=LMI
      RESI(K)=1.
      DO L=2,LMI
ccc     IF(TBI(K,L,NB).LT.1.)THEN
c     if(k.eq.584)then
c       write(0,*)' l=',l,' htmbi=',htmbi(k,l,nb)
c    1,           ' tbi=',tbi(k,l,nb),' etai=',etai(l)
c     endif
        IF(HTMBI(K,L,NB).LT.1.)THEN
          LMHOI(K)=L-1
c     if(k.eq.584)then
c       write(0,*)' in loop lmhoi=',lmhoi(k)
c     endif
          RESI(K)=1./ETAI(L)
          GO TO 50
        ENDIF
      ENDDO
   50 CONTINUE
C
      DO 60 K=1,KBO
      DO L=2,LMIP
        HTMOI(K,L-1)=1.
        IF((FISBO(K)-DFLI(L)).GT..01.AND.DFLI(L).GT..01)THEN
          DO LL=L,LMI
            HTMOI(K,LL)=0.
          ENDDO
          GO TO 60
        ENDIF
      ENDDO
   60 CONTINUE
C
C----------------------------------------------------------------
C***
C***  COMPUTE HEIGHTS OF THE INPUT ETA INTERFACES
C***
C
      DO K=1,KBO
        IF(LMHOI(K).EQ.0)THEN
          RESI(K)=1.
        ENDIF
      ENDDO
C
      DO K=1,KBO
        PDSLI(K)=RESI(K)*PDBI(K,NB)    
        if(k.ge.180.and.k.le.215)then
        write(0,*)' k=',k,' pdsli=',pdsli(k)
     1,           ' resi=',resi(k),' pdbi=',pdbi(k,nb)
        endif
      ENDDO
C       
C***
C***  IF THE OUTPUT POINT IS ABOVE GROUND BUT THE INPUT POINT
C***  IS BELOW GROUND THEN:
C***    (1) IF ADJACENT INPUT POINTS ARE ABOVE GROUND THEN USE
C***        A WEIGHTED AVERAGE OF THOSE ABOVE-GROUND INPUT
C***        TEMPERATURES
C***    (2) IF NO ADJACENT INPUT POINTS ARE ABOVE GROUND THEN
C***        EXTRAPOLATE DOWNWARD FROM THE TWO INPUT LAYERS
C***        ABOVE GROUND AND USE AN ARBITRARY FACTOR TO CHANGE THE
C***        EXTRAPOLATION FROM PURELY LINEAR (FCTR=1 IS LINEAR)
C***
      DO L=1,LMI
        AETAI(L)=0.5*(ETAI(L)+ETAI(L+1))
      ENDDO
C
C***  EXTRAPOLATE INPUT TEMPERATURE DOWNWARD 1 TO 3 LAYERS IN CASE 
C***  SIGMA TERRAIN FALLS UNDER ETA TERRAIN
C
      DO K=1,KBO
        LMHIK=LMHOI(K)
	IF(LMHIK.LT.LMI)THEN
          APEL1=ALOG(AETAI(LMHIK-5)*PDSLI(K)+PT)
          APEL2=ALOG(AETAI(LMHIK)*PDSLI(K)+PT)
          LEND=MIN(LMI,LMHIK+4)
          DO L=LMHIK+1,LEND
            APEL3=ALOG(AETAI(L)*PDSLI(K)+PT)
            TBI(K,L,NB)=TBI(K,LMHIK,NB)+(TBI(K,LMHIK,NB)-
     1               TBI(K,LMHIK-5,NB))*(APEL3-APEL2)/(APEL2-APEL1)
C
C**  USE STANDARD LAPSE RATE TO EXTRAPOLATE TEMPERATURE
C
            HTMOI(K,L)=1.0	  
          ENDDO
        ENDIF
      ENDDO
C            
      DO L=LMI,1,-1
	DO K=1,KBO
	  ZETAI(K,LMIP)=0.0
	  APEL3=AETAI(L)*PDSLI(K)+PT
	  DPDE=DETAI(L)*PDSLI(K)
C
          LMHIK=LMHOI(K)
C
          IF(HTMBI(K,L,NB).LT.0.5.AND.HTMOI(K,L).GT.0.5)THEN
            APEL1=AETAI(LMHIK-5)*PDSLI(K)+PT
            APEL2=AETAI(LMHIK)*PDSLI(K)+PT
            TBI(K,L,NB)=TBI(K,LMHIK,NB)+
     1                 (TBI(K,LMHIK,NB)-TBI(K,LMHIK-5,NB))*
     2                 (APEL3-APEL2)/(APEL2-APEL1)*FCTR
	  ENDIF	  
 61       CONTINUE	  
C
	  RTOP=RD*TBI(K,L,NB)*(1.+0.608*QBI(K,L,NB))/APEL3
          FI=G*ZETAI(K,L+1)+RTOP*DPDE
          ZETAI(K,L)=((FI-DFLI(L))*HTMOI(K,L)+DFLI(L))*RG	  
        ENDDO
      ENDDO	
C***
C***  FIND THE PRESSURE ON THE INPUT INTERFACES
C***
      DO K=1,KBO
        PINTI(K,1)=PT
      ENDDO
C
      DO L=2,LMIP
        DO K=1,KBO
          PINTI(K,L)=PDSLI(K)*ETAI(L)+PT
        ENDDO
      ENDDO
C***
C***  FIND THE PRESSURE AT THE INPUT MIDLAYER LOCATIONS
C***
      DO L=1,LMI
        DO K=1,KBO
          PMIDI(K,L)=PINTI(K,L)+0.5*DETAI(L)*PDSLI(K)
c     if(k.eq.455)then
c       write(0,*)' l=',l,' pmidi=',pmidi(k,l),' pinti=',pinti(k,l)
c    1,           ' lmhoi=',lmhoi(k)
c       write(0,*)' detai=',detai(l),' pdsli=',pdsli(k)
c     endif
	ENDDO
      ENDDO
C
C***  RECOMPUTE SURFACE PRESSURE BASED ON SIGMA TERRAIN HEIGHT
C***  FIRST, FIRST PRESSURE LEVEL IN THE ETA ANALYSIS ABOVE GROUND
C***  IF SURFACE IS < 7.5 MB FROM PRESSURE SURFACE
C***  USE THE NEXT HIGHEST ONE TO PREVENT EXTRAPOLATION
C
      DO K=1,KBO
	PSTAR=PT+PDBI(K,NB)
        DO L=LMIP,1,-1
C
          IF(PSTAR.GE.PINTI(K,L))THEN
            PDIFF=PSTAR-PINTI(K,L)
            LEVATM(K)=L
            IF(PDIFF.LE.7.5)LEVATM(K)=L-1
	    GO TO 65 
          ENDIF
        ENDDO
 65     CONTINUE
      ENDDO
C
      DO 180 K=1,KBO
C
      DO L=1,LMIP
        ALP(K,L)=LOG(PINTI(K,L)) 	 
      ENDDO 
C
      KSTART=LEVATM(K)
      PSTAR=PT+PDBI(K,NB)
C
      DO L=1,KSTART
        TLEV(L)=TBI(K,L,NB)
      ENDDO
C
      L=LEVATM(K)
 170  CONTINUE
C
      ZLO=ZETAI(K,L)        
      TLO=TLEV(L)
      L=L-1
      IF((FISBO(K)/G).GT.ZETAI(K,L))GO TO 170
C
      ALNPB=ALP(K,L+1)
      ALNPT=ALP(K,L)
      DLNP=ALNPB-ALNPT
      ALNP3=0.5*(ALNPB+ALNPT) 
      A=G*(ZETAI(K,L)-ZLO)/DLNP
      B=RD*(TLO-TLEV(L))/DLNP
      PHI3=G*0.5*(ZLO+ZETAI(K,L))+0.125*B*DLNP*DLNP
      RADICL=A*A-2.*B*(FISBO(K)-PHI3) 
C
      IF(RADICL.LE.0.)THEN
        write(6,120)RADICL,K,NB,A,B,FISBO(K)/g,K
  120   FORMAT(' radicl<0,J,NB,A,B,ZS,K ',G15.5,2I4,3G15.5,I5)
        GO TO 172
      ENDIF
C
      ALNPS=ALNP3-2.*(FISBO(K)-PHI3)/
     1     (A+SQRT(RADICL))
      PSNEW(K)=EXP(ALNPS)	
      if(k.ge.180.and.k.le.215)then
        write(0,*)' A k=',k,' psnew=',psnew(k)
      endif
c     if(k.eq.584)then
c       write(0,*)' A psnew=',psnew(k),' fisbo=',fisbo(k)
c    1,           ' phi3=',phi3
c       write(0,*)' radicl=',radicl,' a=',a,' alnp3=',alnp3
c     endif
      GO TO 174
C
C***  NEST TERRAIN IS LOWER THAN PARENT TERRAIN
C         
  172 PSNEW(K)=EXP(ALP(K,L+1)-(ALP(K,L)-ALP(K,L+1))*(ZETAI(K,L+1)
     1         -(FISBO(K)/G))/(ZETAI(K,L)-ZETAI(K,L+1))) 
      if(k.ge.180.and.k.le.215)then
        write(0,*)' B k=',k,' psnew=',psnew(k)
      endif
c     if(k.eq.584)then
c       write(0,*)' B psnew=',psnew(k),' fisbo=',fisbo(k)
c       write(0,*)' zetai=',zetai(k,l),zetai(k,l+1)
c       write(0,*)' alp=',alp(k,l),alp(k,l+1)
c     endif
      print*,'NEW TERRAIN IS LOWER THAN OLD TERRAIN'
      print*,'K,NB,ETA SIGMA PSFC= ',K,NB,PSTAR,PSNEW(K)
  174 CONTINUE
C
  180 CONTINUE	
C
C-------------------------------------------------------------
C***
C***  VERTICAL INTERPOLATION: QUADRATIC INTERPOLATION OF
C***  HEIGHTS (EQUIVALENT TO TEMPERATURE BEING LINEAR IN LN P)
C***  AND LINEAR INTERPOLATION OF WINDS
C***
C-------------------------------------------------------------
C***
C***  COMPUTATION OF THE 'SEA LEVEL' PRESSURE DIFFERENCE
C***
      DO 200 K=1,KBO
C
      DO LD=1,LMI
        ALPR(K,LD)=ALOG(PINTI(K,LD+1)/PINTI(K,LD))
      ENDDO
C
      DO LD=1,LMIP
        ALP(K,LD)=ALOG(PINTI(K,LD))
        ALPSQ(K,LD)=ALP(K,LD)**2
      ENDDO
C
C***
C***  ASSIGN THE VALUE OF PRESSURE DIFFERENCE AT THE NEAREST  
C***  H POINT IN THE INPUT DOMAIN TO PD IN THE OUTPUT DOMAIN
C***  WHERE PD IS EQUAL TO P AT ETA=1 MINUS PT.  
!
!***  THE FOLLOWING LINE IS USED WHEN USING SIGMA OUTPUT TERRAIN
!
      PDIJ=PSNEW(K)-PDTOP-PT
!
!***  IF INPUT ETA TERRAIN ALREADY ON SEA LEVEL AND PDBO>PDBI =>PDBI=PDBO
!
c     if(k.eq.584)then
c       write(0,*)' 1st k=',k,' pdo=',pdij
c       write(0,*)' lmhoi=',lmhoi(k),' pdbi=',pdbi(k,nb)
c    1,           ' pdtop=',pdtop,' psnew=',psnew(k)
c     endif
cccc  IF(LMHOI(K).EQ.LMI.AND.FISBO(K).LT.1.)THEN
      IF(FISBI(K,NB).LT.1.0.AND.FISBO(K).LT.1.)THEN
c     if(k.eq.584)then
c       write(0,*)' 2nd k=',k,' pdbo=',pdij
c     endif
        PDIJ=PDBI(K,NB)-PDTOP
      ENDIF
      if(k.ge.180.and.k.le.215)then
        write(0,*)' k=',k,' pdij=',pdij,' fisbi=',fisbi(k,nb)
     1,           ' fisbo=',fisbo(k)
      endif
C----------------------------------------------------------------------
C***
C***  COMPUTATION OF HEIGHTS AT UPPER BOUNDARIES OF ETA LAYERS 
C***  AND TEMPERATURE AND SPEC HUMIDITY AT MIDDLE OF ETA LAYERS
C***
C----------------------------------------------------------------------
C***  THE FOLLOWING LINE IS ONLY TRUE IF PT (PRESSURE AT MODEL TOP IN
C***  PASCALS) IS THE SAME FOR BOTH THE INPUT AND THE OUTPUT
C----------------------------------------------------------------------
      ZETAO(K,1)=ZETAI(K,1)
!
      KNTIN=1
      PETA=PT
      LDT1(1)=1
!***
!***  LDT1 HOLDS THE INDEX OF THE INPUT ETA INTERFACE THAT IS
!***  DIRECTLY ABOVE THE OUTPUT ETA INTERFACE IN QUESTION.
!***
      DO 100 L=2,LMO
        PETA=SG1(L)*PDTOP+SG2(L)*PDIJ+PT
   90   CONTINUE
C
        IF(PETA.GT.PINTI(K,KNTIN).AND.PETA.LE.PINTI(K,KNTIN+1)
     1      .OR.PETA.LT.PINTI(K,2).OR.KNTIN.GE.LMIM)THEN
          LDT1(L)=KNTIN 
          GO TO 100
        ENDIF
C
        KNTIN=KNTIN+1
        GO TO 90
  100 CONTINUE
!
      DO L=1,LMO
        L1=LDT1(L)
        DLT(L)=ALPR(K,L1+1)*(-ALPR(K,L1))*ALOG(PINTI(K,L1+2)/
     1                                         PINTI(K,L1)) 
      ENDDO
!
!*** start height interpolation from bottom up
!
      L=LMO
!
  125 L1=LDT1(L)
      L2=L1+1
      L3=L1+2
      H1=ZETAI(K,L1)
      H2=ZETAI(K,L2)
      H3=ZETAI(K,L3)
      H3M2=H3-H2
      H3M1=H3-H1
!***
!***  THE COEFFICIENTS FOR QUADRATIC INTERPOLATION OF HEIGHTS
!***
      B=(H3M2*(ALPSQ(K,L3)-ALPSQ(K,L1))-H3M1*(ALPSQ(K,L3)-ALPSQ(K,L2)))
     1        /DLT(L)
      C=(H3M1*(ALP(K,L3)-ALP(K,L2))-H3M2*(ALP(K,L3)-ALP(K,L1)))/DLT(L)
!
  140 ARGLOG=SG1(L)*PDTOP+SG2(L)*PDIJ+PT
      ALPETA=DLOG(ARGLOG)
      ARGLG2=SGML1(L)*PDTOP+SGML2(L)*PDIJ+PT  ! p at mid-hyb level
      ALPM=DLOG(ARGLG2)
!***
!***  THE HEIGHTS OF THE OUTPUT HYBRID INTERFACES
!***
      ZETAO(K,L)=H2+B*(ALPETA-ALP(K,L2))+C*(ALPETA**2-ALPSQ(K,L2)) 
!
      IF(L.EQ.2)GO TO 150
      L=L-1
      IF(LDT1(L).EQ.L1)GO TO 140
      GO TO 125
!
  150 PDBO(K,NB)=PDIJ
      IF(PDBO(K,NB).LT.0.1)print*,'enormal pdo',k,nb,pdbo(k,nb)
      ZETAO(K,LMO+1)=FISBO(K)/G
!
      IF(ZETAO(K,LMO+1).GT.ZETAO(K,LMO))print*,'terrain higher
     1       than zetao(lmo) zsfco,zetao(lm) ',i,j,fisbo(k)/g
     2,       zetao(k,lmo) 
!
  200 CONTINUE
!--------------------------------------------------------------
!***
!***  COMPUTATION OF WINDS WITHIN SIGMA LAYERS (MID-LAYER)
!***
!--------------------------------------------------------------
      IMT=2*IMO-1
      JMT=JMO/2+1
C***
C***  ARRANGEMENT OF LOCATIONS IN THE V BOUNDARY FILE
C***
C                       2ND
C           IBBP--------->----------ITB
C           ILB                     KBI
C            |                       |
C       3RD /|\                     /|\ 4TH
C            |                       |
C           IMT                     ILBP
C            1----------->----------IBB
C                       1ST
      IBB=IMT/2
      IBBP=IBB+1
      ITB=IMT-1
      ILB=IMT+JMT-2
      ILBP=ILB+1
      ILBP2=ILB+2
      ILBM=ILB-1
      IMTP=IMT+1
      KBM=KBO-1
C***
C***  FIND AVERAGE PD OVER VELOCITY BOUNDARY POINTS
C***
      DO K=1,IBB
        K1(K)=K
        K2(K)=K+1
      ENDDO
      DO K=IBBP,ITB
        K1(K)=K+1
        K2(K)=K+2
      ENDDO
      DO K=IMTP,ILBM
        K1(K)=K+1
        K2(K)=K+2
      ENDDO
      DO K=ILBP2,KBM
        K1(K)=K
        K2(K)=K+1
      ENDDO
      K1(IMT)=1
      K2(IMT)=IMT+2
      K1(ILB)=ILB+1
      K2(ILB)=IBB+2
      K1(ILBP)=IBB+1
      K2(ILBP)=ILBP2
      K1(KBO)=KBO
      K2(KBO)=ITB+2
C***
C***  AVERAGE THE PRESSURE AT VELOCITY POINTS 
C***  AT MID-LAYERS ON THE INPUT GRID
C***  AND FOR PDB ON THE OUTPUT GRID.
C***
c     DO K=1,KBO
c       PMIDO(K,NB)=PT+SGML1(1)*PDTOP+SGML2(1)*PDBO(K,NB)
c     ENDDO
C
      DO 210 K=1,KBO
      K1K=K1(K)
      K2K=K2(K)
      PDVP(K)=0.5*(PDBO(K1K,NB)+PDBO(K2K,NB))
c     if(k.eq.583)then
c       write(0,*)' k1=',k1k,' k2=',k2k,' pdvp=',pdvp(k)
c    1,           ' pdbo=',pdbo(k1k,nb),pdbo(k2k,nb)
c     endif
      PMVP(K,1)=0.5*(PMIDI(K1K,1)+PMIDI(K2K,1))
      ALP(K,1)=ALOG(PMVP(K,1))
  210 CONTINUE
C
      DO 225 L=2,LMI
C
c     DO K=1,KBO
c       PMIDO(K,NB)=PT+SGML1(L)*PDTOP+SGML2(L)*PDBO(K,NB)
c     ENDDO
C
      DO K=1,KBO
        K1K=K1(K)
        K2K=K2(K)
        PMVP(K,2)=0.5*(PMIDI(K1K,L)+PMIDI(K2K,L))
        ALPR(K,L-1)=ALOG(PMVP(K,2)/PMVP(K,1))
c     if(k.eq.455)then
c       write(0,*)' l=',l,' k1=',k1k,' k2=',k2k,' alpr=',alpr(k,l-1)
c    1,           ' pmvp=',pmvp(k,1),pmvp(k,2)
c     endif 
        ALP(K,L)=ALOG(PMVP(K,2))
        PMVP(K,1)=PMVP(K,2)
      ENDDO
  225 CONTINUE
C
C----------------------------------------------------------------------
C
      DO 300 K=1,KBO
C
      ALPSFC=ALOG(PDVP(K)+PDTOP+PT)
!***
!***  INTERPOLATE THE WINDS
!***
      DO 270 L=1,LMO
C
      IF(L.EQ.1)ALPLB=ALPT
      ALPUB=ALPLB
      ALPLB=ALOG(PT+SG1(L+1)*PDTOP+SG2(L+1)*PDVP(K))
      prso=PT+SGML1(L)*PDTOP+SGML2(L)*PDVP(K)
c     if(k.eq.583)then
c       write(0,*)' l=',l,' prso=',prso,' pt=',pt
c    1,           ' sgml=',sgml1(l),sgml2(l)
c    2,           ' pdtop=',pdtop,' pdvp=',pdvp(k)
c     endif
      ALPETA=ALOG(PT+SGML1(L)*PDTOP+SGML2(L)*PDVP(K))
!
      DO 260 LD=2,LMI
      prsi=exp(alp(k,ld))
c     if(k.eq.583)then
c       write(0,*)' l=',l,' ld=',ld,' prso=',prso,' prsi=',prsi
c     endif
      IF(ALPETA.GT.ALP(K,LD).AND.LD.LT.LMI)GO TO 260
C
c     if(k.eq.583)then
c       write(0,*)' 1 l=',l,' ld=',ld,' alpeta=',alpeta
c    1,           ' alp=',alp(k,ld) 
c    2,           ' ubi=',ubi(k,ld,nb)
c     endif
      IF((ALPETA.LE.ALP(K,LD).OR.(ALPETA.GT.ALP(K,LD)
     1   .AND.LD.EQ.LMI)).AND.ABS(UBI(K,LD,NB)).GT.0.)
     2    THEN
        ULD=UBI(K,LD,NB)
        VLD=VBI(K,LD,NB)
        LDM1=LD-1
        CF=(ALP(K,LD)-ALPETA)/ALPR(K,LDM1)
        UK=ULD+(UBI(K,LDM1,NB)-ULD)*CF
        VK=VLD+(VBI(K,LDM1,NB)-VLD)*CF
c     if(k.eq.583)then
c       write(0,*)' 1a l=',l,' ldm1=',ldm1,' vbo=',vk
c       write(0,*)' vld=',vld,' vbi=',vbi(k,ldm1,nb)
c    1,           ' cf=',cf
c       write(0,*)' alpr=',alpr(k,ldm1)
c    1,           ' alp=',alp(k,ld),' alpeta=',alpeta
c     endif
        UTOP=UK
        VTOP=VK
        ALPTU=ALPETA
        LTOP=L
        UBO(K,L,NB)=UK
        VBO(K,L,NB)=VK
        GO TO 270
C***
C***  IF THE NEW SIGMA GROUND SURFACE IS BELOW THE OLD GROUND, THEN
C***  ASSUME THE WIND COMPONENTS DECREASE TO ZERO AT THE SURFACE.
C***  THE SAME IS TRUE FOR ANY NEW ETA LEVEL BELOW THE LOWEST
C***  OLD ETA SURFACE.
C***  
      ELSEIF(UBI(K,LD,NB).EQ.0.)THEN
        UDIF=UTOP
        VDIF=VTOP
        ALPDIF=ALPTU-ALPSFC
        LT1=LTOP+1
C
        DO LL=LT1,LMO
          IF(LL.EQ.LT1)ALPLB=ALOG(PT+SG1(LL)*PDTOP+SG2(LL)*PDVP(K))
          ALPUB=ALPLB
          ALPLB=ALOG(PT+SG1(LL+1)*PDTOP+SG2(LL+1)*PDVP(K))
          ALPETA=ALOG(PT+SGML1(LL)*PDTOP+SGML2(LL)*PDVP(K))
          CF=(ALPETA-ALPTU)
     1      /(ALPTU-ALOG(PT+SGML1(LTOP-1)*PDTOP
     2       +SGML2(LTOP-1)*PDVP(K)))
          UBO(K,LL,NB)=UTOP-UDIF*(ALPTU-ALPETA)/ALPDIF
          VBO(K,LL,NB)=VTOP-VDIF*(ALPTU-ALPETA)/ALPDIF
c     if(k.eq.583)then
c       write(0,*)' 2 l=',l,' ld=',ld,' ll=',ll,' vbo=',vbo(k,ll,nb)
c       write(0,*)' vtop=',vtop,' vdif=',vdif,' alptu=',alptu
c    1,           ' alpeta=',alpeta,' alpdif=',alpdif
c     endif
        ENDDO
C
        GO TO 300
      ENDIF
C
  260 CONTINUE
C
  270 CONTINUE
C
  300 CONTINUE
C
      DO L=1,LMO  
      DO K=1,KBO
        IF(ABS(UBO(K,L,NB)).GT.120.)print*,'enormal u',k,l,nb
     1                                    ,ubo(k,l,nb)
      ENDDO 
      ENDDO 
C
      DO L=1,LMO  
      DO K=1,KBO
        IF(ABS(VBO(K,L,NB)).GT.120.)print*,'enormal v',k,l,nb
     1                                    ,vbo(k,l,nb)
      ENDDO
      ENDDO
C-------------------------------------------------------------------
C OBTAIN TEMPERATURE AT MID-SIGMA LAYER BASED ON HYDROSTATIC 
C RELATIONSHIP FROM BOTTOM UP
C THE LOWEST LAYER TEMP IS EXTRAPOLATED FROM TWO LAYER ABOVE
C FOR NOW DUE TO INCONSISTANT LOWER LEVEL HEIGHT
C-------------------------------------------------------------------
C
      DO K=1,KBO
        PHUB=G*ZETAO(K,LMO)
        DO L=LMO-1,1,-1
        PHLB=PHUB
        PHUB=G*ZETAO(K,L)
        TBO(K,L,NB)=-(PHLB-PHUB)
     1              *(PT+SGML1(L)*PDTOP+SGML2(L)*PDBO(K,NB))
     2              /(RD*(DSG1(L)*PDTOP+DSG2(L)*PDBO(K,NB)))
c     if(k.eq.455.and.l.ge.lmo-5)then
c       write(0,*)' l=',l,' tbo=',tbo(k,l,nb),' pt=',pt
c       write(0,*)' phlb=',phlb,' phub=',phub
c    1,           ' pdtop=',pdtop,' pdbo=',pdbo(k,nb)
c       write(0,*)' sgml1=',sgml1(l),' sgml2=',sgml2(l)
c    1,           ' dsg1=',dsg1(l),' dsg2=',dsg2(l)
c     endif
        IF(TBO(K,L,NB).LT.150.)TBO(K,L,NB)=150.
        ENDDO
      ENDDO
C
C***  THE FOLLOWING LINES OBTAIN TEMP(LMO) USING TWO LEVELS ABOVE
C
      DO K=1,KBO
        APEL1=LOG(PT+SGML1(LMO-5)*PDTOP+SGML2(LMO-5)*PDBO(K,NB))
        APEL2=LOG(PT+SGML1(LMO-1)*PDTOP+SGML2(LMO-1)*PDBO(K,NB))
        APEL3=LOG(PT+SGML1(LMO)*PDTOP+SGML2(LMO)*PDBO(K,NB))
        TBO(K,LMO,NB)=TBO(K,LMO-1,NB)
     1              +(TBO(K,LMO-1,NB)-TBO(K,LMO-5,NB))
     2              *(APEL3-APEL2)/(APEL2-APEL1)	
c     if(k.eq.455)then
c       write(0,*)' lmo=',lmo,' tbo=',tbo(k,lmo,nb)
c    1,           ' tbo-=',tbo(k,lmo-1,nb),tbo(k,lmo-5,nb)
c       write(0,*)' apel3=',apel3,' apel2=',apel2,' apel1=',apel1
c     endif
      ENDDO
C
      DO L=1,LMO
      DO K=1,KBO
        IF(TBO(K,L,NB).GT.325.0.OR.TBO(K,L,NB).LT.150.)print*,
     +   'enormal temperature',k,l,nb,tbo(k,l,nb)
      ENDDO
      ENDDO 		 
C
C***  IF SIGMA GROUND FALLS UNDER ETA GROUND, EXTRAPOLATE TO
C
      DO 325 K=1,KBO
      PIN=PT+ETAI(LMHOI(K)-1)*PDSLI(K)
C
      DO L=1,LMO
        PSIGMA=PT+SG1(L)*PDTOP+SG2(L)*PDBO(K,NB)
        IF(PSIGMA.GT.PIN)THEN 
          DO LL=L-1,LMO
            APEL3=LOG(PT+SGML1(LL-5)*PDTOP+SGML2(LL-5)*PDBO(K,NB))
            APEL2=LOG(PT+SGML1(LL-1)*PDTOP+SGML2(LL-1)*PDBO(K,NB))
            APEL1=LOG(PT+SGML1(LL)*PDTOP+SGML2(LL)*PDBO(K,NB))
            TBO(K,LL,NB)=TBO(K,LL-1,NB)
     1                 +(TBO(K,LL-1,NB)-TBO(K,LL-5,NB))
     2                 *(APEL1-APEL2)/(APEL2-APEL3)
c     if(k.eq.455.and.ll.ge.lmo-5)then
c       write(0,*)' ll=',ll,' tbo=',tbo(k,ll,nb)
c    1,           ' tbo-=',tbo(k,ll-1,nb),tbo(k,ll-5,nb)
c       write(0,*)' apel3=',apel3,' apel2=',apel2,' apel1=',apel1
c     endif
          ENDDO
	  GO TO 325
	ENDIF 	
      ENDDO
  325 CONTINUE	    
C
C***  CHECK ON ENORMAL LAPSE RATE
C
      DO L=1,LMO-1
      DO K=1,KBO
        IF(L.GT.30.AND.ABS(TBO(K,L,NB)-TBO(K,L+1,NB)).GT.5.0)THEN
          print*,'LARGE LAPSE RATE AT ',K,L,NB,'DT= '
     1,           TBO(K,L,NB)-TBO(K,L+1,NB)
        ENDIF
      ENDDO
      ENDDO 		 
C***
C***  INTERPOLATE Q AND CLD WATER LINEARLY IN LN(P)
C***
      DO 355 K=1,KBO
C
      DO LD=1,LMIM
        ALPR(K,LD)=ALOG(PMIDI(K,LD+1)/PMIDI(K,LD))
      ENDDO
C
      DO LD=1,LMI
        ALP(K,LD)=ALOG(PMIDI(K,LD))
      ENDDO
C
      LKO=LMO
      ALPLB=ALPT
C
      DO 345 L=1,LKO
      ALPUB=ALPLB
      ALPLB=ALOG(PT+SG1(L+1)*PDTOP+SG2(L+1)*PDBO(K,NB))
      ALPETA=ALOG(PT+SGML1(L)*PDTOP+SGML2(L)*PDBO(K,NB))
C
      DO 335 LD=2,LMI
C
      IF(HTMBI(K,LD,NB).GT.0.5.AND.(ALPETA.LE.ALP(K,LD).OR.
     1                   (ALPETA.GT.ALP(K,LD).AND.LD.EQ.LMI)))THEN
        CWMLD=CWMBI(K,LD,NB)
        LDM1=LD-1
        CF=(ALP(K,LD)-ALPETA)/ALPR(K,LDM1)
        CWMBO(K,L,NB)=CWMLD+(CWMBI(K,LDM1,NB)-CWMLD)*CF
        CWMBO(K,L,NB)=AMAX1(CWMBO(K,L,NB),0.)
        LKI=LD
        GO TO 345
C***
C***  IF THE NEW SIGMA GROUND SURFACE IS BELOW THE OLD GROUND THEN
C***  ASSUME THE RH (GIVEN T) IS EQUAL TO THAT ON THE LOWEST
C***  OLD SURFACE AND THAT THE CLOUD WATER IS ZERO.
C***
C*** IF ALL NEIGHBORING POINTS (BRFORE HORIZONTAL INTERP)FOR THE 
C*** OUTPUT GRID ARE UNDERGROUND THEN THE INPUT QI IS ZERO 
C
      ELSEIF(HTMBI(K,LD,NB).LT.0.5)THEN
        CWMBO(K,L,NB)=0.
        GO TO 345
      ENDIF
  335 CONTINUE
  345 CONTINUE
C
  355 CONTINUE
C  
      DO L=1,LMO
      DO K=1,KBO
        IF(CWMBO(K,L,NB).GT.0.01)print*,'enormal cwmbo',k,l,nb
     1                                 ,cwmbo(k,l,nb)
      ENDDO
      ENDDO
C            
      DO 350 K=1,KBO
C
      DO LD=1,LMIM
        ALPR(K,LD)=ALOG(PMIDI(K,LD+1)/PMIDI(K,LD))
      ENDDO
C
      DO LD=1,LMI
        ALP(K,LD)=ALOG(PMIDI(K,LD))
      ENDDO
C
      LKO=LMO
      ALPLB=ALPT
C
      DO 340 L=1,LKO
      ALPUB=ALPLB
      ALPLB=LOG(PT+SG1(L+1)*PDTOP+SG2(L+1)*PDBO(K,NB))
      ALPETA=LOG(PT+SGML1(L)*PDTOP+SGML2(L)*PDBO(K,NB))
ccc   PMIDO(K,NB)=PT+SGML1(L)*PDTOP+SGML2(L)*PDBO(K,NB)
      PMIDO=PT+SGML1(L)*PDTOP+SGML2(L)*PDBO(K,NB)
C
      DO 330 LD=2,LMI
C
      IF(HTMBI(K,LD,NB).GT.0.5.AND.(ALPETA.LE.ALP(K,LD).OR.
     1                   (ALPETA.GT.ALP(K,LD).AND.LD.EQ.LMI)))THEN
        QLD=QBI(K,LD,NB)
        LDM1=LD-1
        CF=(ALP(K,LD)-ALPETA)/ALPR(K,LDM1)
        QETAIJ=QLD+(QBI(K,LDM1,NB)-QLD)*CF
        CLOGES=-CM1/TBO(K,L,NB)-CM2*ALOG10(TBO(K,L,NB))+CM3
        ESE=10.**(CLOGES+2.)
ccc     QSX=EPS*ESE/(PMIDO(K,NB)-ESE*(1.-EPS))
        QSX=EPS*ESE/(PMIDO-ESE*(1.-EPS))
        QSMX=0.98*QSX
        QBO(K,L,NB)=AMIN1(QETAIJ,QSMX)
        QBO(K,L,NB)=AMAX1(QBO(K,L,NB),EPSQ)
C       IF(QBO(K,L,NB).LT.QBO(K,L-1,NB).AND.LD.EQ.LMHO(K))THEN
C         QBO(K,L,NB)=QBO(K,L-1,NB)
C       ENDIF
        LKI=LD
        GO TO 340
C***
C***  IF THE NEW SIGMA GROUND SURFACE IS BELOW THE OLD GROUND THEN
C***  ASSUME THE RH (GIVEN T) IS EQUAL TO THAT ON THE LOWEST
C***  OLD SURFACE AND THAT THE CLOUD WATER IS ZERO.
C***
C***  IF ALL NEIGHBORING POINTS (BRFORE HORIZONTAL INTERP)FOR THE 
C***  OUTPUT GRID ARE UNDERGROUND THEN THE INPUT QI IS ZERO 
C
      ELSEIF(HTMBI(K,LD,NB).LT.0.5)THEN
        IF(TBI(K,LKI,NB).LT.1.E-2)STOP666
        CLOGES=-CM1/TBI(K,LKI,NB)-CM2*ALOG10(TBI(K,LKI,NB))+CM3
        ESS=10.**(CLOGES+2.)
        QSLDM=EPS*ESS/(PMIDI(K,LKI)-ESS*(1.-EPS))
        RHLDM=QBI(K,LKI,NB)/QSLDM
        RHLDM=AMIN1(RHLDM,0.98)
        CLOGES=-CM1/TBO(K,L,NB)-CM2*ALOG10(TBO(K,L,NB))+CM3
        ESL=10.**(CLOGES+2.)
        QSL=EPS*ESL/(PT+SGML1(L)*PDTOP+SGML2(L)*PDBO(K,NB)
     1                 -ESL*(1.-EPS))
        QBO(K,L,NB)=RHLDM*QSL
        GO TO 340
      ENDIF
  330 CONTINUE
  340 CONTINUE
C
      DO L=1,LMO
        IF(QBO(K,L,NB).LT.0.)QBO(K,L,NB)=0.
        IF(QBO(K,L,NB).GT.25.E-3)QBO(K,L,NB)=25.E-3
      ENDDO   
C
  350 CONTINUE
C***
C***  CONVERT FROM VIRTUAL TO TRUE TEMPERATURE.
C***
      DO 375 L=1,LMO
      DO 375 K=1,KBO
      IF(TBO(K,L,NB).GT.325.)THEN
        print*,'TEMP > 325',K,L,NB,TBO(K,L,NB)
        TBO(K,L,NB)=325.
      ENDIF 
C
      IF(TBO(K,L,NB).LT.150.)TBO(K,L,NB)=150.
      TV=TBO(K,L,NB)
C
      DO ITER=1,3
        TBO(K,L,NB)=TV/(1.+0.608*QBO(K,L,NB))
c     if(k.eq.455.and.l.ge.lmo-5)then
c       write(0,*)' iter=',iter,' l=',l,' tbo=',tbo(k,l,nb)
c       write(0,*)' qbo=',qbo(k,l,nb),' tv=',tv
c     endif
        CLOGES=-CM1/TBO(K,L,NB)-CM2*ALOG10(TBO(K,L,NB))+CM3
        ESE=10.**(CLOGES+2.)
        DENOM2=PT+SGML1(L)*PDTOP+SGML2(L)*PDBO(K,NB)-ESE*(1.-EPS)
        QSX=EPS*ESE/DENOM2
        QSMX=0.98*QSX
        IF(QBO(K,L,NB).LT.QSMX)GO TO 375
        QBO(K,L,NB)=QSMX
      ENDDO
  375 CONTINUE
C
      DO L=1,LMO
      DO K=1,KBO
        IF(TBO(K,L,NB).GT.325.0.OR.TBO(K,L,NB).LT.150.)print*,
     1      'enormal temperature',k,l,nb,tbo(k,l,nb)
        IF(QBO(K,L,NB).GT.0.1)print*,'enormal q',k,l,nb
     1     ,qbo(k,l,nb)
      ENDDO
      ENDDO
C--------------------------------------------------------------------
C***
C***  INTERPOLATE Q2 LINEARLY IN LN(P) (BUT ON INTERFACES)
C***
      DO 450 K=1,KBO
C
      DO LD=1,LMI
        ALPR(K,LD)=ALOG(PINTI(K,LD+1)/PINTI(K,LD))
      ENDDO
C
      DO LD=1,LMIP
        ALP(K,LD)=ALOG(PINTI(K,LD))
      ENDDO
C
      LKO=LMO-1
      Q2SFC=AMAX1(B1**(2./3.)*USTARBI(K,NB)*USTARBI(K,NB),EPSQ2)
      Q2BO(K,LKO+1,NB)=Q2SFC
      ALPSFC=ALOG(PDBO(K,NB)+PDTOP+PT)
C
      DO 440 L=1,LKO
      ALPETA=ALOG(PT+SG1(L+1)*PDTOP+SG2(L+1)*PDBO(K,NB))
C
      DO 430 LD=2,LMI
C
      IF(HTMBI(K,LD,NB).GT.0.5.AND.(ALPETA.LE.ALP(K,LD).OR.
     1                           ALPETA.GT.ALP(K,LMI)))THEN
        Q2LD=Q2BI(K,LD,NB)
        LDM1=LD-1
        CF=(ALP(K,LD)-ALPETA)/ALPR(K,LDM1)
        Q2IJ=Q2LD+(Q2BI(K,LDM1,NB)-Q2LD)*CF
        Q2TOP=AMAX1(Q2IJ,EPSQ2)
        ALPT=ALPETA
        LTOP=L
        Q2BO(K,L,NB)=Q2TOP
        GO TO 440
C***
C***  IF THE NEW ETA GROUND SURFACE IS BELOW THE OLD GROUND THEN
C***  ASSUME Q2 DECREASES LINEARLY TO Q2 AT THE NEW GROUND.
C***
      ELSEIF(HTMBI(K,LD,NB).LT.0.5)THEN
        Q2DIF=Q2TOP-Q2SFC
        ALPDIF=ALPT-ALPSFC
        LT1=LTOP+1
        DO LL=LT1,LKO
          ALPETA=LOG(PT+SG1(LL)*PDTOP+SG2(LL)*PDBO(K,NB))
          Q2BO(K,LL,NB)=Q2TOP-Q2DIF*(ALPT-ALPETA)/ALPDIF
          Q2BO(K,LL,NB)=MAX(Q2BO(K,LL,NB),EPSQ2)
        ENDDO
        GO TO 450
      ENDIF
  430 CONTINUE
  440 CONTINUE
C
  450 CONTINUE
C-----------------------------------------------------------
C***
C***  NOW REDEFINE PD TO HAVE IT EQUAL TO PSFC-PT.
C***
      ix=584
c     write(0,*)' exit pdbo=',pdbo(ix,nb),' reso=',reso(ix)
c    1,         ' lmhoi=',lmhoi(ix)
      DO K=1,KBO
        PDBO(K,NB)=PDBO(K,NB)*RESO(K)
      ENDDO
c     do l=1,lmo
c       write(0,*)' l=',l,' x=',tbo(ix,l,nb),qbo(ix,l,nb)
c    1,           q2bo(ix,l,nb),cwmbo(ix,l,nb)
c     enddo
c     do l=1,lmo
c       write(0,*)' l=',l,' u=',ubo(ix,l,nb),' v=',vbo(ix,l,nb)
c     enddo
C----------------------------------------------------------------
 1000 CONTINUE
C----------------------------------------------------------------
      RETURN
      END
